@@ -3,12 +3,12 @@
  * keywords.c
  *	  lexical token lookup for reserved words in PostgreSQL
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/parser/keywords.c,v 1.127 2002/09/18 21:35:22 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/parser/keywords.c,v 1.154 2004/12/31 22:00:27 pgsql Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -30,19 +30,21 @@
  */
 static const ScanKeyword ScanKeywords[] = {
 	/* name, value */
-	{"abort", ABORT_TRANS},
-	{"absolute", ABSOLUTE},
+	{"abort", ABORT_P},
+	{"absolute", ABSOLUTE_P},
 	{"access", ACCESS},
 	{"action", ACTION},
 	{"add", ADD},
 	{"after", AFTER},
 	{"aggregate", AGGREGATE},
 	{"all", ALL},
+	{"also", ALSO},
 	{"alter", ALTER},
 	{"analyse", ANALYSE},		/* British spelling */
 	{"analyze", ANALYZE},
 	{"and", AND},
 	{"any", ANY},
+	{"array", ARRAY},
 	{"as", AS},
 	{"asc", ASC},
 	{"assertion", ASSERTION},
@@ -51,12 +53,12 @@ static const ScanKeyword ScanKeywords[] = {
 	{"authorization", AUTHORIZATION},
 	{"backward", BACKWARD},
 	{"before", BEFORE},
-	{"begin", BEGIN_TRANS},
+	{"begin", BEGIN_P},
 	{"between", BETWEEN},
 	{"bigint", BIGINT},
 	{"binary", BINARY},
 	{"bit", BIT},
-	{"boolean", BOOLEAN},
+	{"boolean", BOOLEAN_P},
 	{"both", BOTH},
 	{"by", BY},
 	{"cache", CACHE},
@@ -88,6 +90,7 @@ static const ScanKeyword ScanKeywords[] = {
 	{"createdb", CREATEDB},
 	{"createuser", CREATEUSER},
 	{"cross", CROSS},
+	{"csv", CSV},
 	{"current_date", CURRENT_DATE},
 	{"current_time", CURRENT_TIME},
 	{"current_timestamp", CURRENT_TIMESTAMP},
@@ -98,9 +101,10 @@ static const ScanKeyword ScanKeywords[] = {
 	{"day", DAY_P},
 	{"deallocate", DEALLOCATE},
 	{"dec", DEC},
-	{"decimal", DECIMAL},
+	{"decimal", DECIMAL_P},
 	{"declare", DECLARE},
 	{"default", DEFAULT},
+	{"defaults", DEFAULTS},
 	{"deferrable", DEFERRABLE},
 	{"deferred", DEFERRED},
 	{"definer", DEFINER},
@@ -111,15 +115,16 @@ static const ScanKeyword ScanKeywords[] = {
 	{"distinct", DISTINCT},
 	{"do", DO},
 	{"domain", DOMAIN_P},
-	{"double", DOUBLE},
+	{"double", DOUBLE_P},
 	{"drop", DROP},
 	{"each", EACH},
 	{"else", ELSE},
 	{"encoding", ENCODING},
 	{"encrypted", ENCRYPTED},
-	{"end", END_TRANS},
+	{"end", END_P},
 	{"escape", ESCAPE},
 	{"except", EXCEPT},
+	{"excluding", EXCLUDING},
 	{"exclusive", EXCLUSIVE},
 	{"execute", EXECUTE},
 	{"exists", EXISTS},
@@ -128,6 +133,7 @@ static const ScanKeyword ScanKeywords[] = {
 	{"extract", EXTRACT},
 	{"false", FALSE_P},
 	{"fetch", FETCH},
+	{"first", FIRST_P},
 	{"float", FLOAT_P},
 	{"for", FOR},
 	{"force", FORCE},
@@ -137,29 +143,30 @@ static const ScanKeyword ScanKeywords[] = {
 	{"from", FROM},
 	{"full", FULL},
 	{"function", FUNCTION},
-	{"get", GET},
 	{"global", GLOBAL},
 	{"grant", GRANT},
 	{"group", GROUP_P},
 	{"handler", HANDLER},
 	{"having", HAVING},
+	{"hold", HOLD},
 	{"hour", HOUR_P},
 	{"ilike", ILIKE},
 	{"immediate", IMMEDIATE},
 	{"immutable", IMMUTABLE},
 	{"implicit", IMPLICIT_P},
 	{"in", IN_P},
+	{"including", INCLUDING},
 	{"increment", INCREMENT},
 	{"index", INDEX},
 	{"inherits", INHERITS},
 	{"initially", INITIALLY},
 	{"inner", INNER_P},
 	{"inout", INOUT},
-	{"input", INPUT},
+	{"input", INPUT_P},
 	{"insensitive", INSENSITIVE},
 	{"insert", INSERT},
 	{"instead", INSTEAD},
-	{"int", INT},
+	{"int", INT_P},
 	{"integer", INTEGER},
 	{"intersect", INTERSECT},
 	{"interval", INTERVAL},
@@ -172,6 +179,8 @@ static const ScanKeyword ScanKeywords[] = {
 	{"key", KEY},
 	{"lancompiler", LANCOMPILER},
 	{"language", LANGUAGE},
+	{"large", LARGE_P},
+	{"last", LAST_P},
 	{"leading", LEADING},
 	{"left", LEFT},
 	{"level", LEVEL},
@@ -205,9 +214,11 @@ static const ScanKeyword ScanKeywords[] = {
 	{"nothing", NOTHING},
 	{"notify", NOTIFY},
 	{"notnull", NOTNULL},
+	{"nowait", NOWAIT},
 	{"null", NULL_P},
 	{"nullif", NULLIF},
 	{"numeric", NUMERIC},
+	{"object", OBJECT_P},
 	{"of", OF},
 	{"off", OFF},
 	{"offset", OFFSET},
@@ -226,34 +237,38 @@ static const ScanKeyword ScanKeywords[] = {
 	{"owner", OWNER},
 	{"partial", PARTIAL},
 	{"password", PASSWORD},
-	{"path", PATH_P},
-	{"pendant", PENDANT},
 	{"placing", PLACING},
 	{"position", POSITION},
 	{"precision", PRECISION},
 	{"prepare", PREPARE},
+	{"preserve", PRESERVE},
 	{"primary", PRIMARY},
 	{"prior", PRIOR},
 	{"privileges", PRIVILEGES},
 	{"procedural", PROCEDURAL},
 	{"procedure", PROCEDURE},
+	{"quote", QUOTE},
 	{"read", READ},
 	{"real", REAL},
 	{"recheck", RECHECK},
 	{"references", REFERENCES},
 	{"reindex", REINDEX},
-	{"relative", RELATIVE},
+	{"relative", RELATIVE_P},
+	{"release", RELEASE},
 	{"rename", RENAME},
+	{"repeatable", REPEATABLE},
 	{"replace", REPLACE},
-	{"replication", REPLICATION},
 	{"reset", RESET},
+	{"restart", RESTART},
 	{"restrict", RESTRICT},
 	{"returns", RETURNS},
 	{"revoke", REVOKE},
 	{"right", RIGHT},
 	{"rollback", ROLLBACK},
 	{"row", ROW},
+	{"rows", ROWS},
 	{"rule", RULE},
+	{"savepoint", SAVEPOINT},
 	{"schema", SCHEMA},
 	{"scroll", SCROLL},
 	{"second", SECOND_P},
@@ -261,7 +276,6 @@ static const ScanKeyword ScanKeywords[] = {
 	{"select", SELECT},
 	{"sequence", SEQUENCE},
 	{"serializable", SERIALIZABLE},
-	{"server", SERVER},
 	{"session", SESSION},
 	{"session_user", SESSION_USER},
 	{"set", SET},
@@ -279,10 +293,11 @@ static const ScanKeyword ScanKeywords[] = {
 	{"stdin", STDIN},
 	{"stdout", STDOUT},
 	{"storage", STORAGE},
-	{"strict", STRICT},
+	{"strict", STRICT_P},
 	{"substring", SUBSTRING},
 	{"sysid", SYSID},
 	{"table", TABLE},
+	{"tablespace", TABLESPACE},
 	{"temp", TEMP},
 	{"template", TEMPLATE},
 	{"temporary", TEMPORARY},
@@ -300,6 +315,7 @@ static const ScanKeyword ScanKeywords[] = {
 	{"truncate", TRUNCATE},
 	{"trusted", TRUSTED},
 	{"type", TYPE_P},
+	{"uncommitted", UNCOMMITTED},
 	{"unencrypted", UNENCRYPTED},
 	{"union", UNION},
 	{"unique", UNIQUE},
@@ -317,7 +333,6 @@ static const ScanKeyword ScanKeywords[] = {
 	{"varchar", VARCHAR},
 	{"varying", VARYING},
 	{"verbose", VERBOSE},
-	{"version", VERSION},
 	{"view", VIEW},
 	{"volatile", VOLATILE},
 	{"when", WHEN},
@@ -358,17 +373,13 @@ ScanKeywordLookup(const char *text)
 
 	/*
 	 * Apply an ASCII-only downcasing.	We must not use tolower() since it
-	 * may produce the wrong translation in some locales (eg, Turkish),
-	 * and we don't trust isupper() very much either.  In an ASCII-based
-	 * encoding the tests against A and Z are sufficient, but we also
-	 * check isupper() so that we will work correctly under EBCDIC.  The
-	 * actual case conversion step should work for either ASCII or EBCDIC.
+	 * may produce the wrong translation in some locales (eg, Turkish).
 	 */
 	for (i = 0; i < len; i++)
 	{
 		char		ch = text[i];
 
-		if (ch >= 'A' && ch <= 'Z' && isupper((unsigned char) ch))
+		if (ch >= 'A' && ch <= 'Z')
 			ch += 'a' - 'A';
 		word[i] = ch;
 	}
