@@ -5,12 +5,12 @@
  *	  commands.  At one time acted as an interface between the Lisp and C
  *	  systems.
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/tcop/utility.c,v 1.208.2.1 2005/01/24 17:46:41 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/tcop/utility.c,v 1.231.4.1 2005/01/24 17:46:29 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -36,6 +36,7 @@
 #include "commands/schemacmds.h"
 #include "commands/sequence.h"
 #include "commands/tablecmds.h"
+#include "commands/tablespace.h"
 #include "commands/trigger.h"
 #include "commands/typecmds.h"
 #include "commands/user.h"
@@ -43,18 +44,18 @@
 #include "commands/view.h"
 #include "miscadmin.h"
 #include "nodes/makefuncs.h"
-#include "parser/parse_clause.h"
 #include "parser/parse_expr.h"
 #include "parser/parse_type.h"
+#include "postmaster/bgwriter.h"
 #include "rewrite/rewriteDefine.h"
 #include "rewrite/rewriteRemove.h"
+#include "storage/fd.h"
 #include "tcop/pquery.h"
 #include "tcop/utility.h"
 #include "utils/acl.h"
 #include "utils/guc.h"
 #include "utils/lsyscache.h"
 #include "utils/syscache.h"
-#include "access/xlog.h"
 
 #ifdef USE_REPLICATION
 #include "replicate.h"
@@ -103,8 +104,12 @@ static const struct msgstrings msgstringarray[] = {
 };
 
 
+/*
+ * Emit the right error message for a "DROP" command issued on a
+ * relation of the wrong type
+ */
 static void
-DropErrorMsg(char *relname, char wrongkind, char rightkind)
+DropErrorMsgWrongType(char *relname, char wrongkind, char rightkind)
 {
 	const struct msgstrings *rentry;
 	const struct msgstrings *wentry;
@@ -125,24 +130,37 @@ DropErrorMsg(char *relname, char wrongkind, char rightkind)
 			 (wentry->kind != '\0') ? errhint(wentry->drophint_msg) : 0));
 }
 
+/*
+ * Emit the right error message for a "DROP" command issued on a
+ * non-existent relation
+ */
+static void
+DropErrorMsgNonExistent(RangeVar *rel, char rightkind)
+{
+	const struct msgstrings *rentry;
+
+	for (rentry = msgstringarray; rentry->kind != '\0'; rentry++)
+	{
+		if (rentry->kind == rightkind)
+			ereport(ERROR,
+					(errcode(rentry->nonexistent_code),
+					 errmsg(rentry->nonexistent_msg, rel->relname)));
+	}
+
+	Assert(false);				/* Should be impossible */
+}
+
 static void
 CheckDropPermissions(RangeVar *rel, char rightkind)
 {
-	const struct msgstrings *rentry;
 	Oid			relOid;
 	HeapTuple	tuple;
 	Form_pg_class classform;
 
-	for (rentry = msgstringarray; rentry->kind != '\0'; rentry++)
-		if (rentry->kind == rightkind)
-			break;
-	Assert(rentry->kind != '\0');
-
 	relOid = RangeVarGetRelid(rel, true);
 	if (!OidIsValid(relOid))
-		ereport(ERROR,
-				(errcode(rentry->nonexistent_code),
-				 errmsg(rentry->nonexistent_msg, rel->relname)));
+		DropErrorMsgNonExistent(rel, rightkind);
+
 	tuple = SearchSysCache(RELOID,
 						   ObjectIdGetDatum(relOid),
 						   0, 0, 0);
@@ -152,7 +170,8 @@ CheckDropPermissions(RangeVar *rel, char rightkind)
 	classform = (Form_pg_class) GETSTRUCT(tuple);
 
 	if (classform->relkind != rightkind)
-		DropErrorMsg(rel->relname, classform->relkind, rightkind);
+		DropErrorMsgWrongType(rel->relname, classform->relkind,
+							  rightkind);
 
 	/* Allow DROP to either table owner or schema owner */
 	if (!pg_class_ownercheck(relOid, GetUserId()) &&
@@ -206,6 +225,46 @@ CheckRelationOwnership(RangeVar *rel, bool noCatalogs)
 }
 
 
+/*
+ * QueryIsReadOnly: is an analyzed/rewritten query read-only?
+ *
+ * This is a much stricter test than we apply for XactReadOnly mode;
+ * the query must be *in truth* read-only, because the caller wishes
+ * not to do CommandCounterIncrement for it.
+ */
+bool
+QueryIsReadOnly(Query *parsetree)
+{
+	switch (parsetree->commandType)
+	{
+		case CMD_SELECT:
+			if (parsetree->into != NULL)
+				return false;					/* SELECT INTO */
+			else if (parsetree->rowMarks != NIL)
+				return false;					/* SELECT FOR UPDATE */
+			else
+				return true;
+		case CMD_UPDATE:
+		case CMD_INSERT:
+		case CMD_DELETE:
+			return false;
+		case CMD_UTILITY:
+			/* For now, treat all utility commands as read/write */
+			return false;
+		default:
+			elog(WARNING, "unrecognized commandType: %d",
+				 (int) parsetree->commandType);
+			break;
+	}
+	return false;
+}
+
+/*
+ * check_xact_readonly: is a utility command read-only?
+ *
+ * Here we use the loose rules of XactReadOnly mode: no permanent effects
+ * on the database are allowed.
+ */
 static void
 check_xact_readonly(Node *parsetree)
 {
@@ -222,11 +281,12 @@ check_xact_readonly(Node *parsetree)
 		case T_AlterDatabaseSetStmt:
 		case T_AlterDomainStmt:
 		case T_AlterGroupStmt:
+		case T_AlterOwnerStmt:
 		case T_AlterSeqStmt:
 		case T_AlterTableStmt:
-		case T_RenameStmt:
 		case T_AlterUserStmt:
 		case T_AlterUserSetStmt:
+		case T_RenameStmt:
 		case T_CommentStmt:
 		case T_DefineStmt:
 		case T_CreateCastStmt:
@@ -242,6 +302,7 @@ check_xact_readonly(Node *parsetree)
 		case T_CreateSchemaStmt:
 		case T_CreateSeqStmt:
 		case T_CreateStmt:
+		case T_CreateTableSpaceStmt:
 		case T_CreateTrigStmt:
 		case T_CompositeTypeStmt:
 		case T_CreateUserStmt:
@@ -250,6 +311,7 @@ check_xact_readonly(Node *parsetree)
 		case T_DropCastStmt:
 		case T_DropStmt:
 		case T_DropdbStmt:
+		case T_DropTableSpaceStmt:
 		case T_RemoveFuncStmt:
 		case T_DropGroupStmt:
 		case T_DropPLangStmt:
@@ -275,17 +337,18 @@ check_xact_readonly(Node *parsetree)
  *		general utility function invoker
  *
  *	parsetree: the parse tree for the utility statement
+ *	params: parameters to use during execution (currently only used by DECLARE)
  *	dest: where to send results
  *	completionTag: points to a buffer of size COMPLETION_TAG_BUFSIZE
  *		in which to store a command completion status string.
  *
- * completionTag is only set nonempty if we want to return a nondefault
- * status (currently, only used for MOVE/FETCH).
+ * completionTag is only set nonempty if we want to return a nondefault status.
  *
  * completionTag may be NULL if caller doesn't want a status string.
  */
 void
 ProcessUtility(Node *parsetree,
+			   ParamListInfo params,
 			   DestReceiver *dest,
 			   char *completionTag)
 {
@@ -305,45 +368,80 @@ ProcessUtility(Node *parsetree,
 
 				switch (stmt->kind)
 				{
-					case TRANS_STMT_BEGIN:
-						BeginTransactionBlock();
-						break;
-
 						/*
 						 * START TRANSACTION, as defined by SQL99:
-						 * Identical to BEGIN, except that it takes a few
-						 * additional options.
+						 * Identical to BEGIN.	Same code for both.
 						 */
+					case TRANS_STMT_BEGIN:
 					case TRANS_STMT_START:
 						{
+							ListCell   *lc;
+
 							BeginTransactionBlock();
-
-							if (stmt->options)
+							foreach(lc, stmt->options)
 							{
-								List	   *head;
+								DefElem    *item = (DefElem *) lfirst(lc);
 
-								foreach(head, stmt->options)
-								{
-									DefElem    *item = (DefElem *) lfirst(head);
-
-									if (strcmp(item->defname, "transaction_isolation") == 0)
-										SetPGVariable("transaction_isolation",
-											makeList1(item->arg), false);
-									else if (strcmp(item->defname, "transaction_read_only") == 0)
-										SetPGVariable("transaction_read_only",
-											makeList1(item->arg), false);
-								}
+								if (strcmp(item->defname, "transaction_isolation") == 0)
+									SetPGVariable("transaction_isolation",
+												  list_make1(item->arg),
+												  false);
+								else if (strcmp(item->defname, "transaction_read_only") == 0)
+									SetPGVariable("transaction_read_only",
+												  list_make1(item->arg),
+												  false);
 							}
 						}
 						break;
 
 					case TRANS_STMT_COMMIT:
-						EndTransactionBlock();
+						if (!EndTransactionBlock())
+						{
+							/* report unsuccessful commit in completionTag */
+							if (completionTag)
+								strcpy(completionTag, "ROLLBACK");
+						}
 						break;
 
 					case TRANS_STMT_ROLLBACK:
 						UserAbortTransactionBlock();
 						break;
+
+					case TRANS_STMT_SAVEPOINT:
+						{
+							ListCell   *cell;
+							char	   *name = NULL;
+
+							RequireTransactionChain((void *) stmt, "SAVEPOINT");
+
+							foreach(cell, stmt->options)
+							{
+								DefElem    *elem = lfirst(cell);
+
+								if (strcmp(elem->defname, "savepoint_name") == 0)
+									name = strVal(elem->arg);
+							}
+
+							Assert(PointerIsValid(name));
+
+							DefineSavepoint(name);
+						}
+						break;
+
+					case TRANS_STMT_RELEASE:
+						RequireTransactionChain((void *) stmt, "RELEASE SAVEPOINT");
+						ReleaseSavepoint(stmt->options);
+						break;
+
+					case TRANS_STMT_ROLLBACK_TO:
+						RequireTransactionChain((void *) stmt, "ROLLBACK TO SAVEPOINT");
+						RollbackToSavepoint(stmt->options);
+
+						/*
+						 * CommitTransactionCommand is in charge of
+						 * re-defining the savepoint again
+						 */
+						break;
 				}
 			}
 			break;
@@ -352,7 +450,7 @@ ProcessUtility(Node *parsetree,
 			 * Portal (cursor) manipulation
 			 */
 		case T_DeclareCursorStmt:
-			PerformCursorOpen((DeclareCursorStmt *) parsetree);
+			PerformCursorOpen((DeclareCursorStmt *) parsetree, params);
 			break;
 
 		case T_ClosePortalStmt:
@@ -391,10 +489,18 @@ ProcessUtility(Node *parsetree,
 			}
 			break;
 
+		case T_CreateTableSpaceStmt:
+			CreateTableSpace((CreateTableSpaceStmt *) parsetree);
+			break;
+
+		case T_DropTableSpaceStmt:
+			DropTableSpace((DropTableSpaceStmt *) parsetree);
+			break;
+
 		case T_DropStmt:
 			{
 				DropStmt   *stmt = (DropStmt *) parsetree;
-				List	   *arg;
+				ListCell   *arg;
 
 				foreach(arg, stmt->objects)
 				{
@@ -489,7 +595,7 @@ ProcessUtility(Node *parsetree,
 			break;
 
 		case T_ExecuteStmt:
-			ExecuteQuery((ExecuteStmt *) parsetree, dest);
+			ExecuteQuery((ExecuteStmt *) parsetree, dest, completionTag);
 			break;
 
 		case T_DeallocateStmt:
@@ -503,125 +609,12 @@ ProcessUtility(Node *parsetree,
 			ExecRenameStmt((RenameStmt *) parsetree);
 			break;
 
-			/* various Alter Table forms */
+		case T_AlterOwnerStmt:
+			ExecAlterOwnerStmt((AlterOwnerStmt *) parsetree);
+			break;
 
 		case T_AlterTableStmt:
-			{
-				AlterTableStmt *stmt = (AlterTableStmt *) parsetree;
-				Oid			relid;
-
-				relid = RangeVarGetRelid(stmt->relation, false);
-
-				/*
-				 * Some or all of these functions are recursive to cover
-				 * inherited things, so permission checks are done there.
-				 */
-				switch (stmt->subtype)
-				{
-					case 'A':	/* ADD COLUMN */
-
-						/*
-						 * Recursively add column to table and, if
-						 * requested, to descendants
-						 */
-						AlterTableAddColumn(relid,
-							  interpretInhOption(stmt->relation->inhOpt),
-											(ColumnDef *) stmt->def);
-						break;
-					case 'T':	/* ALTER COLUMN DEFAULT */
-
-						/*
-						 * Recursively alter column default for table and,
-						 * if requested, for descendants
-						 */
-						AlterTableAlterColumnDefault(relid,
-							  interpretInhOption(stmt->relation->inhOpt),
-													 stmt->name,
-													 stmt->def);
-						break;
-					case 'N':	/* ALTER COLUMN DROP NOT NULL */
-						AlterTableAlterColumnDropNotNull(relid,
-							  interpretInhOption(stmt->relation->inhOpt),
-														 stmt->name);
-						break;
-					case 'n':	/* ALTER COLUMN SET NOT NULL */
-						AlterTableAlterColumnSetNotNull(relid,
-							  interpretInhOption(stmt->relation->inhOpt),
-														stmt->name);
-						break;
-					case 'S':	/* ALTER COLUMN STATISTICS */
-					case 'M':	/* ALTER COLUMN STORAGE */
-
-						/*
-						 * Recursively alter column statistics for table
-						 * and, if requested, for descendants
-						 */
-						AlterTableAlterColumnFlags(relid,
-							  interpretInhOption(stmt->relation->inhOpt),
-												   stmt->name,
-												   stmt->def,
-												   &(stmt->subtype));
-						break;
-					case 'D':	/* DROP COLUMN */
-
-						/*
-						 * Recursively drop column from table and, if
-						 * requested, from descendants
-						 */
-						AlterTableDropColumn(relid,
-							  interpretInhOption(stmt->relation->inhOpt),
-											 false,
-											 stmt->name,
-											 stmt->behavior);
-						break;
-					case 'C':	/* ADD CONSTRAINT */
-
-						/*
-						 * Recursively add constraint to table and, if
-						 * requested, to descendants
-						 */
-						AlterTableAddConstraint(relid,
-							  interpretInhOption(stmt->relation->inhOpt),
-												(List *) stmt->def);
-						break;
-					case 'X':	/* DROP CONSTRAINT */
-
-						/*
-						 * Recursively drop constraint from table and, if
-						 * requested, from descendants
-						 */
-						AlterTableDropConstraint(relid,
-							  interpretInhOption(stmt->relation->inhOpt),
-												 stmt->name,
-												 stmt->behavior);
-						break;
-					case 'E':	/* CREATE TOAST TABLE */
-						AlterTableCreateToastTable(relid, false);
-						break;
-					case 'U':	/* ALTER OWNER */
-						/* check that we are the superuser */
-						if (!superuser())
-							ereport(ERROR,
-								(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
-							errmsg("must be superuser to alter owner")));
-						/* get_usesysid raises an error if no such user */
-						AlterTableOwner(relid,
-										get_usesysid(stmt->name));
-						break;
-					case 'L':	/* CLUSTER ON */
-						AlterTableClusterOn(relid, stmt->name);
-						break;
-					case 'o':	/* ADD OIDS */
-						AlterTableAlterOids(relid,
-							  interpretInhOption(stmt->relation->inhOpt),
-											false);
-						break;
-					default:	/* oops */
-						elog(ERROR, "unrecognized alter table type: %d",
-							 (int) stmt->subtype);
-						break;
-				}
-			}
+			AlterTable((AlterTableStmt *) parsetree);
 			break;
 
 		case T_AlterDomainStmt:
@@ -660,16 +653,6 @@ ProcessUtility(Node *parsetree,
 												  stmt->name,
 												  stmt->behavior);
 						break;
-					case 'U':	/* OWNER TO */
-						/* check that we are the superuser */
-						if (!superuser())
-							ereport(ERROR,
-								(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
-							errmsg("must be superuser to alter owner")));
-						/* get_usesysid raises an error if no such user */
-						AlterTypeOwner(stmt->typename,
-									   get_usesysid(stmt->name));
-						break;
 					default:	/* oops */
 						elog(ERROR, "unrecognized alter domain type: %d",
 							 (int) stmt->subtype);
@@ -740,12 +723,17 @@ ProcessUtility(Node *parsetree,
 				DefineIndex(stmt->relation,		/* relation */
 							stmt->idxname,		/* index name */
 							stmt->accessMethod, /* am name */
+							stmt->tableSpace,
 							stmt->indexParams,	/* parameters */
+							(Expr *) stmt->whereClause,
+							stmt->rangetable,
 							stmt->unique,
 							stmt->primary,
 							stmt->isconstraint,
-							(Expr *) stmt->whereClause,
-							stmt->rangetable);
+							false,		/* is_alter_table */
+							true,		/* check_rights */
+							false,		/* skip_build */
+							false);		/* quiet */
 			}
 			break;
 
@@ -849,7 +837,7 @@ ProcessUtility(Node *parsetree,
 				 */
 				if (strcmp(n->name, "TRANSACTION") == 0)
 				{
-					List	   *head;
+					ListCell   *head;
 
 					foreach(head, n->args)
 					{
@@ -857,15 +845,15 @@ ProcessUtility(Node *parsetree,
 
 						if (strcmp(item->defname, "transaction_isolation") == 0)
 							SetPGVariable("transaction_isolation",
-									  makeList1(item->arg), n->is_local);
+									 list_make1(item->arg), n->is_local);
 						else if (strcmp(item->defname, "transaction_read_only") == 0)
 							SetPGVariable("transaction_read_only",
-									  makeList1(item->arg), n->is_local);
+									 list_make1(item->arg), n->is_local);
 					}
 				}
 				else if (strcmp(n->name, "SESSION CHARACTERISTICS") == 0)
 				{
-					List	   *head;
+					ListCell   *head;
 
 					foreach(head, n->args)
 					{
@@ -873,10 +861,10 @@ ProcessUtility(Node *parsetree,
 
 						if (strcmp(item->defname, "transaction_isolation") == 0)
 							SetPGVariable("default_transaction_isolation",
-									  makeList1(item->arg), n->is_local);
+									 list_make1(item->arg), n->is_local);
 						else if (strcmp(item->defname, "transaction_read_only") == 0)
 							SetPGVariable("default_transaction_read_only",
-									  makeList1(item->arg), n->is_local);
+									 list_make1(item->arg), n->is_local);
 					}
 				}
 				else
@@ -970,7 +958,7 @@ ProcessUtility(Node *parsetree,
 			break;
 
 		case T_ConstraintsSetStmt:
-			DeferredTriggerSetState((ConstraintsSetStmt *) parsetree);
+			AfterTriggerSetState((ConstraintsSetStmt *) parsetree);
 			break;
 
 		case T_CreateGroupStmt:
@@ -990,7 +978,7 @@ ProcessUtility(Node *parsetree,
 				ereport(ERROR,
 						(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
 						 errmsg("must be superuser to do CHECKPOINT")));
-			CreateCheckPoint(false, false);
+			RequestCheckpoint(true);
 			break;
 
 		case T_ReindexStmt:
@@ -1212,9 +1200,18 @@ CreateCommandTag(Node *parsetree)
 						break;
 
 					case TRANS_STMT_ROLLBACK:
+					case TRANS_STMT_ROLLBACK_TO:
 						tag = "ROLLBACK";
 						break;
 
+					case TRANS_STMT_SAVEPOINT:
+						tag = "SAVEPOINT";
+						break;
+
+					case TRANS_STMT_RELEASE:
+						tag = "RELEASE";
+						break;
+
 					default:
 						tag = "???";
 						break;
@@ -1250,6 +1247,14 @@ CreateCommandTag(Node *parsetree)
 			tag = "CREATE TABLE";
 			break;
 
+		case T_CreateTableSpaceStmt:
+			tag = "CREATE TABLESPACE";
+			break;
+
+		case T_DropTableSpaceStmt:
+			tag = "DROP TABLESPACE";
+			break;
+
 		case T_DropStmt:
 			switch (((DropStmt *) parsetree)->removeType)
 			{
@@ -1312,6 +1317,9 @@ CreateCommandTag(Node *parsetree)
 				case OBJECT_GROUP:
 					tag = "ALTER GROUP";
 					break;
+				case OBJECT_INDEX:
+					tag = "ALTER INDEX";
+					break;
 				case OBJECT_LANGUAGE:
 					tag = "ALTER LANGUAGE";
 					break;
@@ -1321,6 +1329,9 @@ CreateCommandTag(Node *parsetree)
 				case OBJECT_SCHEMA:
 					tag = "ALTER SCHEMA";
 					break;
+				case OBJECT_TABLESPACE:
+					tag = "ALTER TABLESPACE";
+					break;
 				case OBJECT_TRIGGER:
 					tag = "ALTER TRIGGER";
 					break;
@@ -1332,10 +1343,60 @@ CreateCommandTag(Node *parsetree)
 			}
 			break;
 
-		case T_AlterTableStmt:
-			tag = "ALTER TABLE";
+		case T_AlterOwnerStmt:
+			switch (((AlterOwnerStmt *) parsetree)->objectType)
+			{
+				case OBJECT_AGGREGATE:
+					tag = "ALTER AGGREGATE";
+					break;
+				case OBJECT_CONVERSION:
+					tag = "ALTER CONVERSION";
+					break;
+				case OBJECT_DATABASE:
+					tag = "ALTER DATABASE";
+					break;
+				case OBJECT_DOMAIN:
+					tag = "ALTER DOMAIN";
+					break;
+				case OBJECT_FUNCTION:
+					tag = "ALTER FUNCTION";
+					break;
+				case OBJECT_OPERATOR:
+					tag = "ALTER OPERATOR";
+					break;
+				case OBJECT_OPCLASS:
+					tag = "ALTER OPERATOR CLASS";
+					break;
+				case OBJECT_SCHEMA:
+					tag = "ALTER SCHEMA";
+					break;
+				case OBJECT_TABLESPACE:
+					tag = "ALTER TABLESPACE";
+					break;
+				case OBJECT_TYPE:
+					tag = "ALTER TYPE";
+					break;
+				default:
+					tag = "ALTER TABLE";
+			}
 			break;
 
+		case T_AlterTableStmt:
+			{
+				AlterTableStmt *stmt = (AlterTableStmt *) parsetree;
+
+				/*
+				 * We might be supporting ALTER INDEX here, so set the
+				 * completion table appropriately. Catch all other
+				 * possibilities with ALTER TABLE
+				 */
+
+				if (stmt->relkind == OBJECT_INDEX)
+					tag = "ALTER INDEX";
+				else
+					tag = "ALTER TABLE";
+			}
+			break;
 		case T_AlterDomainStmt:
 			tag = "ALTER DOMAIN";
 			break;
@@ -1572,5 +1633,54 @@ CreateCommandTag(Node *parsetree)
 #ifdef USE_REPLICATION
 	PGR_Set_Replication_Flag(parsetree);
 #endif /* USE_REPLICATION */
+
+	return tag;
+}
+
+/*
+ * CreateQueryTag
+ *		utility to get a string representation of a Query operation.
+ *
+ * This is exactly like CreateCommandTag, except it works on a Query
+ * that has already been through parse analysis (and possibly further).
+ */
+const char *
+CreateQueryTag(Query *parsetree)
+{
+	const char *tag;
+
+	switch (parsetree->commandType)
+	{
+		case CMD_SELECT:
+			/*
+			 * We take a little extra care here so that the result will
+			 * be useful for complaints about read-only statements
+			 */
+			if (parsetree->into != NULL)
+				tag = "SELECT INTO";
+			else if (parsetree->rowMarks != NIL)
+				tag = "SELECT FOR UPDATE";
+			else
+				tag = "SELECT";
+			break;
+		case CMD_UPDATE:
+			tag = "UPDATE";
+			break;
+		case CMD_INSERT:
+			tag = "INSERT";
+			break;
+		case CMD_DELETE:
+			tag = "DELETE";
+			break;
+		case CMD_UTILITY:
+			tag = CreateCommandTag(parsetree->utilityStmt);
+			break;
+		default:
+			elog(WARNING, "unrecognized commandType: %d",
+				 (int) parsetree->commandType);
+			tag = "???";
+			break;
+	}
+
 	return tag;
 }
