@@ -5,26 +5,27 @@
  *	  Functions for the built-in type "RelativeTime".
  *	  Functions for the built-in type "TimeInterval".
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/utils/adt/nabstime.c,v 1.117.2.1 2004/05/05 17:28:57 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/adt/nabstime.c,v 1.127 2004/12/31 22:01:22 pgsql Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include "postgres.h"
 
 #include <ctype.h>
-#include <time.h>
-#include <sys/time.h>
 #include <float.h>
 #include <limits.h>
+#include <time.h>
+#include <sys/time.h>
 
 #include "access/xact.h"
 #include "libpq/pqformat.h"
 #include "miscadmin.h"
+#include "pgtime.h"
 #include "utils/builtins.h"
 
 #ifdef USE_REPLICATION
@@ -79,8 +80,8 @@
  * Function prototypes -- internal to this file only
  */
 
-static AbsoluteTime tm2abstime(struct tm * tm, int tz);
-static void reltime2tm(RelativeTime time, struct tm * tm);
+static AbsoluteTime tm2abstime(struct pg_tm * tm, int tz);
+static void reltime2tm(RelativeTime time, struct pg_tm * tm);
 static int istinterval(char *i_string,
 			AbsoluteTime *i_start,
 			AbsoluteTime *i_end);
@@ -149,10 +150,10 @@ AbsoluteTimeUsecToTimestampTz(AbsoluteTime sec, int usec)
 /*
  * GetCurrentDateTime()
  *
- * Get the transaction start time ("now()") broken down as a struct tm.
+ * Get the transaction start time ("now()") broken down as a struct pg_tm.
  */
 void
-GetCurrentDateTime(struct tm * tm)
+GetCurrentDateTime(struct pg_tm * tm)
 {
 	int			tz;
 
@@ -174,11 +175,11 @@ GetCurrentDateTime(struct tm * tm)
 /*
  * GetCurrentTimeUsec()
  *
- * Get the transaction start time ("now()") broken down as a struct tm,
+ * Get the transaction start time ("now()") broken down as a struct pg_tm,
  * including fractional seconds and timezone offset.
  */
 void
-GetCurrentTimeUsec(struct tm * tm, fsec_t *fsec, int *tzp)
+GetCurrentTimeUsec(struct pg_tm * tm, fsec_t *fsec, int *tzp)
 {
 	int			tz;
 	int			usec;
@@ -208,10 +209,10 @@ GetCurrentTimeUsec(struct tm * tm, fsec_t *fsec, int *tzp)
 
 
 void
-abstime2tm(AbsoluteTime _time, int *tzp, struct tm * tm, char **tzn)
+abstime2tm(AbsoluteTime _time, int *tzp, struct pg_tm * tm, char **tzn)
 {
-	time_t		time = (time_t) _time;
-	struct tm  *tx;
+	pg_time_t	time = (pg_time_t) _time;
+	struct pg_tm *tx;
 
 	/*
 	 * If HasCTZSet is true then we have a brute force time zone
@@ -222,9 +223,9 @@ abstime2tm(AbsoluteTime _time, int *tzp, struct tm * tm, char **tzn)
 		time -= CTimeZone;
 
 	if ((!HasCTZSet) && (tzp != NULL))
-		tx = localtime(&time);
+		tx = pg_localtime(&time);
 	else
-		tx = gmtime(&time);
+		tx = pg_gmtime(&time);
 
 	tm->tm_year = tx->tm_year + 1900;
 	tm->tm_mon = tx->tm_mon + 1;
@@ -234,7 +235,6 @@ abstime2tm(AbsoluteTime _time, int *tzp, struct tm * tm, char **tzn)
 	tm->tm_sec = tx->tm_sec;
 	tm->tm_isdst = tx->tm_isdst;
 
-#if defined(HAVE_TM_ZONE)
 	tm->tm_gmtoff = tx->tm_gmtoff;
 	tm->tm_zone = tx->tm_zone;
 
@@ -279,66 +279,6 @@ abstime2tm(AbsoluteTime _time, int *tzp, struct tm * tm, char **tzn)
 	}
 	else
 		tm->tm_isdst = -1;
-#elif defined(HAVE_INT_TIMEZONE)
-	if (tzp != NULL)
-	{
-		/*
-		 * We have a brute force time zone per SQL99? Then use it without
-		 * change since we have already rotated to the time zone.
-		 */
-		if (HasCTZSet)
-		{
-			*tzp = CTimeZone;
-			tm->tm_isdst = 0;
-			if (tzn != NULL)
-				*tzn = NULL;
-		}
-		else
-		{
-			*tzp = ((tm->tm_isdst > 0) ? (TIMEZONE_GLOBAL - 3600) : TIMEZONE_GLOBAL);
-
-			if (tzn != NULL)
-			{
-				/*
-				 * Copy no more than MAXTZLEN bytes of timezone to tzn, in
-				 * case it contains an error message, which doesn't fit in
-				 * the buffer
-				 */
-				StrNCpy(*tzn, tzname[tm->tm_isdst], MAXTZLEN + 1);
-				if (strlen(tzname[tm->tm_isdst]) > MAXTZLEN)
-					ereport(WARNING,
-							(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
-							 errmsg("invalid time zone name: \"%s\"",
-									tzname[tm->tm_isdst])));
-			}
-		}
-	}
-	else
-		tm->tm_isdst = -1;
-#else							/* not (HAVE_TM_ZONE || HAVE_INT_TIMEZONE) */
-	if (tzp != NULL)
-	{
-		/*
-		 * We have a brute force time zone per SQL99? Then use it without
-		 * change since we have already rotated to the time zone.
-		 */
-		if (HasCTZSet)
-		{
-			*tzp = CTimeZone;
-			if (tzn != NULL)
-				*tzn = NULL;
-		}
-		else
-		{
-			/* default to UTC */
-			*tzp = 0;
-			if (tzn != NULL)
-				*tzn = NULL;
-		}
-	}
-	else
-		tm->tm_isdst = -1;
-#endif
 }
 
 
@@ -347,7 +287,7 @@ abstime2tm(AbsoluteTime _time, int *tzp, struct tm * tm, char **tzn)
  * Note that tm has full year (not 1900-based) and 1-based month.
  */
 static AbsoluteTime
-tm2abstime(struct tm * tm, int tz)
+tm2abstime(struct pg_tm * tm, int tz)
 {
 	int			day;
 	AbsoluteTime sec;
@@ -393,7 +333,7 @@ abstimein(PG_FUNCTION_ARGS)
 	AbsoluteTime result;
 	fsec_t		fsec;
 	int			tz = 0;
-	struct tm	date,
+	struct pg_tm date,
 			   *tm = &date;
 	int			dterr;
 	char	   *field[MAXDATEFIELDS];
@@ -459,7 +399,7 @@ abstimeout(PG_FUNCTION_ARGS)
 	char	   *result;
 	int			tz;
 	double		fsec = 0;
-	struct tm	tt,
+	struct pg_tm tt,
 			   *tm = &tt;
 	char		buf[MAXDATELEN + 1];
 	char		zone[MAXDATELEN + 1],
@@ -536,9 +476,9 @@ static int
 abstime_cmp_internal(AbsoluteTime a, AbsoluteTime b)
 {
 	/*
-	 * We consider all INVALIDs to be equal and larger than any non-INVALID.
-	 * This is somewhat arbitrary; the important thing is to have a
-	 * consistent sort order.
+	 * We consider all INVALIDs to be equal and larger than any
+	 * non-INVALID. This is somewhat arbitrary; the important thing is to
+	 * have a consistent sort order.
 	 */
 	if (a == INVALID_ABSTIME)
 	{
@@ -638,11 +578,11 @@ btabstimecmp(PG_FUNCTION_ARGS)
 Datum
 timestamp_abstime(PG_FUNCTION_ARGS)
 {
-	Timestamp	timestamp = PG_GETARG_TIMESTAMP(0);
+	Timestamp timestamp = PG_GETARG_TIMESTAMP(0);
 	AbsoluteTime result;
 	fsec_t		fsec;
 	int			tz;
-	struct tm	tt,
+	struct pg_tm tt,
 			   *tm = &tt;
 
 	if (TIMESTAMP_IS_NOBEGIN(timestamp))
@@ -673,7 +613,7 @@ abstime_timestamp(PG_FUNCTION_ARGS)
 {
 	AbsoluteTime abstime = PG_GETARG_ABSOLUTETIME(0);
 	Timestamp	result;
-	struct tm	tt,
+	struct pg_tm tt,
 			   *tm = &tt;
 	int			tz;
 	char		zone[MAXDATELEN + 1],
@@ -718,7 +658,7 @@ timestamptz_abstime(PG_FUNCTION_ARGS)
 	TimestampTz timestamp = PG_GETARG_TIMESTAMP(0);
 	AbsoluteTime result;
 	fsec_t		fsec;
-	struct tm	tt,
+	struct pg_tm tt,
 			   *tm = &tt;
 
 	if (TIMESTAMP_IS_NOBEGIN(timestamp))
@@ -746,7 +686,7 @@ abstime_timestamptz(PG_FUNCTION_ARGS)
 {
 	AbsoluteTime abstime = PG_GETARG_ABSOLUTETIME(0);
 	TimestampTz result;
-	struct tm	tt,
+	struct pg_tm tt,
 			   *tm = &tt;
 	int			tz;
 	char		zone[MAXDATELEN + 1],
@@ -794,7 +734,7 @@ reltimein(PG_FUNCTION_ARGS)
 {
 	char	   *str = PG_GETARG_CSTRING(0);
 	RelativeTime result;
-	struct tm	tt,
+	struct pg_tm tt,
 			   *tm = &tt;
 	fsec_t		fsec;
 	int			dtype;
@@ -842,7 +782,7 @@ reltimeout(PG_FUNCTION_ARGS)
 {
 	RelativeTime time = PG_GETARG_RELATIVETIME(0);
 	char	   *result;
-	struct tm	tt,
+	struct pg_tm tt,
 			   *tm = &tt;
 	char		buf[MAXDATELEN + 1];
 
@@ -880,7 +820,7 @@ reltimesend(PG_FUNCTION_ARGS)
 
 
 static void
-reltime2tm(RelativeTime time, struct tm * tm)
+reltime2tm(RelativeTime time, struct pg_tm * tm)
 {
 	double		dtime = time;
 
@@ -907,6 +847,7 @@ tintervalin(PG_FUNCTION_ARGS)
 				t2;
 
 	interval = (TimeInterval) palloc(sizeof(TimeIntervalData));
+
 	if (istinterval(intervalstr, &t1, &t2) == 0)
 		ereport(ERROR,
 				(errcode(ERRCODE_INVALID_DATETIME_FORMAT),
@@ -914,14 +855,15 @@ tintervalin(PG_FUNCTION_ARGS)
 						intervalstr)));
 
 	if (t1 == INVALID_ABSTIME || t2 == INVALID_ABSTIME)
-		interval->status = T_INTERVAL_INVAL;	/* undefined  */
+		interval  ->status = T_INTERVAL_INVAL;	/* undefined  */
+
 	else
-		interval->status = T_INTERVAL_VALID;
+		interval  ->status = T_INTERVAL_VALID;
 
 	i_start = ABSTIMEMIN(t1, t2);
 	i_end = ABSTIMEMAX(t1, t2);
-	interval->data[0] = i_start;
-	interval->data[1] = i_end;
+	interval  ->data[0] = i_start;
+	interval  ->data[1] = i_end;
 
 	PG_RETURN_TIMEINTERVAL(interval);
 }
@@ -968,15 +910,16 @@ tintervalrecv(PG_FUNCTION_ARGS)
 
 	interval = (TimeInterval) palloc(sizeof(TimeIntervalData));
 
-	interval->status = pq_getmsgint(buf, sizeof(interval->status));
+	interval  ->status = pq_getmsgint(buf, sizeof(interval->status));
+
 	if (!(interval->status == T_INTERVAL_INVAL ||
 		  interval->status == T_INTERVAL_VALID))
 		ereport(ERROR,
 				(errcode(ERRCODE_INVALID_BINARY_REPRESENTATION),
-				 errmsg("invalid status in external \"tinterval\" value")));
+			  errmsg("invalid status in external \"tinterval\" value")));
 
-	interval->data[0] = pq_getmsgint(buf, sizeof(interval->data[0]));
-	interval->data[1] = pq_getmsgint(buf, sizeof(interval->data[1]));
+	interval  ->data[0] = pq_getmsgint(buf, sizeof(interval->data[0]));
+	interval  ->data[1] = pq_getmsgint(buf, sizeof(interval->data[1]));
 
 	PG_RETURN_TIMEINTERVAL(interval);
 }
@@ -1022,7 +965,7 @@ interval_reltime(PG_FUNCTION_ARGS)
 		year = 0;
 		month = 0;
 	}
-	else if (abs(interval->month) >= 12)
+	else if (abs(interval->month) >=12)
 	{
 		year = (interval->month / 12);
 		month = (interval->month % 12);
@@ -1107,12 +1050,13 @@ mktinterval(PG_FUNCTION_ARGS)
 	interval = (TimeInterval) palloc(sizeof(TimeIntervalData));
 
 	if (t1 == INVALID_ABSTIME || t2 == INVALID_ABSTIME)
-		interval->status = T_INTERVAL_INVAL;
+		interval  ->status = T_INTERVAL_INVAL;
+
 	else
 	{
-		interval->status = T_INTERVAL_VALID;
-		interval->data[0] = tstart;
-		interval->data[1] = tend;
+		interval  ->status = T_INTERVAL_VALID;
+		interval  ->data[0] = tstart;
+		interval  ->data[1] = tend;
 	}
 
 	PG_RETURN_TIMEINTERVAL(interval);
@@ -1229,9 +1173,9 @@ static int
 reltime_cmp_internal(RelativeTime a, RelativeTime b)
 {
 	/*
-	 * We consider all INVALIDs to be equal and larger than any non-INVALID.
-	 * This is somewhat arbitrary; the important thing is to have a
-	 * consistent sort order.
+	 * We consider all INVALIDs to be equal and larger than any
+	 * non-INVALID. This is somewhat arbitrary; the important thing is to
+	 * have a consistent sort order.
 	 */
 	if (a == INVALID_RELTIME)
 	{
@@ -1343,7 +1287,7 @@ tintervalsame(PG_FUNCTION_ARGS)
  * tinterval comparison routines
  *
  * Note: comparison is based on the lengths of the intervals, not on
- * endpoint value.  This is pretty bogus, but since it's only a legacy
+ * endpoint value.	This is pretty bogus, but since it's only a legacy
  * datatype I'm not going to propose changing it.
  */
 static int
@@ -1355,9 +1299,9 @@ tinterval_cmp_internal(TimeInterval a, TimeInterval b)
 	AbsoluteTime b_len;
 
 	/*
-	 * We consider all INVALIDs to be equal and larger than any non-INVALID.
-	 * This is somewhat arbitrary; the important thing is to have a
-	 * consistent sort order.
+	 * We consider all INVALIDs to be equal and larger than any
+	 * non-INVALID. This is somewhat arbitrary; the important thing is to
+	 * have a consistent sort order.
 	 */
 	a_invalid = ((a->status == T_INTERVAL_INVAL) ||
 				 (a->data[0] == INVALID_ABSTIME) ||
@@ -1755,20 +1699,20 @@ timeofday(PG_FUNCTION_ARGS)
 {
 	struct timeval tp;
 	struct timezone tpz;
-	char		templ[100];
-	char		buf[100];
+	char		templ[128];
+	char		buf[128];
 	text	   *result;
 	int			len;
-	time_t		tt;
+	pg_time_t	tt;
 
 #ifdef USE_REPLICATION
 	PGR_GetTimeOfDay(&tp,&tpz);
 #else
 	gettimeofday(&tp, &tpz);
 #endif /* USE_REPLICATION */
-	tt = (time_t) tp.tv_sec;
-	strftime(templ, sizeof(templ), "%a %b %d %H:%M:%S.%%06d %Y %Z",
-			 localtime(&tt));
+	tt = (pg_time_t) tp.tv_sec;
+	pg_strftime(templ, sizeof(templ), "%a %b %d %H:%M:%S.%%06d %Y %Z",
+				pg_localtime(&tt));
 	snprintf(buf, sizeof(buf), templ, tp.tv_usec);
 
 	len = VARHDRSZ + strlen(buf);
