@@ -3,12 +3,12 @@
  * assert.c
  *	  Assert code.
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/utils/error/assert.c,v 1.24 2003/08/04 02:40:06 momjian Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/error/assert.c,v 1.30 2004/12/31 22:01:27 pgsql Exp $
  *
  * NOTE
  *	  This should eventually work with elog()
@@ -22,6 +22,7 @@
 #ifdef USE_REPLICATION
 #include "replicate.h"
 #endif /* USE_REPLICATION */
+
 /*
  * ExceptionalCondition - Handles the failure of an Assert()
  */
@@ -34,18 +35,23 @@ ExceptionalCondition(char *conditionName,
 	if (!PointerIsValid(conditionName)
 		|| !PointerIsValid(fileName)
 		|| !PointerIsValid(errorType))
-		fprintf(stderr, "TRAP: ExceptionalCondition: bad arguments\n");
+		write_stderr("TRAP: ExceptionalCondition: bad arguments\n");
 	else
 	{
-		fprintf(stderr, "TRAP: %s(\"%s\", File: \"%s\", Line: %d)\n",
-				errorType, conditionName,
-				fileName, lineNumber);
+		write_stderr("TRAP: %s(\"%s\", File: \"%s\", Line: %d)\n",
+					 errorType, conditionName,
+					 fileName, lineNumber);
 	}
 
 #ifdef USE_REPLICATION
 	PGR_ExceptionalCondition();
 #endif /* USE_REPLICATION */
 #ifdef SLEEP_ON_ASSERT
+
+	/*
+	 * It would be nice to use pg_usleep() here, but only does 2000 sec or
+	 * 33 minutes, which seems too short.
+	 */
 	sleep(1000000);
 #endif
 
