@@ -4,7 +4,7 @@
  * (currently mule internal code (mic) is used)
  * Tatsuo Ishii
  *
- * $Header: /cvsroot/pgsql/src/backend/utils/mb/mbutils.c,v 1.44 2003/09/25 06:58:05 petere Exp $
+ * $PostgreSQL: pgsql/src/backend/utils/mb/mbutils.c,v 1.48 2004/10/13 01:25:12 neilc Exp $
  */
 #include "postgres.h"
 
@@ -252,9 +252,9 @@ pg_do_encoding_conversion(unsigned char *src, int len,
 	{
 		ereport(LOG,
 				(errcode(ERRCODE_UNDEFINED_FUNCTION),
-			errmsg("default conversion function for encoding \"%s\" to \"%s\" does not exist",
-				   pg_encoding_to_char(src_encoding),
-				   pg_encoding_to_char(dest_encoding))));
+				 errmsg("default conversion function for encoding \"%s\" to \"%s\" does not exist",
+						pg_encoding_to_char(src_encoding),
+						pg_encoding_to_char(dest_encoding))));
 		return src;
 	}
 
@@ -473,6 +473,13 @@ pg_mblen(const unsigned char *mbstr)
 	return ((*pg_wchar_table[DatabaseEncoding->encoding].mblen) (mbstr));
 }
 
+/* returns the display length of a multibyte word */
+int
+pg_dsplen(const unsigned char *mbstr)
+{
+	return ((*pg_wchar_table[DatabaseEncoding->encoding].dsplen) (mbstr));
+}
+
 /* returns the length (counted as a wchar) of a multibyte string */
 int
 pg_mbstrlen(const unsigned char *mbstr)
@@ -577,7 +584,7 @@ SetDatabaseEncoding(int encoding)
 }
 
 void
-SetDefaultClientEncoding()
+SetDefaultClientEncoding(void)
 {
 	ClientEncoding = &pg_enc2name_tbl[GetDatabaseEncoding()];
 }
