@@ -534,7 +534,7 @@ rsync_pg_data(char * src, char * dest)
 	{
 		status = execv(PGCluster_Info->RsyncPath,args);
 	}
-	else
+	else if (pid > 0)
 	{
 		while (waitpid(pid, &status, WNOHANG) < 0 ) 
 		{
@@ -683,6 +683,8 @@ first_recovery(char * src, char * dest, char * dir)
 	int status = STATUS_OK;
 	char src_dir[256];
 	char dest_dir[256];
+	struct stat fstat;
+	int cnt = 0;
 
 	memset(src_dir,0,sizeof(src_dir));
 	memset(dest_dir,0,sizeof(dest_dir));
@@ -699,6 +701,17 @@ first_recovery(char * src, char * dest, char * dir)
 		restore_dir(dest_dir);
 		return STATUS_ERROR;
 	}
+	/* check pg_control file */
+	cnt = 0;
+	do {
+		if (cnt > MAX_RETRY_TIMES )
+		{
+			restore_dir(dest_dir);
+			return STATUS_ERROR;
+		}
+		cnt ++;
+		sleep(1);
+	} while (stat(dest_dir,&fstat) < 0);
 	return STATUS_OK;
 }
 
@@ -708,6 +721,8 @@ second_recovery(char * src, char * dest, char * dir)
 	int status = STATUS_OK;
 	char src_dir[256];
 	char dest_dir[256];
+	struct stat fstat;
+	int cnt = 0;
 
 	memset(src_dir,0,sizeof(src_dir));
 	memset(dest_dir,0,sizeof(dest_dir));
@@ -722,6 +737,17 @@ second_recovery(char * src, char * dest, char * dir)
 	}
 	clear_bkup_dir(dest_dir);
 
+	/* check pg_control file */
+	cnt = 0;
+	do {
+		if (cnt > MAX_RETRY_TIMES )
+		{
+			restore_dir(dest_dir);
+			return STATUS_ERROR;
+		}
+		cnt ++;
+		sleep(1);
+	} while (stat(dest_dir,&fstat) < 0);
 	return STATUS_OK;
 }
 
@@ -734,74 +760,66 @@ recovery_rsync(char * src , char * dest, int stage)
 	}
 
 	/* recovery step of "global" directory */
-	fprintf(stderr,"%s recovery step of [global] directory...",
-			((stage == 1)?"1st":"2nd"));
 	if (rsync_global_dir(src, dest) != STATUS_OK)
 	{
-		fprintf(stderr,"NG\n");
 		return STATUS_ERROR;
 	}
-	fprintf(stderr,"OK\n");
 
 	if (stage == PGR_1ST_RECOVERY)
 	{
 		/* 1st recovery step of "base" directory */
-		fprintf(stderr,"1st recovery step of [base] directory...");
 		if (first_recovery(src,dest,"base") != STATUS_OK)
 		{
-			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
-		fprintf(stderr,"OK\n");
-
-		fprintf(stderr,"1st recovery step of [pg_clog] directory...");
 		/* 1st recovery step of "pg_clog" directory */
 		if (first_recovery(src,dest,"pg_clog") != STATUS_OK)
 		{
-			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
-		fprintf(stderr,"OK\n");
-
-		fprintf(stderr,"1st recovery step of [pg_xlog] directory...");
 		/* 1st recovery step of "pg_xlog" directory */
 		if (first_recovery(src,dest,"pg_xlog") != STATUS_OK)
 		{
-			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
-		fprintf(stderr,"OK\n");
-
+		/* 1st recovery step of "pg_subtrans" directory */
+		if (first_recovery(src,dest,"pg_subtrans") != STATUS_OK)
+		{
+			return STATUS_ERROR;
+		}
+		/* 1st recovery step of "pg_tblspc" directory */
+		if (first_recovery(src,dest,"pg_tblspc") != STATUS_OK)
+		{
+			return STATUS_ERROR;
+		}
 	}
 	else
 	{
 		/* 2nd recovery step of "base" directory */
-		fprintf(stderr,"2nd recovery step of [base] directory...");
 		if (second_recovery(src,dest,"base") != STATUS_OK)
 		{
-			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
-		fprintf(stderr,"OK\n");
-
 		/* 2nd recovery step of "pg_clog" directory */
-		fprintf(stderr,"2nd recovery step of [pg_clog] directory...");
 		if (second_recovery(src,dest,"pg_clog") != STATUS_OK)
 		{
-			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
-		fprintf(stderr,"OK\n");
-
 		/* 2nd recovery step of "pg_xlog" directory */
-		fprintf(stderr,"2nd recovery step of [pg_xlog] directory...");
 		if (second_recovery(src,dest,"pg_xlog") != STATUS_OK)
 		{
-			fprintf(stderr,"NG\n");
 			return STATUS_ERROR;
 		}
-		fprintf(stderr,"OK\n");
-
+		/* 2nd recovery step of "pg_subtrans" directory */
+		if (second_recovery(src,dest,"pg_subtrans") != STATUS_OK)
+		{
+			return STATUS_ERROR;
+		}
+		/* 2nd recovery step of "pg_tblspc" directory */
+		if (second_recovery(src,dest,"pg_tblspc") != STATUS_OK)
+		{
+			return STATUS_ERROR;
+		}
 	}
 
 	return STATUS_OK;
