@@ -13,6 +13,7 @@
 #ifndef _SYS_TIME_H
 #include <sys/time.h>
 #endif
+#include "postmaster/postmaster.h"
 #include "tcop/dest.h"
 #include "storage/proc.h"
 #include "replicate_com.h"
@@ -27,6 +28,8 @@
 #define SESSION_AUTHORIZATION_BEGIN	(1)
 #define SESSION_AUTHORIZATION_END	(2)
 
+#define CLUSTER_ERROR_LOG_FILE	"pgcluster.log"
+#define CLUSTER_STATUS_LOG_FILE	"pgcluster.sts"
 #define READ_ONLY_IF_STAND_ALONE	"read_only"
 #define READ_WRITE_IF_STAND_ALONE	"read_write"
 #define PERMIT_READ_ONLY		(1)
@@ -51,18 +54,24 @@
 #define SKIP_QUERY_3 "SET autocommit TO 'on'"
 #define SKIP_QUERY_4 "SET search_path = public"
 
-#define PGR_SEND_RETRY_CNT (3)
+#define PGR_SEND_RETRY_CNT (1500)
 #define PGR_SEND_WAIT_MSEC (150)
-#define PGR_RECV_RETRY_CNT (3)
+#define PGR_RECV_RETRY_CNT (1500)
 #define PGR_RECV_WAIT_MSEC (150)
-#define PGR_SEND_TIMEOUT	(5)
-#define PGR_RECV_TIMEOUT	(5)
+#define PGR_SEND_TIMEOUT	(600)
+#define PGR_RECV_TIMEOUT	(30)
 #define PGR_LOCK_RETRY_CNT (50)
 #define PGR_LOCK_WAIT_MSEC (20)
 
 #define PGR_1ST_RECOVERY (1)
 #define PGR_2ND_RECOVERY (2)
 
+typedef struct CopyDataType
+{
+	int cnt;
+	char copy_data[COPYBUFSIZ];
+} CopyData;
+
 typedef struct
 {
 	bool is_stand_alone;
@@ -85,13 +94,62 @@ typedef struct
 
 typedef struct
 {
-	char * query_string;
+	const char * query_string;
 	int query_len;
 	char cmdSts;
 	char cmdType;
 	char useFlag;
 } PGR_Retry_Query_Type;
 
+/* cluster db data */
+typedef struct {
+	char * PGRSelfHostName;
+	ClusterDBInfo * ClusterDBData;
+	int ReplicateServerShmid;
+	int ClusterDBShmid;
+} PGR_ClusterDB_Info;
+
+/* partial replication data */
+typedef struct {
+	int PGR_Not_Replicate_Rec_Num;
+	PGR_Not_Replicate_Type * PGR_Not_Replicate;
+} PGR_Partial_Info;
+
+/* query string data */
+typedef struct {
+	const char * Query_String;
+	const char * Query_Ptr;
+	int Skip_Cnt;
+	int PGR_Status;
+	bool Transaction_Aborted;
+	bool PGR_Not_Replication_Query;
+} PGR_QueryString_Info;
+
+/*life check process pid */
+typedef struct {
+	pid_t Master_Pid;
+	pid_t SYN_Pid;
+	pid_t ACK_Pid;
+} PGR_Lifecheck_Pid_Info;
+
+/* cluster db mode */
+typedef struct {
+	bool Transaction_Mode;
+	bool Session_Authorization_Mode;
+	bool Create_Temp_Table_Mode;
+	bool PGR_Copy_Data_Need_Replicate;
+	bool PGR_Reliable_Mode_Wait;
+} PGR_ClusterDB_Mode_Info;
+
+/* replication server data */
+typedef struct {
+	ReplicateServerInfo * ReplicateServerData;
+	ReplicateServerInfo * CurrentReplicateServer;
+	ReplicateServerInfo * LastReplicateServer;
+	unsigned long * PGR_Current_Replication_Id;
+	int TransactionSock;
+	int PGR_Sock_To_Replication_Server;
+} PGR_ReplicationServer_Info;
 
 /* replicaition log */
 typedef struct {
@@ -99,77 +157,87 @@ typedef struct {
 	uint32_t PGR_Request_ID;
 } PGR_ReplicationLog_Info;
 
+/* Cluster DB global */
+typedef struct {
+	/* common data */
+	PGR_Com_Info Com_Info;
+	/* recovery option data */
+	char * RsyncPath;
+	char * RsyncOption;
+	bool PGR_Recovery_Option;
+	/* query buffer */
+	CopyData * PGRCopyData;
+	PGR_Retry_Query_Type PGR_Retry_Query;
+	/* status flag */
+	PGR_Stand_Alone_Type * PGR_Stand_Alone;
+	bool PGR_Is_Replicated_Query;
+	/* check & notice lock conflict */
+	bool PGR_Need_Notice;
+	bool PGR_Lock_Noticed;
+	PGR_Check_Lock_Type PGR_Check_Lock;
+	/* sync data */
+	ReplicateNow * ReplicateCurrentTime;
+} PGR_PGCluster_Info;
 
-extern char * Query_String;
-extern int TransactionQuery;
-extern bool Transaction_Mode;
-extern bool Session_Authorization_Mode;
-extern bool Create_Temp_Table_Mode;
-extern int RecoveryPortNumber;
-extern char * RsyncPath;
-extern char * RsyncOption;
-extern bool RsyncCompress;
-extern int TransactionSock;
-extern ReplicateNow * ReplicateCurrentTime;
-extern CopyData * PGRCopyData;
-extern bool PGR_Copy_Data_Need_Replicate;
-extern PGR_Stand_Alone_Type * PGR_Stand_Alone;
-extern PGR_Not_Replicate_Type * PGR_Not_Replicate;
-extern int PGR_Not_Replicate_Rec_Num;
-extern bool autocommit;
-extern bool PGR_Is_Replicated_Query;
-extern PGR_Check_Lock_Type PGR_Check_Lock;
-extern int PGR_Sock_To_Replication_Server;
-extern bool PGR_Need_Notice;
-extern bool PGR_Lock_Noticed;
-extern ReplicateServerInfo * CurrentReplicateServer;
-extern ReplicateServerInfo * LastReplicateServer;
-extern char * PGRSelfHostName;
-extern int PGR_Pending_Sem_Num;
-extern int PGR_Response_Mode;
-extern bool PGR_Reliable_Mode_Wait;
-extern PGR_Retry_Query_Type PGR_Retry_Query;
-extern bool PGRforceLoadBalance;
-extern bool needToUpdateReplicateIdOnNextQueryIsDone;
-extern PGR_ReplicationLog_Info ReplicationLog_Info;
+extern PGR_PGCluster_Info *PGCluster_Info;
 
-/* in backend/libpq/replicate.c */
-extern int PGR_Init_Replicate_Server_Data(void);
+/* in backend/utiles/pgcluster/replicate.c */
+extern int PGR_Init_Replicate_Server_Data(char * path);
 extern int PGR_Set_Replicate_Server_Socket(void);
-extern ReplicateServerInfo * PGR_get_replicate_server_info(void);
-extern char * PGR_Send_Replicate_Command(char * query_string, int query_len, char cmdSts ,char cmdType);
-extern bool PGR_Is_Replicated_Command(char * query);
+extern char * PGR_Send_Replicate_Command(const char * query_string, int query_len, char cmdSts ,char cmdType);
+extern bool PGR_Is_Replicated_Command(const char * query);
 extern int Xlog_Check_Replicate(int operation);
 extern int PGR_Replicate_Function_Call(void);
 extern void PGR_delete_shm(void);
-extern int PGR_replication(char * query_string, CommandDest dest, Node *parsetree, const char * commandTag);
-extern bool PGR_Is_System_Command(char * query);
+extern ReplicateServerInfo * PGR_get_replicate_server_info(void);
+extern bool PGR_Is_System_Command(const char * query);
 extern int PGR_Call_System_Command(char * command);
-extern int PGR_GetTimeOfDay(struct timeval *tp,struct timezone *tpz);
+extern int PGR_GetTimeOfDay(struct timeval *tp, struct timezone *tpz);
 extern long PGR_Random(void);
 extern int PGR_Set_Current_Time(char * sec, char * usec);
-extern int PGR_Send_Copy(CopyData * copy, int end);
-extern CopyData * PGR_Set_Copy_Data(CopyData * copy, char *str, int len, int end);
-extern char * PGR_scan_terminate( char * str);
+extern int PGR_Send_Copy(CopyData * copy,int end);
+extern CopyData * PGR_Set_Copy_Data(CopyData * copy, char * str, int len, int end);
+extern char * PGR_scan_terminate(const char * str);
 extern bool PGR_Is_Stand_Alone(void);
 extern void PGR_Send_Message_To_Frontend(char * msg);
 extern void PGR_Notice_Transaction_Query_Done(void);
 extern void PGR_Notice_Transaction_Query_Aborted(void);
 extern int PGR_Notice_Conflict(void);
 extern int PGR_Recv_Trigger (int user_timeout);
-extern bool PGR_Check_DeadLock(LOCKMETHODTABLE *lockMethodTable, LOCKMODE lockmode, LOCK *lock, PROCLOCK *holder);
 extern void PGR_Set_Replication_Server_Status( ReplicateServerInfo * sp, int status);
-extern int PGR_Is_Skip_Replication(char * query);
+extern int PGR_Is_Skip_Replication(const char * query);
 extern bool PGR_Did_Commit_Transaction(void);
 extern bool PGR_Set_Transaction_Mode(bool mode,const char * commandTag);
-extern char * PGR_Remove_Comment(char * str);
+extern char * PGR_Remove_Comment(const char * str);
 extern void PGR_Force_Replicate_Query(void);
 extern void PGR_Notice_DeadLock(void);
 extern void PGR_Set_Cluster_Status(int status);
 extern int PGR_Get_Cluster_Status(void);
-extern int PGR_Check_Replicate_Server_Status(ReplicateServerInfo * sp);
+extern int PGR_lo_create(int flags);
+extern int PGR_lo_open(Oid lobjId,int32 mode);
+extern int PGR_lo_close(int32 fd);
+extern int PGR_lo_write(int fd, char *buf, int len);
+extern int PGR_lo_lseek(int32 fd, int32 offset, int32 whence);
+extern int PGR_lo_unlink(Oid lobjId);
+extern void PGR_init_replication(const char * query, int len);
+extern void PGR_init_each_query(void);
+extern int PGR_exec_replication(const char * commandTag,CommandDest dest);
+extern int PGR_startup_lifecheck(void);
+extern void PGR_finish_lifecheck(int sig);
+extern int PGR_Wait_Reliable_Trigger(const char * commandTag);
+extern void PGR_Set_Replication_Flag(Node * parsetree);
+extern int PGR_Init_PGCluster_Info(void);
+extern int PGR_Get_Cluster_Conf_Data(char * path);
+extern void PGR_Clear_Children(int sig);
+extern void PGR_Recovery_Failed(void);
+extern void PGR_Elog_Abort_Copy(void);
+extern void PGR_Elog_Abort_Proc(int timeout);
+extern void PGR_Elog_Warn_Proc(char * msg, char * fix);
+extern void PGR_ExceptionalCondition(void);
+extern bool PGR_Is_This_Transaction_Committed(void);
+extern void PGR_Set_Copy_Char(int ch);
 
-/* in backend/libpq/recovery.c */
+/* in backend/utiles/pgcluster/recovery.c */
 extern int PGR_Master_Main(void);
 extern int PGR_Recovery_Main(void);
 extern int PGR_recovery_error_send(void);
