@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
  *
- * replicate.h
+ * replicate_com.h
  *	  Primary include file for replicate server .c files
  *
  * This should be the first file included by replicate modules.
@@ -64,6 +64,9 @@
 #define CMD_TYPE_BEGIN	'B'
 #define CMD_TYPE_COMMIT	'E'
 #define CMD_TYPE_ROLLBACK	'R'
+#define CMD_TYPE_SAVEPOINT	's'
+#define CMD_TYPE_ROLLBACK_TO_SAVEPOINT	'r'
+#define CMD_TYPE_RELEASE_SAVEPOINT	'l'
 #define CMD_TYPE_SESSION_AUTHORIZATION_BEGIN	'a'
 #define CMD_TYPE_SESSION_AUTHORIZATION_END	'b'
 #define CMD_TYPE_OTHER	'O'
