@@ -21,6 +21,10 @@
  * is" without express or implied warranty.
  *
 */
+#ifdef USE_REPLICATION
+
+#include "postgres.h"
+
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
@@ -28,7 +32,6 @@
 #include <unistd.h>
 #include <signal.h>
 #include <sys/wait.h>
-#include <sys/time.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -41,6 +44,7 @@
 #include <netinet/in.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <sys/time.h>
 #include <time.h>
 #include <sys/param.h>
 #include <sys/select.h>
@@ -59,7 +63,6 @@
  * PROTOTYPE DECLARATION
  *--------------------------------------
  */
-int PGRis_cluster_alive(void) ;
 ClusterTbl * PGRscan_cluster(void);
 void PGRset_key_of_cluster(ClusterTbl * ptr, RecoveryPacket * packet);
 ClusterTbl * PGRadd_cluster_tbl (ClusterTbl * conf_data);
@@ -70,24 +73,6 @@ static int set_cluster_tbl(ClusterTbl * ptr , ClusterTbl * conf_data);
 static ClusterTbl * search_free_cluster_tbl(void );
 static void write_cluster_status_file(ClusterTbl * ptr);
 
-int PGRis_cluster_alive(void) 
-{
-	ClusterTbl * ptr = NULL;
-	int use=0;
-	ptr = Cluster_Tbl;
-
-	PGRsem_lock(ClusterSemid,MAX_DB_SERVER);
-	while (ptr->useFlag != TBL_END)
-	{
-		if (ptr->useFlag == TBL_USE)
-		{
-		  use++;
-		}
-		ptr++;
-	}
-	PGRsem_unlock(ClusterSemid,MAX_DB_SERVER);
-	return use==0 ? STATUS_ERROR : STATUS_OK;
-}
 
 ClusterTbl * 
 PGRscan_cluster(void)
@@ -110,18 +95,18 @@ PGRscan_cluster(void)
 	show_debug("%s:%d ClusterDB can be used",func,ClusterNum);
 #endif			
 	PGRsem_lock(ClusterSemid,MAX_DB_SERVER);
-	while ((cnt <= ClusterNum) && (ptr->useFlag != TBL_END))
+	while ((cnt < ClusterNum) && (ptr->useFlag != TBL_END))
 	{
 #ifdef PRINT_DEBUG
 		show_debug("%s:%s [%d],useFlag->%d max->%d use_num->%d\n",
 			func, ptr->hostName,ptr->port,ptr->useFlag,ptr->max_connect,ptr->use_num);
 #endif			
-		cnt ++;
 		if (ptr->useFlag != TBL_USE)
 		{
 			ptr ++;
 			continue;
 		}
+		cnt ++;
 		if (ptr->max_connect <= ptr->use_num)
 		{
 			ptr ++;
@@ -159,7 +144,7 @@ PGRset_key_of_cluster(ClusterTbl * ptr, RecoveryPacket * packet)
 	int port = 0;
 
 	memset(ptr,0,sizeof(ClusterTbl));
-	memcpy(ptr->hostName,packet->hostName,sizeof(ptr->hostName));
+	strncpy(ptr->hostName,packet->hostName,sizeof(ptr->hostName));
 	max_connect = ntohs(packet->max_connect);
 	if (max_connect >= 0)
 	{
@@ -227,10 +212,9 @@ PGRset_status_on_cluster_tbl (int status, ClusterTbl * ptr)
 			write_cluster_status_file(ptr);
 			if (status == TBL_USE)
 			{
-				if (ClusterNum < Max_DB_Server)
-					ClusterNum ++ ;
+				ClusterNum ++ ;
 			}
-			else if (status != TBL_STOP)
+			else
 			{
 				if (ClusterNum > 0)
 					ClusterNum -- ;
@@ -246,27 +230,27 @@ write_cluster_status_file(ClusterTbl * ptr)
 	switch( ptr->useFlag)
 	{
 		case TBL_FREE:
-			PGRwrite_log_file(StatusFp,"port(%d) host:%s free",
+			PGRwrite_log_file(Com_Info.Log_Info.StatusFp,"port(%d) host:%s free",
 					ptr->port,
 					ptr->hostName);
 			break;
 		case TBL_INIT:
-			PGRwrite_log_file(StatusFp,"port(%d) host:%s initialize",
+			PGRwrite_log_file(Com_Info.Log_Info.StatusFp,"port(%d) host:%s initialize",
 					ptr->port,
 					ptr->hostName);
 			break;
 		case TBL_USE:
-			PGRwrite_log_file(StatusFp,"port(%d) host:%s start use",
+			PGRwrite_log_file(Com_Info.Log_Info.StatusFp,"port(%d) host:%s start use",
 					ptr->port,
 					ptr->hostName);
 			break;
 		case TBL_ERROR:
-			PGRwrite_log_file(StatusFp,"port(%d) host:%s error",
+			PGRwrite_log_file(Com_Info.Log_Info.StatusFp,"port(%d) host:%s error",
 					ptr->port,
 					ptr->hostName);
 			break;
 		case TBL_END:
-			PGRwrite_log_file(StatusFp,"port(%d) host:%s end",
+			PGRwrite_log_file(Com_Info.Log_Info.StatusFp,"port(%d) host:%s end",
 					ptr->port,
 					ptr->hostName);
 			break;
@@ -283,17 +267,14 @@ PGRsearch_cluster_tbl(ClusterTbl * conf_data)
 	ptr = Cluster_Tbl;
 	while ((cnt <= ClusterNum) && (rec_num < Max_DB_Server))
 	{
-		if (ptr->port > 0)
+		if ((!strcmp(ptr->hostName,conf_data->hostName)) &&
+			(ptr->port == conf_data->port))
 		{
-			if ((!strcmp(ptr->hostName,conf_data->hostName)) &&
-				(ptr->port == conf_data->port))
-			{
-				return ptr;
-			}
-			if (ptr->useFlag == TBL_USE)
-			{
-				cnt ++;
-			}
+			return ptr;
+		}
+		if (ptr->useFlag == TBL_USE)
+		{
+			cnt ++;
 		}
 		ptr ++;
 		rec_num ++;
@@ -307,7 +288,7 @@ set_cluster_tbl(ClusterTbl * ptr , ClusterTbl * conf_data)
 	int rec_no;
 
 	rec_no = ptr->rec_no;
-	memcpy(ptr->hostName,conf_data->hostName,sizeof(ptr->hostName));
+	strncpy(ptr->hostName,conf_data->hostName,sizeof(ptr->hostName));
 	ptr->max_connect = conf_data->max_connect;
 	ptr->port = conf_data->port;
 	ptr->use_num = conf_data->use_num;
@@ -336,3 +317,4 @@ search_free_cluster_tbl(void )
 	return (ClusterTbl *)NULL;
 }
 
+#endif /* USE_REPLICATION */
