@@ -5,7 +5,7 @@
  * NOTE:
  *     This file is composed of the main function of pglb.
  *
- * Portions Copyright (c) 2003-2006, Atsushi Mitani
+ * Portions Copyright (c) 2003, Atsushi Mitani
  *--------------------------------------------------------------------
  */
 /*
@@ -21,7 +21,10 @@
  * is" without express or implied warranty.
  *
 */
+#ifdef USE_REPLICATION
+
 #include "postgres.h"
+
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
@@ -29,7 +32,6 @@
 #include <unistd.h>
 #include <signal.h>
 #include <sys/wait.h>
-#include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
@@ -40,6 +42,7 @@
 #include <netdb.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <sys/time.h>
 #include <time.h>
 #include <sys/param.h>
 #include <sys/select.h>
@@ -63,31 +66,16 @@
 #include "replicate_com.h"
 #include "pglb.h"
 
-
-
 #define IPC_NMAXSEM (32)
 /*--------------------------------------
  * GLOBAL VARIABLE DECLARATION
  *--------------------------------------
  */
+
 /* for replicate_com.h */
-ConfDataType * ConfData_Top = (ConfDataType *)NULL;
-ConfDataType * ConfData_End = (ConfDataType *)NULL;
-int SynSemID = -1;
-int MapTableShmid = -1;
-int LifeCheckStartShmid = -1;
-char * LifeCheckStartFlag = NULL;
-int LifeCheckTimeOut = 10;
-FILE * StatusFp = (FILE *)NULL;
-FILE * LogFp = (FILE *)NULL;
-char * PGRStatusFileName = NULL;
-char * PGRLogFileName = NULL;
-int Log_Print = 0;
-int Debug_Print = 0;
+PGR_Com_Info Com_Info;
 
 int Recv_Port_Number = 0;
-int Recovery_Port_Number = 0;
-uint16_t LifeCheck_Port_Number = 0;
 int Use_Connection_Pool = 0;
 int Max_Pool = 1;
 int Connection_Life_Time = 0;
@@ -165,34 +153,34 @@ init_pglb(char * path)
 	}
 	
 	/* cluster db status file open */
-	if (PGRStatusFileName == NULL)
+	if (Com_Info.Log_Info.PGRStatusFileName == NULL)
 	{
 		snprintf(fname,sizeof(fname),"%s/%s",PGR_Write_Path,PGLB_STATUS_FILE);
 	}
 	else
 	{
-		memcpy(fname,PGRStatusFileName,sizeof(fname));
+		strncpy(fname,Com_Info.Log_Info.PGRStatusFileName,sizeof(fname));
 	}
-	StatusFp = fopen(fname, "a");
-	if (StatusFp == NULL)
+	Com_Info.Log_Info.StatusFp = fopen(fname, "a");
+	if (Com_Info.Log_Info.StatusFp == NULL)
 	{
 		show_error("%s:open() %s file failed. (%s)",
 				   func,fname, strerror(errno));
 		exit(1);
 	}
 	/* pglb log file open */
-	if (Log_Print)
+	if (Com_Info.Log_Info.Log_Print)
 	{
-		if (PGRLogFileName == NULL)
+		if (Com_Info.Log_Info.PGRLogFileName == NULL)
 		{
 			snprintf(fname,sizeof(fname),"%s/%s",PGR_Write_Path,PGLB_LOG_FILE);
 		}
 		else
 		{
-			memcpy(fname,PGRLogFileName,sizeof(fname));
+			strncpy(fname,Com_Info.Log_Info.PGRLogFileName,sizeof(fname));
 		}
-		LogFp = fopen(fname,"a");
-		if (LogFp == NULL)
+		Com_Info.Log_Info.LogFp = fopen(fname,"a");
+		if (Com_Info.Log_Info.LogFp == NULL)
 		{
 			show_error("%s:fopen failed: (%s)",func,strerror(errno));
 			exit(1);
@@ -215,7 +203,7 @@ init_pglb(char * path)
 	Connection_Life_Time = 0;
 	Use_Connection_Pool = 0;
 
-	conf = ConfData_Top;
+	conf = Com_Info.ConfData_Info.ConfData_Top;
 	while (conf != (ConfDataType *)NULL) 
 	{
 		/* get cluster db servers name */
@@ -224,7 +212,7 @@ init_pglb(char * path)
 			rec_no = conf->rec_no;
 			if (!strcmp(conf->key,HOST_NAME_TAG))
 			{
-				memcpy(cluster_tbl[rec_no].hostName,conf->value,sizeof(cluster_tbl[rec_no].hostName));
+				strncpy(cluster_tbl[rec_no].hostName,conf->value,sizeof(cluster_tbl[rec_no].hostName));
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
@@ -253,30 +241,28 @@ init_pglb(char * path)
 			/* get port number for recovery session */
 			if (!strcmp(conf->key,RECOVERY_PORT_TAG))
 			{
-				Recovery_Port_Number = atoi(conf->value);
+				Com_Info.ConfData_Info.Recovery_Port_Number = atoi(conf->value);
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
-			/*
 			if (!STRCMP(conf->key,LIFECHECK_PORT_TAG))
 			{
-				LifeCheck_Port_Number = atoi(conf->value);
+				Com_Info.ConfData_Info.LifeCheck_Port_Number = atoi(conf->value);
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
 			if (!STRCMP(conf->key,STATUS_LOG_FILE_TAG))
 			{
-				PGRStatusFileName = strdup(conf->value);
+				Com_Info.Log_Info.PGRStatusFileName = strdup(conf->value);
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
 			if (!STRCMP(conf->key,ERROR_LOG_FILE_TAG))
 			{
-				PGRLogFileName = strdup(conf->value);
+				Com_Info.Log_Info.PGRLogFileName = strdup(conf->value);
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
-			*/
 			if (!strcmp(conf->key,MAX_CLUSTER_TAG))
 			{
 				Max_DB_Server = atoi(conf->value);
@@ -415,22 +401,22 @@ pglb_exit(int signal_args)
 	int rtn;
 	
 	Child_Tbl->useFlag = TBL_END;
-	PGRsignal(SIGCHLD,SIG_IGN);
-	PGRsignal(signal_args,SIG_IGN);
+	signal(SIGCHLD,SIG_IGN);
+	signal(signal_args,SIG_IGN);
 	kill (0,signal_args);
 	while (wait(NULL) > 0 )
 		;
 
 	if (ClusterShmid > 0)
 	{
-		rtn = shmdt((char *)Cluster_Tbl);
+		rtn = shmdt(Cluster_Tbl);
 		shmctl(ClusterShmid,IPC_RMID,(struct shmid_ds *)NULL);
 		ClusterShmid = 0;
 		Cluster_Tbl = NULL;
 	}
 	if (ChildShmid > 0)
 	{
-		rtn = shmdt((char *)Child_Tbl);
+		rtn = shmdt(Child_Tbl);
 		shmctl(ChildShmid,IPC_RMID,(struct shmid_ds *)NULL);
 		ChildShmid = 0;
 		Child_Tbl = NULL;
@@ -442,15 +428,15 @@ pglb_exit(int signal_args)
 	snprintf(fname, sizeof(fname), "%s/%s", PGR_Write_Path, PGLB_PID_FILE);
 	unlink(fname);
 	
-	if (StatusFp != NULL)
+	if (Com_Info.Log_Info.StatusFp != NULL)
 	{
-		fflush(StatusFp);
-		fclose(StatusFp);
+		fflush(Com_Info.Log_Info.StatusFp);
+		fclose(Com_Info.Log_Info.StatusFp);
 	}
-	if (LogFp != NULL)
+	if (Com_Info.Log_Info.LogFp != NULL)
 	{
-		fflush(LogFp);
-		fclose(LogFp);
+		fflush(Com_Info.Log_Info.LogFp);
+		fclose(Com_Info.Log_Info.LogFp);
 	}
 	if (Frontend_FD.unix_fd != 0)
 	{
@@ -462,9 +448,7 @@ pglb_exit(int signal_args)
 	{
 		close(Frontend_FD.inet_fd);
 	}
-	/*
 	PGRsyn_quit();
-	*/
 	snprintf(fname, sizeof(fname), "%s/%s", PGR_Write_Path, PGLB_PID_FILE);
 	unlink(fname);
 
@@ -504,7 +488,7 @@ load_balance_main(void)
 	}
 	if (Use_Connection_Pool)
 	{
-		PGRsignal(SIGCHLD,PGRrecreate_child);
+		signal(SIGCHLD,PGRrecreate_child);
 		rtn = PGRpre_fork_children(Cluster_Tbl);
 		if (rtn != STATUS_OK)
 		{
@@ -529,14 +513,8 @@ load_balance_main(void)
 		if(Frontend_FD.inet_fd)
 			FD_SET(Frontend_FD.inet_fd,&rmask);
 		rtn = select(Max(Frontend_FD.unix_fd, Frontend_FD.inet_fd) + 1, &rmask, (fd_set *)NULL, (fd_set *)NULL, &timeout);
-		if( rtn > 0)
+		if (rtn > 0)
 		{
-		  if(PGRis_cluster_alive() == STATUS_ERROR) {
-		    show_error("%s:all clusters were dead.",func);				    PGRreturn_no_connection_error();        
-		    count=0;
-		  }
-		  else 
-		    {
 			if (Use_Connection_Pool)
 			{
 				status = PGRload_balance_with_pool();
@@ -560,7 +538,6 @@ load_balance_main(void)
 			{
 				count = 0;
 			}
-		    }
 		}
 	}
 }
@@ -744,9 +721,6 @@ PGRrecreate_child(int signal_args)
 	int status;
 	ClusterTbl * cluster_p;
 
-ReWait:
-
-	errno = 0;
 #ifdef HAVE_WAITPID
 	while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
 	{
@@ -757,8 +731,6 @@ ReWait:
 		cluster_p = scan_cluster_by_pid(pid);
 		pid = PGRcreate_child(cluster_p);	
 	}
-	if ((pid < 0) && (errno == EINTR))
-		goto ReWait;
 }
 
 /*--------------------------------------------------------------------
@@ -786,6 +758,7 @@ close_child(int signal_args)
 		return ;
 	}
 	cluster = Cluster_Tbl;
+	signal(SIGCHLD,SIG_IGN);
 	while(cluster->useFlag != TBL_END)
 	{
 		if (cluster->useFlag == TBL_ERROR_NOTICE) 
@@ -803,7 +776,7 @@ close_child(int signal_args)
 	child = Child_Tbl;
 	while(child->useFlag != TBL_END)
 	{
-		if (child->rec_no == rec_no) 
+		if ((child->rec_no == rec_no) && (child->pid > 0)) 
 		{
 			if (kill (child->pid,SIGTERM) == -1)
 			{
@@ -812,10 +785,10 @@ close_child(int signal_args)
 			}
 			PGRchild_wait(signal_args);
 			child->useFlag = DATA_FREE;
+			child->pid = 0;
 		}
 		child++;
 	}
-	PGRsignal(SIGUSR2, close_child);
 }
 
 /*--------------------------------------------------------------------
@@ -899,14 +872,13 @@ usage(void)
 		path = ".";
 	fprintf(stderr,"pglb version [%s]\n",PGLB_VERSION);
 	fprintf(stderr,"A load balancer for PostgreSQL\n\n");
-	fprintf(stderr,"usage: pglb [-D path_of_config_file] [-W path_of_work_files] [-n][-v][-h][stop | restart]\n");
+	fprintf(stderr,"usage: pglb [-D path_of_config_file] [-W path_of_work_files] [-n][-v][-h][stop]\n");
 	fprintf(stderr,"    config file default path: %s/%s\n",path, PGLB_CONF_FILE);
 	fprintf(stderr,"    -l: print error logs in the log file.\n");
 	fprintf(stderr,"    -n: don't run in daemon mode.\n");
 	fprintf(stderr,"    -v: debug mode. need '-n' flag\n");
 	fprintf(stderr,"    -h: print this help\n");
 	fprintf(stderr,"    stop: stop pglb\n");
-	fprintf(stderr,"    restart: restart pglb\n");
 }
 
 /*--------------------------------------------------------------------
@@ -930,17 +902,20 @@ main(int argc, char ** argv)
 	char * w_path = NULL;
 	int detach = 1;
 
-	PGRsignal(SIGHUP, pglb_exit);
-	PGRsignal(SIGINT, pglb_exit);	
-	PGRsignal(SIGQUIT, pglb_exit);
-	PGRsignal(SIGTERM, pglb_exit);
-	PGRsignal(SIGALRM, SIG_IGN); /* ignored */
-	PGRsignal(SIGPIPE, SIG_IGN); /* ignored */
-	PGRsignal(SIGTTIN, SIG_IGN); /* ignored */
-	PGRsignal(SIGTTOU, SIG_IGN); /* ignored */
-	PGRsignal(SIGCHLD,PGRchild_wait);
-	PGRsignal(SIGUSR1, SIG_IGN); /* ignored */
-	PGRsignal(SIGUSR2, close_child); /* close child process */
+	/* init global variables */
+	PGR_Init_Com_Info(&Com_Info);
+
+	signal(SIGHUP, pglb_exit);
+	signal(SIGINT, pglb_exit);	
+	signal(SIGQUIT, pglb_exit);
+	signal(SIGTERM, pglb_exit);
+	signal(SIGALRM, SIG_IGN); /* ignored */
+	signal(SIGPIPE, SIG_IGN); /* ignored */
+	signal(SIGTTIN, SIG_IGN); /* ignored */
+	signal(SIGTTOU, SIG_IGN); /* ignored */
+	signal(SIGCHLD,PGRchild_wait);
+	signal(SIGUSR1, SIG_IGN); /* ignored */
+	signal(SIGUSR2, close_child); /* close child process */
 	r_path = getenv("PGDATA");
 	if (r_path == NULL)
 		r_path = ".";
@@ -966,10 +941,10 @@ main(int argc, char ** argv)
 				w_path = optarg;
 				break;
 			case 'l':
-				Log_Print = 1;
+				Com_Info.Log_Info.Log_Print = 1;
 				break;
 			case 'v':
-				Debug_Print = 1;
+				Com_Info.Log_Info.Debug_Print = 1;
 				break;
 			case 'n':
 				detach = 0;
@@ -993,15 +968,10 @@ main(int argc, char ** argv)
 		PGR_Write_Path = w_path;
 	}
 
-	if (optind == (argc-1) &&
-			((!strcmp(argv[optind],"stop")) ||
-			(!strcmp(argv[optind],"restart"))))
+	if (optind == (argc-1) && !strcmp(argv[optind],"stop"))
 	{
 		stop_pglb();
-		if (!strcmp(argv[optind],"stop"))
-		{
-			exit(0);
-		}
+		exit(0);
 	}
 	else if (optind == argc)
 	{
@@ -1047,4 +1017,6 @@ main(int argc, char ** argv)
 	pglb_exit(0);
 	return STATUS_OK;
 }
-
+#else
+void main(){}
+#endif /* USE_REPLICATION */
