@@ -22,6 +22,10 @@
  * is" without express or implied warranty.
  *
 */
+#ifdef USE_REPLICATION
+
+#include "postgres.h"
+
 #include <sys/types.h>
 #include <netinet/in.h>
 #include <sys/param.h>
@@ -956,3 +960,5 @@ signed char pool_read_kind(POOL_CONNECTION_POOL *cp)
 
 	return kind;
 }
+
+#endif /* USE_REPLICATION */
