@@ -6,7 +6,7 @@
  *     This file is composed of the functions to call with the source
  *     at pgreplicate for backup and cascade .
  *
- * Portions Copyright (c) 2003-2006, Atsushi Mitani
+ * Portions Copyright (c) 2003, Atsushi Mitani
  *--------------------------------------------------------------------
  */
 #ifdef USE_REPLICATION
@@ -51,32 +51,32 @@
 #include "libpq-fe.h"
 #include "libpq-int.h"
 #include "fe-auth.h"
-
 #include "access/xact.h"
 #include "replicate_com.h"
 #include "pgreplicate.h"
 
+int PGRstartup_cascade(void);
+int PGRsend_lower_cascade(ReplicateHeader * header, char * query);
+int PGRsend_upper_cascade(ReplicateHeader * header, char * query);
+int PGRwait_answer_cascade(int  sock);
+ReplicateServerInfo * PGRget_lower_cascade(void);
+ReplicateServerInfo * PGRget_upper_cascade(void);
+void PGRset_cascade_server_status(ReplicateServerInfo * cascade, int status);
+ReplicateServerInfo * PGRrecv_cascade_answer(ReplicateServerInfo * cascade,ReplicateHeader * header);
+int PGRsend_cascade(int sock , ReplicateHeader * header, char * query);
+int PGRcascade_main(int sock, ReplicateHeader * header, char * query);
+int PGRwait_notice_rlog_done(void);
+int PGRsend_notice_rlog_done(int sock);
+
 #if 0
 static int count_cascade(int flag);
 #endif
-
-static int fixup_socket_for_cascades(int *sock ,ReplicateServerInfo * target);
 static ReplicateServerInfo * get_cascade_data(int * cnt, int flag);
 static int add_cascade_data(ReplicateHeader * header, ReplicateServerInfo * add_data);
 static int update_cascade_data(ReplicateHeader * header, ReplicateServerInfo * update_data);
 static void write_cascade_status_file(ReplicateServerInfo * cascade);
 static int notice_cascade_data(int sock);
 static int notice_cascade_data_to_cluster_db(void);
-static void PGRinit_cascade_child(void); 
-
-/**
- * socket variables, moved from  Cascade_Inf->(lower|upper)->sock.
- * Cascade->Inf is in shared memory, so sometimes cascades returns EBADF due to not initialized socket in specified process.
- * 05/10/05 tanida@sraoss.co.jp
- */
-
-static int lsock=-1; /* socket for lower-cascade. */
-static int usock=-1; /* socket for upper-cascade. */
 
 /*--------------------------------------
  * PROTOTYPE DECLARATION
@@ -112,9 +112,9 @@ count_cascade(int flag)
 	{
 		return 0;
 	}
-	while (cascade->useFlag != DB_TBL_END)
+	while (cascade->useFlag != DATA_END)
 	{
-		if (cascade->useFlag == DB_TBL_USE)
+		if (cascade->useFlag == DATA_USE)
 		{
 			cascade_cnt ++;
 		}
@@ -134,12 +134,6 @@ count_cascade(int flag)
 }
 #endif /* if 0 */
 
-static void 
-PGRinit_cascade_child(void) {
-      fixup_socket_for_cascades(&usock,NULL);
-      fixup_socket_for_cascades(&lsock,NULL);
-}
-
 static ReplicateServerInfo * 
 get_cascade_data(int * cnt, int flag)
 {
@@ -169,11 +163,6 @@ get_cascade_data(int * cnt, int flag)
 		case LOWER_CASCADE:
 			cascade = Cascade_Inf->myself;
 			break;
-			default:
-		free(buf);
-		*cnt = 0;
-		return NULL;
-					
 	}
 
 	if (cascade == NULL)
@@ -182,12 +171,11 @@ get_cascade_data(int * cnt, int flag)
 		*cnt = 0;
 		return NULL;
 	}
-	PGRsem_lock(CascadeSemID,1);
 	i = 0;
 	loop_cnt = 0;
-	while (cascade->useFlag != DB_TBL_END)
+	while (cascade->useFlag != DATA_END)
 	{
-		if (cascade->useFlag == DB_TBL_USE) 
+		if (cascade->useFlag == DATA_USE) 
 		{
 			(buf + i)->useFlag = htonl(cascade->useFlag);
 			strncpy((buf + i)->hostName,cascade->hostName,sizeof(cascade->hostName));
@@ -213,8 +201,6 @@ get_cascade_data(int * cnt, int flag)
 		cascade ++;
 	}
 	*cnt = i;
-	PGRsem_unlock(CascadeSemID,1);
-
 	return buf;
 }
 
@@ -228,8 +214,6 @@ update_cascade_data(ReplicateHeader * header, ReplicateServerInfo * update_data)
 	ReplicateServerInfo * cascade = NULL;
 	char hostName[HOSTNAME_MAX_LENGTH];
 
-
-	show_debug("executing %s",func);
 	if ((header == NULL ) || ( update_data == NULL))
 	{
 		show_error("%s:receive data is wrong",func);
@@ -240,8 +224,6 @@ update_cascade_data(ReplicateHeader * header, ReplicateServerInfo * update_data)
 		show_error("%s:config data read error",func);
 		return STATUS_ERROR;
 	}
-
-
 	size = ntohl(header->query_size);
 	cnt = size / sizeof(ReplicateServerInfo);
 	if (cnt >= MAX_DB_SERVER)
@@ -250,10 +232,16 @@ update_cascade_data(ReplicateHeader * header, ReplicateServerInfo * update_data)
 		return STATUS_ERROR;
 	}
 
-	Cascade_Inf->useFlag = DB_TBL_INIT;
-	fixup_socket_for_cascades(&usock,NULL);
-	fixup_socket_for_cascades(&lsock,NULL);
-
+	Cascade_Inf->useFlag = DATA_INIT;
+	if ((Cascade_Inf->upper != NULL) && (Cascade_Inf->upper->sock > 0))
+	{
+		close(Cascade_Inf->upper->sock);
+	}
+	if ((Cascade_Inf->lower != NULL) && (Cascade_Inf->lower->sock > 0))
+	{
+		close(Cascade_Inf->lower->sock);
+	}
+	Cascade_Inf->myself = NULL;
 	Cascade_Inf->upper = NULL;
 	Cascade_Inf->lower = NULL;
 
@@ -264,7 +252,6 @@ update_cascade_data(ReplicateHeader * header, ReplicateServerInfo * update_data)
 	Cascade_Inf->top = cascade;
 	while (cnt > 0)
 	{
-
 		cascade->useFlag = ntohl(ptr->useFlag);
 		strncpy(cascade->hostName,ptr->hostName,sizeof(cascade->hostName));
 		cascade->portNumber = ntohs(ptr->portNumber);
@@ -277,14 +264,13 @@ update_cascade_data(ReplicateHeader * header, ReplicateServerInfo * update_data)
 		{
 			Cascade_Inf->myself = cascade;
 		}
-
 		Cascade_Inf->end = cascade;
 		cascade ++;
 		ptr ++;
 		cnt --;
-		cascade->useFlag = DB_TBL_END;
+		cascade->useFlag = DATA_END;
 	}
-	Cascade_Inf->useFlag = DB_TBL_USE;
+	Cascade_Inf->useFlag = DATA_USE;
 
 	return STATUS_OK;
 }
@@ -317,8 +303,11 @@ add_cascade_data(ReplicateHeader * header, ReplicateServerInfo * add_data)
 		return STATUS_ERROR;
 	}
 
-	Cascade_Inf->useFlag = DB_TBL_INIT;
-	fixup_socket_for_cascades(&lsock,NULL);
+	Cascade_Inf->useFlag = DATA_INIT;
+	if ((Cascade_Inf->lower != NULL) && (Cascade_Inf->lower->sock > 0))
+	{
+		close(Cascade_Inf->lower->sock);
+	}
 	Cascade_Inf->lower = NULL;
 
 	gethostname(hostName,sizeof(hostName));
@@ -332,10 +321,6 @@ add_cascade_data(ReplicateHeader * header, ReplicateServerInfo * add_data)
 		cascade->portNumber = ntohs(ptr->portNumber);
 		cascade->recoveryPortNumber = ntohs(ptr->recoveryPortNumber);
 		cascade->lifecheckPortNumber = ntohs(ptr->lifecheckPortNumber);
-                cascade->replicate_id=-1;
-	        cascade->response_mode=-1;
-
-		Cascade_Inf->end = cascade;
 
 		if ((!strncmp(cascade->hostName,hostName,sizeof(cascade->hostName)))  &&
 			(cascade->portNumber == Port_Number) &&
@@ -346,11 +331,12 @@ add_cascade_data(ReplicateHeader * header, ReplicateServerInfo * add_data)
 			continue;
 		}
 		cascade ++;
-		cascade->useFlag = DB_TBL_END;
+		cascade->useFlag = DATA_END;
 		ptr ++;
 		cnt --;
 	}
-	Cascade_Inf->useFlag = DB_TBL_USE;
+	Cascade_Inf->useFlag = DATA_USE;
+
 	return STATUS_OK;
 }
 
@@ -383,7 +369,7 @@ PGRstartup_cascade(void)
 		show_error("%s:cascade data get error",func);
 		return STATUS_ERROR;
 	}
-
+	
 	memset(&header,0,sizeof(ReplicateHeader));
 	header.cmdSys = CMD_SYS_CASCADE;
 	header.cmdSts = CMD_STS_TO_UPPER;
@@ -399,103 +385,147 @@ PGRstartup_cascade(void)
 	{
 		memset(&header,0,sizeof(ReplicateHeader));
 		buf = PGRrecv_cascade_answer( Cascade_Inf->upper, &header);
-		if (buf == NULL)
+		if ((buf != NULL) &&
+			(header.cmdSys == CMD_SYS_CASCADE) &&
+			(header.cmdSts == CMD_STS_TO_LOWER) &&
+			(header.cmdType == CMD_TYPE_UPDATE_ALL))
 		{
-				status=STATUS_ERROR;
+			status = update_cascade_data(&header,buf);
+			free(buf);
 		}
-		else if((header.cmdSys == CMD_SYS_CASCADE) &&
-		    (header.cmdSts == CMD_STS_TO_LOWER) &&
-		    (header.cmdType == CMD_TYPE_UPDATE_ALL))
-		{
-				status = update_cascade_data(&header,buf);
-				free(buf);
-		}
-		
 	}
-	show_debug("%s:startup packet result is %d",func,status);
+	
 	return status;
 }
 
 int
 PGRsend_lower_cascade(ReplicateHeader * header, char * query)
 {
+	char * func = "PGRsend_lower_cascade()";
+	int status = STATUS_OK;
 
-
-		char * func = "PGRsend_lower_cascade()";
-		ReplicateServerInfo *lower = PGRget_lower_cascade();
-
-
-		while(lower!=NULL) 
+	for(;;)
+	{
+		if (Cascade_Inf->lower != NULL)
 		{
-				/**
-				 * check lower_cascade validaty.
-				 *
-				 */	       		
-				if(lsock!=-1 &&			   
-				   PGRsend_cascade(lsock,header,query)==STATUS_OK) 
+			if (Cascade_Inf->lower->sock <= 0)
+			{
+#ifdef PRINT_DEBUG
+				show_debug("%s:sock is not ready, Cascade_Inf->lower set NULL",func);
+#endif			
+				status = PGR_Create_Socket_Connect(&(Cascade_Inf->lower->sock),Cascade_Inf->lower->hostName,Cascade_Inf->lower->portNumber);
+		
+				if (status != STATUS_OK)
 				{
-						return STATUS_OK;
+					PGRset_cascade_server_status(Cascade_Inf->lower,DATA_ERR);
+					Cascade_Inf->lower = NULL;
 				}
-				else
-				{
-						/**
-						 * current lower cascade is missing.
-						 * fix socket , or go to next one. 
-						 *
-						 */
-						while(  lower!=NULL &&
-								fixup_socket_for_cascades(&lsock,lower)!=STATUS_OK)
-						{
-								show_error("%s:lower cascade maybe down,challenge new one.",func);
-								PGRset_cascade_server_status(lower,DB_TBL_ERROR);
-								lower =PGRget_lower_cascade();
-						}
-				}
-				Cascade_Inf->lower=lower;
-        }
-
-
-		return STATUS_ERROR;
+			}
+		}
+		while (Cascade_Inf->lower == NULL)
+		{
+			Cascade_Inf->lower = PGRget_lower_cascade();	
+			if (Cascade_Inf->lower == NULL)
+			{
+#ifdef PRINT_DEBUG
+				show_debug("%s:there is no lower cascaded server",func);
+#endif			
+				return STATUS_ERROR;
+			}
+			status = PGR_Create_Socket_Connect(&(Cascade_Inf->lower->sock),Cascade_Inf->lower->hostName,Cascade_Inf->lower->portNumber);
+	
+			if (status != STATUS_OK)
+			{
+#ifdef PRINT_DEBUG
+				show_debug("%s:PGRset_cascade_server_status call status ERROR",func);
+#endif			
+				PGRset_cascade_server_status(Cascade_Inf->lower,DATA_ERR);
+				Cascade_Inf->lower = NULL;
+			}
+			else
+			{
+				break;
+			}
+		}
+		if (PGRsend_cascade(Cascade_Inf->lower->sock,header,query) != STATUS_OK)
+		{
+			show_error("%s:PGRsend_cascade failed",func);
+			if (Cascade_Inf->lower->sock > 0)
+			{
+				close(Cascade_Inf->lower->sock);
+				Cascade_Inf->lower->sock = -1;
+			}
+			PGRset_cascade_server_status(Cascade_Inf->lower,DATA_ERR);
+			Cascade_Inf->lower = NULL;
+		}
+		if (Cascade_Inf->lower != NULL)
+		{
+			break;
+		}
+	}
+	return STATUS_OK;
 }
 
-
 int
 PGRsend_upper_cascade(ReplicateHeader * header, char * query)
 {
 	char * func = "PGRsend_upper_cascade()";
-	ReplicateServerInfo *upper = PGRget_upper_cascade();
+	int status = STATUS_OK;
+	int sock;
 
-
-	while(upper!=NULL)
-	{				
-			/**
-			 * check upper_cascade validaty.
-			 *
-			 */	       		
-			if(usock!=-1 && 
-			   PGRsend_cascade(usock,header,query)==STATUS_OK) 
+	for(;;)
+	{
+		if (Cascade_Inf->upper != NULL)
+		{
+			if (Cascade_Inf->upper->sock <= 0)
 			{
-					return STATUS_OK;
+				Cascade_Inf->upper = NULL;
+			}
+		}
+		while (Cascade_Inf->upper == NULL)
+		{
+			Cascade_Inf->upper = PGRget_upper_cascade();	
+			if (Cascade_Inf->upper == NULL)
+			{
+				show_debug("Cascade_Inf->upper is NULL");
+				return STATUS_ERROR;
+			}
+#ifdef PRINT_DEBUG
+			show_debug("%s:sock[%d] host[%s] port[%d]", func,sock,Cascade_Inf->upper->hostName,Cascade_Inf->upper->portNumber);
+#endif			
+			status = PGR_Create_Socket_Connect(&sock,Cascade_Inf->upper->hostName,Cascade_Inf->upper->portNumber);
+	
+			if (status != STATUS_OK)
+			{
+#ifdef PRINT_DEBUG
+				show_debug("%s:PGR_Create_Socket_Connect failed",func);
+#endif			
+				PGRset_cascade_server_status(Cascade_Inf->upper,DATA_ERR);
+				Cascade_Inf->upper = NULL;
 			}
 			else
 			{
-					/**
-					 * current upper cascade is missing.
-					 * fix socket , or go to next one. 
-					 *
-					 */
-					while(  upper!=NULL &&
-							fixup_socket_for_cascades(&usock,upper)!=STATUS_OK) 
-					{
-							show_error("%s:upper cascade maybe down,challenge new one.",func);
-							PGRset_cascade_server_status(upper,DB_TBL_ERROR);
-							upper =PGRget_upper_cascade();
-					}
+#ifdef PRINT_DEBUG
+				show_debug("%s:PGR_Create_Socket_Connect OK[%d]",func,sock);
+#endif			
+				Cascade_Inf->upper->sock = sock;
+				break;
 			}
-			Cascade_Inf->upper=upper;
+		}
+		if (PGRsend_cascade(Cascade_Inf->upper->sock,header,query) != STATUS_OK)
+		{
+#ifdef PRINT_DEBUG
+			show_debug("%s:PGRsend_cascade failed",func);
+#endif			
+			PGRset_cascade_server_status(Cascade_Inf->upper,DATA_ERR);
+			Cascade_Inf->upper = NULL;
+		}
+		if (Cascade_Inf->upper != NULL)
+		{
+			break;
+		}
 	}
-
-	return STATUS_ERROR;
+	return STATUS_OK;
 }
 
 ReplicateServerInfo *
@@ -511,18 +541,17 @@ PGRget_lower_cascade(void)
 	}
 
 	/* count lower server */
-
 	cascade = Cascade_Inf->myself;
 	if (cascade == NULL)
 	{
 		show_error("%s:cascade data initialize error",func);
 		return NULL;
 	}
-	if (cascade->useFlag != DB_TBL_END)
+	if (cascade->useFlag != DATA_END)
 	{
 		cascade ++;
 	}
-	while (cascade->useFlag != DB_TBL_END)
+	while (cascade->useFlag != DATA_END)
 	{
 #ifdef PRINT_DEBUG
 		show_debug("%s:lower cascade search[%d]@[%s] use[%d]",
@@ -531,7 +560,7 @@ PGRget_lower_cascade(void)
 			cascade->hostName,
 			cascade->useFlag);
 #endif			
-		if (cascade->useFlag == DB_TBL_USE)
+		if (cascade->useFlag == DATA_USE)
 		{
 #ifdef PRINT_DEBUG
 			show_debug("%s:find lower cascade",func);
@@ -555,7 +584,6 @@ PGRget_upper_cascade(void)
 		return NULL;
 	}
 
-
 	/* count lower server */
 	cascade = Cascade_Inf->myself;
 	if ((cascade == NULL) || (Cascade_Inf->top == cascade))
@@ -565,7 +593,7 @@ PGRget_upper_cascade(void)
 	cascade --;
 	while (cascade != NULL)
 	{
-		if (cascade->useFlag == DB_TBL_USE)
+		if (cascade->useFlag == DATA_USE)
 		{
 			return cascade;
 		}
@@ -583,28 +611,28 @@ write_cascade_status_file(ReplicateServerInfo * cascade)
 {
 	switch( cascade->useFlag)
 	{
-		case DB_TBL_FREE:
-			PGRwrite_log_file(StatusFp,"cascade(%s) port(%d) free",
+		case DATA_FREE:
+			PGRwrite_log_file(Com_Info.Log_Info.StatusFp,"cascade(%s) port(%d) free",
 					cascade->hostName,
 					cascade->portNumber);
 			break;
-		case DB_TBL_INIT:
-			PGRwrite_log_file(StatusFp,"cascade(%s) port(%d) initialize",
+		case DATA_INIT:
+			PGRwrite_log_file(Com_Info.Log_Info.StatusFp,"cascade(%s) port(%d) initialize",
 					cascade->hostName,
 					cascade->portNumber);
 			break;
-		case DB_TBL_USE:
-			PGRwrite_log_file(StatusFp,"cascade(%s) port(%d) start use",
+		case DATA_USE:
+			PGRwrite_log_file(Com_Info.Log_Info.StatusFp,"cascade(%s) port(%d) start use",
 					cascade->hostName,
 					cascade->portNumber);
 			break;
-		case DB_TBL_ERROR:
-			PGRwrite_log_file(StatusFp,"cascade(%s) port(%d) error",
+		case DATA_ERR:
+			PGRwrite_log_file(Com_Info.Log_Info.StatusFp,"cascade(%s) port(%d) error",
 					cascade->hostName,
 					cascade->portNumber);
 			break;
-		case DB_TBL_TOP:
-			PGRwrite_log_file(StatusFp,"cascade(%s) port(%d) become top",
+		case DATA_TOP:
+			PGRwrite_log_file(Com_Info.Log_Info.StatusFp,"cascade(%s) port(%d) become top",
 					cascade->hostName,
 					cascade->portNumber);
 			break;
@@ -629,28 +657,12 @@ ReplicateServerInfo *
 PGRrecv_cascade_answer(ReplicateServerInfo * cascade,ReplicateHeader * header)
 {
 	ReplicateServerInfo * answer = NULL;
-        int sock;
 
 	if ((cascade == NULL) || (header == NULL))
 	{
 		return NULL;
 	}
-
-	/* FIXME: ReplicateServerInfo->sock must be removed in cascading. */
-	if(cascade == Cascade_Inf->upper ) 
-	{
-	  sock=usock;
-	}
-	else if (cascade == Cascade_Inf->lower )
-	{
-	  sock=lsock;
-	}
-	else 
-	{
-	  show_debug("PGRrecv_cascade_answer:receiving packet from sock not belogs to cascade->upper / lower. maybe missing .");
-	  sock=cascade->sock;
-	}
-	answer = (ReplicateServerInfo*)PGRread_packet(sock,header);
+	answer = (ReplicateServerInfo*)PGRread_packet(cascade->sock,header);
 	return answer;
 }
 
@@ -670,8 +682,9 @@ PGRsend_cascade(int sock , ReplicateHeader * header, char * query)
 	int query_size = 0;
 
 	/* check parameter */
-	if ((header == NULL) || (sock == -1))
+	if ((header == NULL) || (sock <= 0))
 	{
+		show_error("%s:header or sock is null",func);
 		return STATUS_ERROR;
 	}
 
@@ -691,45 +704,31 @@ PGRsend_cascade(int sock , ReplicateHeader * header, char * query)
 	}
 	send_ptr = buf;
 
-	for (;;)
+	timeout.tv_sec = 10;
+	timeout.tv_usec = 0;
+
+	/*
+	 * Wait for something to happen.
+	 */
+	FD_ZERO(&wmask);
+	FD_SET(sock,&wmask);
+	rtn = select(sock+1, (fd_set *)NULL, &wmask, (fd_set *)NULL, &timeout);
+	if (rtn && FD_ISSET(sock, &wmask))
 	{
-		timeout.tv_sec = 10;
-		timeout.tv_usec = 0;
-
-		/*
-		 * Wait for something to happen.
-		 */
-		FD_ZERO(&wmask);
-		FD_SET(sock,&wmask);
-		rtn = select(sock+1, (fd_set *)NULL, &wmask, (fd_set *)NULL, &timeout);
-
-		if (rtn < 0)
-		{
-			if (errno == EINTR || errno == EAGAIN)
-				continue;
-
-			show_error("%s:select failed ,errno is %s",func , strerror(errno));
-			free(buf);
-			return STATUS_ERROR;
-		}
-
-		if (rtn && FD_ISSET(sock, &wmask))
+		for (;;)
 		{
 			s = send(sock,send_ptr + send_size,buf_size - send_size ,0);
-			if (s < 0)
-			{
-				if (errno == EINTR || errno == EAGAIN)
-					continue;
-				else
+			if (s < 0){
+				if (errno == EINTR)
 				{
-					show_error("%s:send failed: %d(%s)",func, errno, strerror(errno));
-					free(buf);
-					return STATUS_ERROR;
+					continue;
 				}
+				show_error("%s:send failed",func);
+				free(buf);
+				return STATUS_ERROR;
 			}
-			else if (s == 0)
+			if (s == 0)
 			{
-				show_error("%s:unexpected EOF", func);
 				free(buf);
 				return STATUS_ERROR;
 			}
@@ -761,27 +760,6 @@ PGRwait_answer_cascade(int  sock)
 	}
 	return STATUS_ERROR;
 }
-/**
- * fixup_socket_for_cascades checks socket's validaty.
- * returns STATUS_OK if succeeded , or STATUS_ERROR if some error occured.
- * if target is null , only close socket.
- *
- * originally written by tanida@sraoss.co.jp
- */
-static int
-fixup_socket_for_cascades(int *sock, ReplicateServerInfo *target) 
-{
-	if (*sock > 0)
-	{
-		close(*sock);	
-		*sock=-1;
-	}
-        if(target!=NULL) {
-	       return PGR_Create_Socket_Connect(sock,target->hostName,target->portNumber);
-        }
-	return STATUS_OK;
-}
-
 
 static int
 notice_cascade_data(int sock)
@@ -850,7 +828,6 @@ static int
 notice_cascade_data_to_cluster_db(void)
 {
 	char userName[USERNAME_MAX_LENGTH];
-	ReplicateServerInfo *s=NULL;
 
 	if (Cascade_Inf->lower == NULL)
 	{
@@ -860,15 +837,10 @@ notice_cascade_data_to_cluster_db(void)
 	{
 		return STATUS_ERROR;
 	}
-	s=Cascade_Inf->lower;
 	memset(userName,0,sizeof(userName));
 	strncpy(userName ,getenv("LOGNAME"),sizeof(userName)-1);
 
-	PGRnotice_replication_server(s->hostName,
-								 s->portNumber,
-								 s->recoveryPortNumber,
-								 s->lifecheckPortNumber,
-								 userName);
+	PGRnotice_replication_server(Cascade_Inf->lower->hostName, Cascade_Inf->lower->portNumber, Cascade_Inf->lower->recoveryPortNumber,Cascade_Inf->lower->lifecheckPortNumber,userName);
 
 	return STATUS_OK;
 }
@@ -876,32 +848,16 @@ notice_cascade_data_to_cluster_db(void)
 int
 PGRwait_notice_rlog_done(void)
 {
-        ReplicateHeader header;
-		if (lsock != -1)
+
+	if (Cascade_Inf->lower != NULL)
+	{
+		if (Cascade_Inf->lower->sock > 0)
 		{
-				PGRread_packet(lsock,&header);
-				return STATUS_OK;
+			PGRreturn_result(Cascade_Inf->lower->sock,PGR_WAIT_ANSWER);
+			return STATUS_OK;
 		}
-		return STATUS_ERROR;
-
-}
-
-
-int
-PGRsend_notice_quit(void )
-{
-		ReplicateHeader header;
-		int size = 0;
-
-		size = strlen("QUIT_SAFELY");
-		memset(&header,0,sizeof(ReplicateHeader));
-		header.cmdSys = CMD_SYS_CALL ;
-		header.cmdSts = CMD_STS_RESPONSE ;
-		header.cmdType = CMD_TYPE_FRONTEND_CLOSED;
-		header.query_size = htonl(size);
-		PGRsend_lower_cascade(&header, "QUIT_SAFELY");
-		PGRwait_notice_rlog_done();
-		return STATUS_OK;
+	}
+	return STATUS_ERROR;
 }
 
 int
