@@ -6,9 +6,11 @@
  * NOTE:
  *    Read and set configuration data in this modul.
  *
- * Portions Copyright (c) 2003-2006, Atsushi Mitani
+ * Portions Copyright (c) 2003, Atsushi Mitani
  *--------------------------------------------------------------------
  */
+#ifdef USE_REPLICATION
+
 #include "postgres.h"
 
 #include <stdio.h>
@@ -25,15 +27,12 @@
 #include <errno.h>
 #include <sys/file.h>
 
-
-
 #include "libpq-fe.h"
-#include "libpq-int.h"
-#include "fe-auth.h"
-
 #include "replicate_com.h"
 #include "pgreplicate.h"
 
+int PGRset_Conf_Data(char * path);
+
 /*--------------------------------------------------------------------
  * SYMBOL
  *    PGRset_Conf_Data()
@@ -49,7 +48,7 @@
 int
 PGRset_Conf_Data(char * path)
 {
-	char * func = "PGRset_Conf_Data()";
+	char * func = "init_server_tbl()";
 	HostTbl host_tbl[MAX_DB_SERVER];
 	ConfDataType * conf = NULL;
 	int cnt = 0;
@@ -71,29 +70,31 @@ PGRset_Conf_Data(char * path)
 		path = ".";
 	}
 
-	if (Log_Print)
+	if (Com_Info.Log_Info.Log_Print)
 	{
 		snprintf(fname,sizeof(fname),"%s/%s",path,PGREPLICATE_LOG_FILE);
-		LogFp = fopen(fname,"a");
-		if (LogFp == NULL)
+		Com_Info.Log_Info.LogFp = fopen(fname,"a");
+		if (Com_Info.Log_Info.LogFp == NULL)
 		{
 			show_error("%s:fopen failed: (%s)",func,strerror(errno));
 			return STATUS_ERROR;
 		}
-		show_debug("%s open ok\n",fname);
-#ifdef PRINT_DEBUG
-
-#endif			
 	}
 
 	snprintf(fname,sizeof(fname),"%s/%s",path,PGREPLICATE_STATUS_FILE);
-	StatusFp = fopen(fname,"a");
-	if (StatusFp == NULL)
+	Com_Info.Log_Info.StatusFp = fopen(fname,"a");
+	if (Com_Info.Log_Info.StatusFp == NULL)
 	{
 		show_error("%s:fopen failed: (%s)",func,strerror(errno));
 		return STATUS_ERROR;
 	}
 
+	/*
+	 * Recovery Queue structure initialize
+	 */
+	RecoveryQueue.queue_fp = (FILE *)NULL;
+	RecoveryQueue.current_queue_no = 0;
+
 	/*
 	 * read configuration file
 	 */
@@ -102,9 +103,6 @@ PGRset_Conf_Data(char * path)
 		show_error("%s:PGR_Get_Conf_Data failed",func);
 		return STATUS_ERROR;
 	}
-#ifdef PRINT_DEBUG
-	show_debug("PGR_Get_Conf_Data ok");
-#endif			
 
 	/* allocate response information table */
 	PGR_Response_Inf = (ResponseInf *)malloc(sizeof(ResponseInf));
@@ -119,15 +117,25 @@ PGRset_Conf_Data(char * path)
 	/*
 	 * memory allocate load balance table buffer
 	 */
-	LoadBalanceTbl = (RecoveryTbl *)malloc(sizeof(RecoveryTbl)*MAX_DB_SERVER);
+	size = sizeof(RecoveryTbl) * MAX_DB_SERVER;
+	LoadBalanceTblShmid = shmget(IPC_PRIVATE,size,IPC_CREAT | IPC_EXCL | 0600);
+	if (LoadBalanceTblShmid < 0)
+	{
+		show_error("shmget() failed. reason: %s", strerror(errno));
+		return STATUS_ERROR;
+	}
+	LoadBalanceTbl = (RecoveryTbl *)shmat(LoadBalanceTblShmid,0,0);
+	if (LoadBalanceTbl == (RecoveryTbl *)-1)
+	{
+		show_error("%s:shmat() failed. reason: %s", func,strerror(errno));
+		return STATUS_ERROR;
+	}
 	if (LoadBalanceTbl == (RecoveryTbl *)NULL)
 	{
 		show_error("%s:malloc failed: (%s)",func,strerror(errno));
 		return STATUS_ERROR;
 	}
-#ifdef PRINT_DEBUG
-	show_debug("LoadBalanceTbl allocate ok");
-#endif			
+	memset(LoadBalanceTbl , 0 , size );
 
 	/*
 	 * memory allocate cascade server table buffer
@@ -139,18 +147,12 @@ PGRset_Conf_Data(char * path)
 		show_error("%s:shmget() failed. reason: %s", func,strerror(errno));
 		return STATUS_ERROR;
 	}
-#ifdef PRINT_DEBUG
-	show_debug("%s:CascadeTbl shmget ok",func);
-#endif			
 	Cascade_Tbl = (ReplicateServerInfo *)shmat(CascadeTblShmid,0,0);
 	if (Cascade_Tbl == (ReplicateServerInfo *)-1)
 	{
-		show_error("%s:shmat() failed. reason: %s", func,strerror(errno));
+		show_error("%s:shmat() failed. reason: %s",func, strerror(errno));
 		return STATUS_ERROR;
 	}
-#ifdef PRINT_DEBUG
-	show_debug("%s:CascadeTbl shmat ok",func);
-#endif			
 	memset(Cascade_Tbl , 0 , size );
 
 	/*
@@ -163,18 +165,12 @@ PGRset_Conf_Data(char * path)
 		show_error("%s:shmget() failed. reason: %s", func,strerror(errno));
 		return STATUS_ERROR;
 	}
-#ifdef PRINT_DEBUG
-	show_debug("%s:CascadeInf shmget ok",func);
-#endif			
 	Cascade_Inf = (CascadeInf *)shmat(CascadeInfShmid,0,0);
 	if (Cascade_Inf == (CascadeInf *)-1)
 	{
-		show_error("%s:shmat() failed. reason: %s",func, strerror(errno));
+		show_error("%s:shmat() failed. reason: %s", func,strerror(errno));
 		return STATUS_ERROR;
 	}
-#ifdef PRINT_DEBUG
-	show_debug("%s:CascadeInf shmat ok",func);
-#endif			
 	memset(Cascade_Inf , 0 , size );
 
 	/*
@@ -184,62 +180,43 @@ PGRset_Conf_Data(char * path)
 	CommitLogShmid = shmget(IPC_PRIVATE,size,IPC_CREAT | IPC_EXCL | 0600);
 	if (CommitLogShmid < 0)
 	{
-		show_error("%s:shmget() failed. reason: %s", func, strerror(errno));
+		show_error("%s:shmget() failed. reason: %s", func,strerror(errno));
 		return STATUS_ERROR;
 	}
-#ifdef PRINT_DEBUG
-	show_debug("%s:CommitLog shmget ok",func);
-#endif			
 	Commit_Log_Tbl = (CommitLogInf *)shmat(CommitLogShmid,0,0);
 	if (Commit_Log_Tbl == (CommitLogInf *)-1)
 	{
 		show_error("%s:shmat() failed. reason: %s",func, strerror(errno));
 		return STATUS_ERROR;
 	}
-#ifdef PRINT_DEBUG
-	show_debug("%s:Commit_Log_Tbl shmat ok",func);
-#endif			
 	memset(Commit_Log_Tbl , 0 , size );
-	(Commit_Log_Tbl + (MAX_DB_SERVER * MAX_CONNECTIONS) -1)->inf.useFlag = DB_TBL_END;
+	(Commit_Log_Tbl + (MAX_DB_SERVER * MAX_CONNECTIONS) -1)->inf.useFlag = DATA_END;
 
 	/* create semapho */
-	if ((SemID = semget(IPC_PRIVATE,2,IPC_CREAT | IPC_EXCL | 0600)) < 0)
+	if ((SemID = semget(IPC_PRIVATE,4,IPC_CREAT | IPC_EXCL | 0600)) < 0)
 	{
 		show_error("%s:semget() failed. (%s)",func,strerror(errno));
 		return STATUS_ERROR;
 	}
-	for ( i = 0 ; i < 2 ; i ++)
+	for ( i = 0 ; i < 4 ; i ++)
 	{
 		semctl(SemID, i, GETVAL, sem_arg);
 		sem_arg.val = 1;
 		semctl(SemID, i, SETVAL, sem_arg);
 	}
 
-	/* create semapho */
-	if ((CascadeSemID = semget(IPC_PRIVATE,2,IPC_CREAT | IPC_EXCL | 0600)) < 0)
+	if ((LockSemID = semget(IPC_PRIVATE,4,IPC_CREAT | IPC_EXCL | 0600)) < 0)
 	{
 		show_error("%s:semget() failed. (%s)",func,strerror(errno));
 		return STATUS_ERROR;
 	}
-	for ( i = 0 ; i < 2 ; i ++)
+	for ( i = 0 ; i < 4 ; i ++)
 	{
-		semctl(CascadeSemID, i, GETVAL, sem_arg);
+		semctl(LockSemID, i, GETVAL, sem_arg);
 		sem_arg.val = 1;
-		semctl(CascadeSemID, i, SETVAL, sem_arg);
+		semctl(LockSemID, i, SETVAL, sem_arg);
 	}
 
-
-	if ((VacuumSemID = semget(IPC_PRIVATE,2,IPC_CREAT | IPC_EXCL | 0600)) < 0)
-	{
-		show_error("%s:semget() failed. (%s)",func,strerror(errno));
-		return STATUS_ERROR;
-	}
-	for ( i = 0 ; i < 2 ; i ++)
-	{
-		semctl(VacuumSemID, i, GETVAL, sem_arg);
-		sem_arg.val = 1;
-		semctl(VacuumSemID, i, SETVAL, sem_arg);
-	}
 	size = sizeof(ReplicationLogInf);
 	Replicateion_Log = malloc(size);
 	if (Replicateion_Log == NULL)
@@ -249,18 +226,13 @@ PGRset_Conf_Data(char * path)
 	}
 	memset(Replicateion_Log , 0 , size );
 	Replicateion_Log->RLog_Sock_Path = NULL;
-#ifdef PRINT_DEBUG
-	show_debug("%s:RLog Memory Allocation ok",func);
-#endif			
-
 
 	/*
 	 * set each datas into the tables
 	 */
-	conf = ConfData_Top;
+	conf = Com_Info.ConfData_Info.ConfData_Top;
 	while (conf != (ConfDataType *)NULL) 
 	{
-		show_debug("registering (key,value)=(%s,%s)",conf->key,conf->value);
 		/* get cluster db data */
 		if (!strcmp(conf->table,CLUSTER_SERVER_TAG))
 		{
@@ -275,19 +247,7 @@ PGRset_Conf_Data(char * path)
 			}
 			if (!strcmp(conf->key,HOST_NAME_TAG))
 			{
-			        int ip;
-				memcpy(host_tbl[rec_no].hostName,conf->value,sizeof(host_tbl[rec_no].hostName));
-				show_debug("registering hostname %s",host_tbl[rec_no].hostName);
-				ip=PGRget_ip_by_name(conf->value);
-
-				sprintf(host_tbl[rec_no].resolvedName,
-					 "%d.%d.%d.%d",
-					 (ip      ) & 0xff ,
-					 (ip >>  8) & 0xff ,
-					 (ip >> 16) & 0xff ,
-					 (ip >> 24) & 0xff );
-				show_debug("resolved name is %s",host_tbl[rec_no].resolvedName);
-
+				strncpy(host_tbl[rec_no].hostName,conf->value,sizeof(host_tbl[rec_no].hostName));
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
@@ -303,6 +263,19 @@ PGRset_Conf_Data(char * path)
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
+			if (!STRCMP(conf->key,LIFECHECK_PORT_TAG))
+			{
+				if (atoi(conf->value) > 0)
+				{
+					host_tbl[rec_no].lifecheckPort = atoi(conf->value);
+				}
+				else
+				{
+					host_tbl[rec_no].lifecheckPort = DEFAULT_CLUSTER_LIFECHECK_PORT;
+				}
+				conf = (ConfDataType*)conf->next;
+				continue;
+			}
 		}
 		/* get cascade server data */
 		else if (!strcmp(conf->table, REPLICATION_SERVER_INFO_TAG))
@@ -318,7 +291,7 @@ PGRset_Conf_Data(char * path)
 			}
 			if (!strcmp(conf->key,HOST_NAME_TAG))
 			{
-				memcpy((Cascade_Tbl+cascade_rec_no)->hostName,conf->value,sizeof(Cascade_Tbl->hostName));
+				strncpy((Cascade_Tbl+cascade_rec_no)->hostName,conf->value,sizeof(Cascade_Tbl->hostName));
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
@@ -332,10 +305,8 @@ PGRset_Conf_Data(char * path)
 				{
 					(Cascade_Tbl+cascade_rec_no)->portNumber = DEFAULT_PGRP_PORT;
 				}
-				(Cascade_Tbl+cascade_rec_no)->sock = -1;
-
 				conf = (ConfDataType*)conf->next;
-				PGRset_cascade_server_status(Cascade_Tbl+cascade_rec_no,DB_TBL_USE);
+				PGRset_cascade_server_status(Cascade_Tbl+cascade_rec_no,DATA_USE);
 				if (cascade_rec_no == 0)
 				{
 					Cascade_Inf->top = Cascade_Tbl;
@@ -352,8 +323,19 @@ PGRset_Conf_Data(char * path)
 				{
 					(Cascade_Tbl+cascade_rec_no)->recoveryPortNumber = DEFAULT_PGRP_RECOVERY_PORT;
 				}
-				(Cascade_Tbl+cascade_rec_no)->rlog_sock=-1;
-				(Cascade_Tbl+cascade_rec_no +1)->useFlag = DB_TBL_END;
+				conf = (ConfDataType*)conf->next;
+				continue;
+			}
+			if (!STRCMP(conf->key,LIFECHECK_PORT_TAG))
+			{
+				if (atoi(conf->value) > 0)
+				{
+					(Cascade_Tbl+cascade_rec_no)->lifecheckPortNumber = atoi(conf->value);
+				}
+				else
+				{
+					(Cascade_Tbl+cascade_rec_no)->lifecheckPortNumber = DEFAULT_PGRP_LIFECHECK_PORT;
+				}
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
@@ -372,15 +354,35 @@ PGRset_Conf_Data(char * path)
 			}
 			if (!strcmp(conf->key,HOST_NAME_TAG))
 			{
-				memcpy((LoadBalanceTbl + lb_rec_no)->hostName, conf->value,sizeof(LoadBalanceTbl->hostName));
+				strncpy((LoadBalanceTbl + lb_rec_no)->hostName, conf->value,sizeof(LoadBalanceTbl->hostName));
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
 			if (!strcmp(conf->key,RECOVERY_PORT_TAG))
 			{
-				(LoadBalanceTbl + lb_rec_no)->recoveryPort = atoi(conf->value);
-				(LoadBalanceTbl + lb_rec_no)->sock = -1;
-				(LoadBalanceTbl + lb_rec_no)->recovery_sock = -1;
+				(LoadBalanceTbl + lb_rec_no)->useFlag = DATA_USE; 
+				(LoadBalanceTbl + lb_rec_no+1)->useFlag = DATA_END; 
+				if (atoi(conf->value) > 0)
+				{
+					(LoadBalanceTbl + lb_rec_no)->recoveryPort = atoi(conf->value);
+				}
+				else
+				{
+					(LoadBalanceTbl + lb_rec_no)->recoveryPort = DEFAULT_PGLB_RECOVERY_PORT;
+				}
+				conf = (ConfDataType*)conf->next;
+				continue;
+			}
+			if (!STRCMP(conf->key,LIFECHECK_PORT_TAG))
+			{
+				if (atoi(conf->value) > 0)
+				{
+					(LoadBalanceTbl + lb_rec_no)->lifecheckPort = atoi(conf->value);
+				}
+				else
+				{
+					(LoadBalanceTbl + lb_rec_no)->lifecheckPort = DEFAULT_PGLB_LIFECHECK_PORT;
+				}
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
@@ -389,11 +391,17 @@ PGRset_Conf_Data(char * path)
 		{
 			if (!strcmp(conf->key,REPLICATE_PORT_TAG))
 			{
-				Port_Number = atoi(conf->value);
+				if (atoi(conf->value) > 0)
+				{
+					Port_Number = atoi(conf->value);
+				}
+				else
+				{
+					Port_Number =DEFAULT_PGRP_PORT;
+				}
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
-			/* get port number for recovery cluster db server */
 			if (!strcmp(conf->key,RECOVERY_PORT_TAG))
 			{
 				if (atoi(conf->value) > 0)
@@ -433,7 +441,6 @@ PGRset_Conf_Data(char * path)
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
-			/*
 			if (!STRCMP(conf->key,STATUS_LOG_FILE_TAG))
 			{
 				Com_Info.Log_Info.PGRStatusFileName = strdup(conf->value);
@@ -446,7 +453,6 @@ PGRset_Conf_Data(char * path)
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
-			*/
 			/* get response mode */
 			if (!strcmp(conf->key,RESPONSE_MODE_TAG))
 			{
@@ -475,6 +481,20 @@ PGRset_Conf_Data(char * path)
 				conf = (ConfDataType*)conf->next;
 				continue;
 			}
+			/* get reserved connection number */
+			if (!strcmp(conf->key,RESERVED_CONNECTIONS_TAG))
+			{
+				if (atoi(conf->value) > 1)
+				{
+					Reserved_Connections = atoi(conf->value);
+				}
+				else
+				{
+					Reserved_Connections = 1;
+				}
+				conf = (ConfDataType*)conf->next;
+				continue;
+			}
 			conf = (ConfDataType*)conf->next;
 		}
 		conf = (ConfDataType*)conf->next;
@@ -483,21 +503,34 @@ PGRset_Conf_Data(char * path)
 	/*
 	 * open log file
 	 */
-	if (Log_Print)
+	if (Com_Info.Log_Info.Log_Print)
 	{
-		snprintf(fname,sizeof(fname),"%s/%s",path,PGREPLICATE_LOG_FILE);
-
-		LogFp = fopen(fname,"a");
-		if (LogFp == NULL)
+		if (Com_Info.Log_Info.PGRLogFileName == NULL)
+		{
+			snprintf(fname,sizeof(fname),"%s/%s",path,PGREPLICATE_LOG_FILE);
+		}
+		else
+		{
+			strncpy(fname,Com_Info.Log_Info.PGRLogFileName,sizeof(fname));
+		}
+		Com_Info.Log_Info.LogFp = fopen(fname,"a");
+		if (Com_Info.Log_Info.LogFp == NULL)
 		{
 			show_error("%s:fopen failed: (%s)",func,strerror(errno));
 			return STATUS_ERROR;
 		}
 	}
 
-        snprintf(fname,sizeof(fname),"%s/%s",path,PGREPLICATE_STATUS_FILE);
-	StatusFp = fopen(fname,"a");
-	if (StatusFp == NULL)
+	if (Com_Info.Log_Info.PGRStatusFileName == NULL)
+	{
+		snprintf(fname,sizeof(fname),"%s/%s",path,PGREPLICATE_STATUS_FILE);
+	}
+	else
+	{
+		strncpy(fname,Com_Info.Log_Info.PGRStatusFileName,sizeof(fname));
+	}
+	Com_Info.Log_Info.StatusFp = fopen(fname,"a");
+	if (Com_Info.Log_Info.StatusFp == NULL)
 	{
 		show_error("%s:fopen failed: (%s)",func,strerror(errno));
 		return STATUS_ERROR;
@@ -510,27 +543,21 @@ PGRset_Conf_Data(char * path)
 	HostTblShmid = shmget(IPC_PRIVATE,size,IPC_CREAT | IPC_EXCL | 0600);
 	if (HostTblShmid < 0)
 	{
-		show_error("%s:shmget() failed. reason: %s", func,strerror(errno));
+		show_error("%s:shmget() failed. reason: %s",func, strerror(errno));
 		return STATUS_ERROR;
 	}
-#ifdef PRINT_DEBUG
-	show_debug("%s:HostTbl shmget ok",func);
-#endif			
 	Host_Tbl_Begin = (HostTbl *)shmat(HostTblShmid,0,0);
 	if (Host_Tbl_Begin == (HostTbl *)-1)
 	{
-		show_error("%s:shmat() failed. reason: %s", func, strerror(errno));
+		show_error("%s:shmat() failed. reason: %s",func, strerror(errno));
 		return STATUS_ERROR;
 	}
-#ifdef PRINT_DEBUG
-	show_debug("%s:HostTbl shmat ok",func);
-#endif			
 	memset(Host_Tbl_Begin , 0 , size );
-	Host_Tbl_Begin -> useFlag = DB_TBL_END;
+	Host_Tbl_Begin -> useFlag = DATA_END;
 
 	for ( i = 0 ; i <= cnt ; i ++)
 	{
-		PGRadd_HostTbl(&host_tbl[i],DB_TBL_USE);
+		PGRadd_HostTbl(&host_tbl[i],DATA_USE);
 	}
 	/* set load balance table */
 	for ( i = 0 ; i <= lb_cnt ; i ++)
@@ -545,7 +572,7 @@ PGRset_Conf_Data(char * path)
 	PGR_Result = malloc(PGR_MESSAGE_BUFSIZE);
 	if (PGR_Result == NULL)
 	{
-		show_error("%s:malloc() failed. reason: %s", func, strerror(errno));
+		show_error("%s:malloc() failed. reason: %s",func,strerror(errno));
 		return STATUS_ERROR;
 	}
 	memset(PGR_Result,0,PGR_MESSAGE_BUFSIZE);
@@ -554,20 +581,37 @@ PGRset_Conf_Data(char * path)
 	PGR_Log_Header = malloc(sizeof(ReplicateHeader));
 	if (PGR_Log_Header == NULL)
 	{
-		show_error("%s:malloc() failed. reason: %s", func, strerror(errno));
+		show_error("%s:malloc() failed. reason: %s",func, strerror(errno));
 		return STATUS_ERROR;
 	}
 	memset(PGR_Log_Header,0,sizeof(ReplicateHeader));
 
 	/* allocate send query id */
 	size = sizeof(unsigned int) * (MAX_DB_SERVER +1);
-	PGR_Send_Query_ID = malloc (size);
+	PGR_Send_Query_ID = malloc (sizeof(unsigned int) * MAX_DB_SERVER);
 	if (PGR_Send_Query_ID == NULL)
 	{
-		show_error("%s:malloc() failed. reason: %s", func, strerror(errno));
+		show_error("%s:malloc() failed. reason: %s",func, strerror(errno));
 		return STATUS_ERROR;
 	}
 	memset(PGR_Send_Query_ID, 0, size);
+
+	/* create lock wait table */
+	size = sizeof(LockWaitInf);
+	LockWaitTblShmid = shmget(IPC_PRIVATE,size,IPC_CREAT | IPC_EXCL | 0600);
+	if (LockWaitTblShmid < 0)
+	{
+		show_error("%s:shmget() failed. reason: %s", func,strerror(errno));
+		return STATUS_ERROR;
+	}
+	Lock_Wait_Tbl = (LockWaitInf *)shmat(LockWaitTblShmid,0,0);
+	if (Lock_Wait_Tbl == (LockWaitInf *)-1)
+	{
+		show_error("%s:shmat() failed. reason: %s",func, strerror(errno));
+		return STATUS_ERROR;
+	}
+	memset(Lock_Wait_Tbl , 0 , size );
+	
 	for ( i = 0 ; i < MAX_DB_SERVER ; i ++)
 	{
 		StartReplication[i] = true;
@@ -578,28 +622,24 @@ PGRset_Conf_Data(char * path)
 	cascade_rec_no ++;
 	gethostname((Cascade_Tbl+cascade_rec_no)->hostName,sizeof(Cascade_Tbl->hostName));
 	(Cascade_Tbl+cascade_rec_no)->portNumber = Port_Number;
-	(Cascade_Tbl+cascade_rec_no)->recoveryPortNumber = Recovery_Port_Number;
-	(Cascade_Tbl+cascade_rec_no)->sock = -1;
+	(Cascade_Tbl+cascade_rec_no)->recoveryPortNumber = Com_Info.ConfData_Info.Recovery_Port_Number;
 
-	PGRset_cascade_server_status(Cascade_Tbl+cascade_rec_no,DB_TBL_USE);
+	PGRset_cascade_server_status(Cascade_Tbl+cascade_rec_no,DATA_USE);
 	/* terminate */
-	(Cascade_Tbl+(cascade_rec_no+1))->useFlag = DB_TBL_END;
+	(Cascade_Tbl+(cascade_rec_no+1))->useFlag = DATA_END;
 
 	Cascade_Inf->top = Cascade_Tbl;
 	Cascade_Inf->end = Cascade_Tbl+cascade_rec_no;
-      	Cascade_Inf->upper = NULL;
 	Cascade_Inf->lower = NULL;
 	if (cascade_rec_no >= 1)
 	{
 		Cascade_Inf->upper = (Cascade_Tbl+cascade_rec_no - 1);
 	}
-	(Cascade_Tbl+(cascade_rec_no+1))->useFlag = DB_TBL_END;
 
 	Cascade_Inf->myself = (Cascade_Tbl+cascade_rec_no);
-	Cascade_Inf->useFlag = DB_TBL_USE;
-
-	PGR_Response_Inf->response_mode = PGR_NORMAL_MODE;
+	Cascade_Inf->useFlag = DATA_USE;
 
 	return STATUS_OK;
 }
 
+#endif /* USE_REPLICATION */
