@@ -6,11 +6,14 @@
  * NOTE:
  *    This is the main module of the replication server.
  *
- * Portions Copyright (c) 2003-2006, Atsushi Mitani
+ * Portions Copyright (c) 2003, Atsushi Mitani
  *--------------------------------------------------------------------
  */
+#ifdef USE_REPLICATION
+
 #include "postgres.h"
 
+#include <pthread.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -31,7 +34,6 @@
 #include <sys/param.h>
 #include <arpa/inet.h>
 #include <sys/file.h>
-#include <pthread.h>
 
 #ifdef HAVE_NETINET_TCP_H
 #include <netinet/tcp.h>
@@ -46,13 +48,10 @@
 
 #include "miscadmin.h"
 #include "nodes/nodes.h"
-
 #include "libpq-fe.h"
 #include "libpq/libpq-fs.h"
 #include "libpq-int.h"
 #include "fe-auth.h"
-
-
 #include "access/xact.h"
 #include "replicate_com.h"
 #include "pgreplicate.h"
@@ -74,9 +73,7 @@
  *--------------------------------------
  */
 /* for replicate_com.h */
-
-ConfDataType * ConfData_Top = (ConfDataType *)NULL;
-ConfDataType * ConfData_End = (ConfDataType *)NULL;
+PGR_Com_Info Com_Info;
 
 /* replication server data */
 uint16_t Port_Number = 0;
@@ -85,10 +82,13 @@ uint16_t Recovery_Port_Number = 0;
 
 /* global table data */
 HostTbl *Host_Tbl_Begin = NULL;
-Dllist * Transaction_Tbl_Begin = NULL;
+TransactionTbl * Transaction_Tbl_Begin = NULL;
 TransactionTbl * Transaction_Tbl_End = NULL;
+TransactionTbl * PGconn_Tbl_Begin = NULL;
+TransactionTbl * PGconn_Tbl_End = NULL;
 RecoveryTbl * LoadBalanceTbl = NULL;
 RecoveryStatusInf * Recovery_Status_Inf = NULL;
+LockWaitInf * Lock_Wait_Tbl = NULL;
 ReplicateHeader * PGR_Log_Header = NULL;
 ReplicateServerInfo * Cascade_Tbl = NULL;;
 CommitLogInf * Commit_Log_Tbl = NULL;
@@ -98,9 +98,6 @@ CascadeInf * Cascade_Inf = NULL;
 ReplicationLogInf * Replicateion_Log = NULL;
 /* IPC's id data */
 int RecoveryShmid = 0;
-int ReplicateSerializationShmid=0;
-int RecoveryMsgShmid = 0;
-int *RecoveryMsgid = NULL;
 int HostTblShmid = 0;
 int LockWaitTblShmid = 0;
 int LoadBalanceTblShmid = 0;
@@ -113,10 +110,9 @@ int PGconnMsgid = 0;
 int MaxBackends = 0;
 char * PGR_Result = NULL;
 int SemID = 0;
-int RecoverySemID= 0;
-int RecovErysemid = 0;
+int RecoverySemID = 0;
+int LockSemID = 0;
 int VacuumSemID = 0;
-int CascadeSemID= 0;
 char * PGR_Data_Path = NULL;
 char * PGR_Write_Path = NULL;
 int IS_SESSION_AUTHORIZATION = 0;
@@ -127,29 +123,14 @@ bool PGR_Use_Replication_Log = false;
 bool	PGR_AutoCommit = true;
 unsigned int * PGR_Send_Query_ID = NULL;
 unsigned int PGR_Query_ID = 0;
-volatile bool exit_processing = false;
-int pgreplicate_pid = 0;
-
-int ReplicateSock = -1;
-
 RecoveryQueueInf RecoveryQueue;
 char * Backend_Socket_Dir = NULL;
-
-unsigned int * PGR_ReplicateSerializationID = NULL;
-
-int Log_Print = 0;
-int Debug_Print = 0;
-FILE * LogFp = (FILE *)NULL;
-FILE * StatusFp = (FILE *)NULL;
+char * PGRuserName = NULL;
+int Reserved_Connections = 1;
+int Idle_Flag = IDLE_MODE;
+bool Exit_Request = false;
 
 extern char *optarg;
-char * PGRuserName = NULL;
-
-int fork_wait_time = 0;
-int Idle_Flag = IDLE_MODE;
-volatile bool Exit_Request = false;
-
-pthread_mutex_t transaction_table_mutex;
 
 /*--------------------------------------
  * PROTOTYPE DECLARATION
@@ -161,10 +142,12 @@ static void replicate_main(void);
 static void quick_exit(SIGNAL_ARGS);
 static void daemonize(void);
 static void write_pid_file(void);
-static void stop_pgreplicate(void);
 static bool is_exist_pid_file(void);
+static void child_wait(SIGNAL_ARGS);
 static void usage(void);
 
+void PGRstop_pgreplicate(int sig);
+
 /*--------------------------------------------------------------------
  * SYMBOL
  *    replicate_loop()
@@ -180,26 +163,14 @@ static void usage(void);
 static int
 replicate_loop(int fd)
 {
-	char * func = "replicate_loop()";
+	char *func = "replicate_loop()";
 	pid_t pgid = 0;
 	pid_t pid = 0;
 	int sock = -1;
 	int rtn = 0;
-	int cnt = 0;
-	int result;
-	bool exist_sys_log=false;
-	bool exist_replicate=false;
-
-
-	result = PGR_Create_Acception(fd,&sock,"",Port_Number);
-	if (result == STATUS_ERROR)
-	{
-		show_error("%s: accept failed (%s)", func, strerror(errno));
-		if (sock != -1)
-			close(sock);
-		return 1;
-	}
+	bool exist_sys_log = false;
 
+	PGR_Create_Acception(fd,&sock,"",Port_Number);
 	pgid = getpgid(0);
 	pid = fork();
 	if (pid <0)
@@ -212,39 +183,28 @@ replicate_loop(int fd)
 		int status = LOOP_CONTINUE;
 		bool PGR_Cascade = false;
 		ReplicateHeader  header;
-		ReplicateHeader  header_save_for_recovering;
 		char * query = NULL;
 
-		if (fork_wait_time > 0) {
-#ifdef PRINT_DEBUG
-			show_debug("wait fork(): pid = %d", getpid());
-#endif			
-			sleep(fork_wait_time);
-		}
-
-		close(fd);
-
-		PGRsignal(SIGHUP, quick_exit);	
-		PGRsignal(SIGINT, quick_exit);	
-		PGRsignal(SIGQUIT, quick_exit);	
-		PGRsignal(SIGTERM, quick_exit);	
-		PGRsignal(SIGALRM, quick_exit); 
-		PGRsignal(SIGPIPE, SIG_IGN); 
+		signal(SIGHUP, SIG_IGN);	
+		signal(SIGINT, quick_exit);	
+		signal(SIGQUIT, quick_exit);	
+		signal(SIGTERM, quick_exit);	
+		signal(SIGALRM, quick_exit); 
 		setpgid(0,pgid);
-		
-		if (PGRinit_transaction_table() != STATUS_OK)
-		{
-			show_error("transaction table memory allocate failed");
-			PGR_Close_Sock(&sock);
-			exit(1);
-		}
-
-		pthread_mutex_init(&transaction_table_mutex, NULL);
 
 		/* child loop */
 		for (;;)
 		{
 			fd_set	  rmask;
+			struct timeval timeout;
+
+			Idle_Flag = IDLE_MODE;
+			if (Exit_Request)
+			{
+				quick_exit(SIGINT);
+			}
+			timeout.tv_sec = 1;
+			timeout.tv_usec = 0;
 			
 			if (query != NULL)
 			{
@@ -256,23 +216,22 @@ replicate_loop(int fd)
 			 */
 			FD_ZERO(&rmask);
 			FD_SET(sock,&rmask);
-			rtn = select(sock+1, &rmask, (fd_set *)NULL, (fd_set *)NULL, NULL);
-			if (rtn < 0)
-			{
-				if (errno == EINTR)
-					continue;
-			}
-
+			rtn = select(sock+1, &rmask, (fd_set *)NULL, (fd_set *)NULL, &timeout);
 			if (rtn && FD_ISSET(sock, &rmask))
 			{
-				query = NULL;
-				query = PGRread_packet(sock,&header);				
-				if ((query == NULL) || (header.cmdSts == 0))
+				Idle_Flag = BUSY_MODE;
+#ifdef PRINT_DEBUG
+				show_debug("%s:replicate_loop selected",func);
+#endif			
+				query = PGRread_packet(sock,&header);
+				if ((query == NULL) && (header.cmdSys == 0))
 				{
-
+#ifdef PRINT_DEBUG
+				show_debug("%s: PGRread_packet failed query[%s] cmdSys[%c]",func,query,header.cmdSys);
+#endif			
 					if (exist_sys_log)
 					{
-						show_error("%s:upper cascade closed? , errno=%d(%s)",func,errno,strerror(errno));
+						show_error("%s:upper cascade closed",func);
 						memset(&header, 0, sizeof(ReplicateHeader));
 						header.cmdSys = CMD_SYS_CALL;
 						header.cmdSts = CMD_STS_QUERY_SUSPEND;
@@ -284,97 +243,66 @@ replicate_loop(int fd)
 					{
 #ifdef PRINT_DEBUG
 						show_debug("%s:session closed",func);
-#endif
-						if (exist_replicate)
-						{
-							header_save_for_recovering.cmdSts=CMD_TYPE_OTHER;
-							header_save_for_recovering.cmdType=CMD_TYPE_CONNECTION_CLOSE;
-							header_save_for_recovering.query_size = htonl(21);
-							PGRdo_replicate(sock,&header_save_for_recovering,"PGR_CLOSE_CONNECTION");
-						}
-						PGRsend_notice_quit();
+#endif			
 					}
 					break;
 				}
-				cnt = 0;
 				switch (header.cmdSys)
 				{
-				        case CMD_SYS_PREREPLICATE:
-								if(Cascade_Inf!=NULL ||
-								   Cascade_Inf->upper == NULL) 
-								{
-										/* 1 means "I am primary replicate server." */
-										PGRreturn_result(sock,"1", PGR_NOWAIT_ANSWER);
-								}
-								else
-								{
-										/* 1 means "I am not primary replicate server." */
-										PGRreturn_result(sock,"0", PGR_NOWAIT_ANSWER);
-				                }
-								break;
-						case CMD_SYS_REPLICATE:
-								if (exist_replicate == false)
-								{
-										exist_replicate=true;
-										memcpy(&header_save_for_recovering,
-											   &header,
-											   sizeof(ReplicateHeader));
-								}
-								status = PGRdo_replicate(sock,&header,query);
-								break;
-						case CMD_SYS_LOG:
-								exist_sys_log = true;
-								PGRsend_rlog_to_local(&header, query);
-								/* set own replicate id by rlog */
-								PGRset_replication_id(ntohl(header.replicate_id));
-								PGRsend_notice_rlog_done(sock);
-								break;
-						case  CMD_SYS_CASCADE:
-								PGR_Cascade = true;
-								PGRcascade_main(sock,&header,query);
-								break;
-						case  CMD_SYS_CALL:
-								if (header.cmdSts == CMD_STS_TRANSACTION_ABORT)
-								{
-										PGRreconfirm_commit(sock,&header);
-								
-								}
-								else if (header.cmdSts == CMD_STS_NOTICE)
-								{
-
-								}
-								else if (header.cmdSts == CMD_STS_RESPONSE)
-								{
-										if (header.cmdType == CMD_TYPE_FRONTEND_CLOSED)
-										{
-												show_debug("replicate_loop ends due to frontend_closed.");
-												PGRsend_notice_rlog_done(sock);
-												status = LOOP_END;
-										}
-								}
-								break;
-						default:
-								show_debug("WARNING: unknown Header->cmdSys %c",header.cmdSys);
+					case CMD_SYS_REPLICATE:
+						status = PGRdo_replicate(sock,&header,query);
+						PGRset_start_life_check();
+						break;
+					case CMD_SYS_LOG:
+						exist_sys_log = true;
+						PGRsend_rlog_to_local(&header, query);
+						/* set own replicate id by rlog */
+						PGRset_replication_id(header.replicate_id);
+						PGRsend_notice_rlog_done(sock);
+						break;
+					case  CMD_SYS_CASCADE:
+						PGR_Cascade = true;
+						PGRcascade_main(sock,&header,query);
+						break;
+					case  CMD_SYS_CALL:
+						if (header.cmdSts == CMD_STS_TRANSACTION_ABORT)
+						{
+							PGRreconfirm_commit(sock,&header);
+						}
+						else if (header.cmdSts == CMD_STS_NOTICE)
+						{
+							if (header.cmdType == CMD_TYPE_DEADLOCK_DETECT)
+							{
+								status = LOOP_END;
+							}
+						}
+						else if (header.cmdSts == CMD_STS_RESPONSE)
+						{
+							if (header.cmdType == CMD_TYPE_FRONTEND_CLOSED)
+							{
+								status = LOOP_END;
+							}
+						}
+						break;
 				}
 			}
+			if (query != NULL)
+			{
+				free(query);
+				query = NULL;
+			}
 			if (status == LOOP_END)
 			{
 				break;
 			}
 		}
-
-		pthread_mutex_destroy(&transaction_table_mutex);
 		PGR_Close_Sock(&sock);
-		if (query != NULL)
-		{
-			free(query);
-			query = NULL;
-		}
-		PGRclear_connections();
+		PGRclear_transactions();
+		PGRclear_connections( DATA_TEMP_USE );
 #ifdef PRINT_DEBUG
 		show_debug("%s:replicate loop exit",func);
 #endif			
-		exit(0);
+		pthread_exit((void*)0);
 	}
 	else
 	{
@@ -386,11 +314,10 @@ replicate_loop(int fd)
 static void
 startup_replication_server(void)
 {
-	ReplicateHeader  header;
 	char hostName[HOSTNAME_MAX_LENGTH];
-	char userName[USERNAME_MAX_LENGTH];
-	char query[256];
 
+	memset(hostName,0,sizeof(hostName));
+	gethostname(hostName,sizeof(hostName)-1);
 	if (PGRuserName == NULL)
 	{
 		PGRuserName = getenv("LOGNAME");
@@ -401,25 +328,7 @@ startup_replication_server(void)
 				PGRuserName = "postgres";
 		}
 	}
-	memset(&header,0,sizeof(ReplicateHeader));
-	memset(query,0,sizeof(query));
-	memset(hostName,0,sizeof(hostName));
-	memset(userName,0,sizeof(userName));
-	gethostname(hostName,sizeof(hostName)-1);
-	strncpy(userName ,PGRuserName,sizeof(userName)-1);
-	snprintf(query,sizeof(query)-1,"SELECT %s(%d,'%s',%d,%d)",
-			PGR_SYSTEM_COMMAND_FUNC,
-			PGR_STARTUP_REPLICATION_SERVER_FUNC_NO,
-			hostName,
-			Port_Number,
-			Recovery_Port_Number);
-	header.cmdSts = CMD_STS_NOTICE;
-	header.query_id = htonl(PGRget_next_query_id());
-	header.query_size = htonl(strlen(query));
-	memcpy(header.from_host,hostName,sizeof(header.from_host));
-	memcpy(header.userName,userName,sizeof(header.userName));
-	strcpy(header.dbName,"template1");
-	replicate_packet_send_internal( &header, query,-1,PGRget_recovery_status(),true);
+	PGRnotice_replication_server(hostName,Port_Number,Recovery_Port_Number,LifeCheck_Port_Number,PGRuserName);
 }
 
 /*--------------------------------------------------------------------
@@ -436,32 +345,25 @@ startup_replication_server(void)
 static void
 replicate_main(void)
 {
-#ifdef PRINT_DEBUG
 	char * func = "replicate_main()";
-#endif			
 	int status;
+	int fd = -1;
 	int rtn;
-	show_debug ("%s:entering replicate_main",func);
-
-	/* cascade start up notice */
-	if (Cascade_Inf->upper != NULL)
-	{
-		show_debug("initialize cascade information");
-		PGRstartup_cascade();
-	}
-
-	status = PGR_Create_Socket_Bind(&ReplicateSock, "", Port_Number);
 
+	status = PGR_Create_Socket_Bind(&fd, "", Port_Number);
 	if (status != STATUS_OK)
 	{
-		show_debug("%s %d port bind failed. quit.",func,Port_Number);
-		stop_pgreplicate();
 		PGRreplicate_exit(0);
 	}
 #ifdef PRINT_DEBUG
-	show_debug("%s %d port bind OK",func,Port_Number);
+	show_debug("%s:replicate main %d port bind OK",func,Port_Number);
 #endif			
 	
+	/* cascade start up notice */
+	if (Cascade_Inf->upper != NULL)
+	{
+		PGRstartup_cascade();
+	}
 
 	/* replication start up notice */
 	startup_replication_server();
@@ -471,9 +373,11 @@ replicate_main(void)
 		fd_set	  rmask;
 		struct timeval timeout;
 
-		if (exit_processing == true)
-			PGRreplicate_exit(0);
-
+		Idle_Flag = IDLE_MODE;
+		if (Exit_Request)
+		{
+			PGRreplicate_exit(SIGINT);
+		}
 		timeout.tv_sec = PGR_RECV_TIMEOUT;
 		timeout.tv_usec = 0;
 
@@ -482,25 +386,23 @@ replicate_main(void)
 		 * Wait for something to happen.
 		 */
 		FD_ZERO(&rmask);
-		FD_SET(ReplicateSock,&rmask);
-		rtn = select(ReplicateSock+1, &rmask, (fd_set *)NULL, (fd_set *)NULL, &timeout);
-		if (rtn < 0)
-			continue;
-
-		if (rtn && FD_ISSET(ReplicateSock, &rmask))
+		FD_SET(fd,&rmask);
+		rtn = select(fd+1, &rmask, (fd_set *)NULL, (fd_set *)NULL, &timeout);
+		if (rtn && FD_ISSET(fd, &rmask))
 		{
+			Idle_Flag = BUSY_MODE;
 			/*
 			 * get recovery status.
 			 */
 			PGRcheck_recovered_host();
-
-			if (exit_processing == true)
-				break;
-
+			/* 
+			 * get new db login info
+			 */
+			PGRget_pgconn_queue();
 			/*
 			 * call replication module
 			 */
-			replicate_loop(ReplicateSock);
+			replicate_loop(fd);
 		}
 	}
 }
@@ -517,11 +419,16 @@ replicate_main(void)
  *--------------------------------------------------------------------
  */
 static void
-quick_exit(SIGNAL_ARGS)
+quick_exit(int sig)
 {
-#ifdef PRINT_DEBUG
-	show_debug("quick_exit:signo = %d", postgres_signal_arg);
-#endif
+	Exit_Request = true;
+	if (sig == SIGTERM)
+	{
+		if (Idle_Flag == BUSY_MODE)
+		{
+			return;
+		}
+	}
 	exit(0);
 }
 
@@ -558,7 +465,7 @@ daemonize(void)
 #ifdef HAVE_SETSID
 	if (setsid() < 0)
 	{
-		show_error("%s:setsid() failed. reason:%s", func,strerror(errno));
+		show_error("%s:setsid() failed. reason:%s",func,strerror(errno));
 		exit(1);
 	}
 #endif
@@ -595,7 +502,7 @@ write_pid_file(void)
 	if (!fd)
 	{
 		show_error("%s:could not open pid file as %s. reason: %s",
-				   func, fname, strerror(errno));
+				   func,fname, strerror(errno));
 		exit(1);
 	}
 	snprintf(pidbuf, sizeof(pidbuf), "%d", getpid());
@@ -610,7 +517,7 @@ write_pid_file(void)
 
 /*--------------------------------------------------------------------
  * SYMBOL
- *    stop_pgreplicate()
+ *    PGRstop_pgreplicate()
  * NOTES
  *    Stop the pgreplicate process
  * ARGS
@@ -619,15 +526,21 @@ write_pid_file(void)
  *    none
  *--------------------------------------------------------------------
  */
-static void 
-stop_pgreplicate(void)
+void 
+PGRstop_pgreplicate(int sig)
 {
-	char * func = "stop_pgreplicate()";
+	char * func = "PGRstop_pgreplicate()";
 	FILE *fd;
 	char fname[256];
 	char pidbuf[128];
 	pid_t pid;
 
+	if (sig == 0)
+	{
+		sig = SIGTERM;
+	}
+	signal(sig,SIG_IGN);
+
 	snprintf(fname, sizeof(fname), "%s/%s", PGR_Write_Path, PGREPLICATE_PID_FILE);
 	fd = fopen(fname, "r");
 	if (!fd)
@@ -641,11 +554,13 @@ stop_pgreplicate(void)
 	fclose(fd);
 	pid = atoi(pidbuf);
 
-	if (kill (pid,SIGTERM) == -1)
+	if (kill (pid,sig) == -1)
 	{
 		show_error("%s:could not stop pid: %d, reason: %s",func,pid,strerror(errno));
 		exit(1);
 	}
+	if (sig != SIGTERM)
+		exit(0);
 }
 
 /*--------------------------------------------------------------------
@@ -690,7 +605,7 @@ is_exist_pid_file(void)
  *    none
  *--------------------------------------------------------------------
  */
-void
+static void
 child_wait(SIGNAL_ARGS)
 {
 	pid_t pid = 0;
@@ -722,13 +637,14 @@ usage(void)
 		path = ".";
 	fprintf(stderr,"PGReplicate version [%s]\n",PGREPLICATE_VERSION);
 	fprintf(stderr,"A replication server for cluster DB servers (based on PostgreSQL)\n\n");
-	fprintf(stderr,"usage: pgreplicate [-D path_of_config_file] [-W path_of_work_files] [-U login user][-l][-n][-v][-h][stop]\n");
+	fprintf(stderr,"usage: pgreplicate [-D path_of_config_file] [-W path_of_work_files] [-U login user][-l][-n][-v][-h][stop][restart]\n");
 	fprintf(stderr,"    config file default path: %s/%s\n",path, PGREPLICATE_CONF_FILE);
 	fprintf(stderr,"    -l: print error logs in the log file.\n");
 	fprintf(stderr,"    -n: don't run in daemon mode.\n");
 	fprintf(stderr,"    -v: debug mode. need '-n' flag\n");
 	fprintf(stderr,"    -h: print this help\n");
 	fprintf(stderr,"    stop: stop pgreplicate\n");
+	fprintf(stderr,"    restart: restart pgreplicate\n");
 }
 
 /*--------------------------------------------------------------------
@@ -753,12 +669,18 @@ main(int argc, char * argv[])
 	char * r_path = NULL;
 	char * w_path = NULL;
 	bool detach = true;
-	pid_t rlog_pid;
+	pid_t syn_pid = 0;
+	pid_t ack_pid = 0;
+	pid_t rlog_pid = 0;
+	int sig_opt = SIGTERM;
+
+	/* init global variables */
+	PGR_Init_Com_Info(&Com_Info);
 
 	r_path = getenv("PGDATA");
 	if (r_path == NULL)
 		r_path = ".";
-	while ((opt = getopt(argc, argv, "U:D:W:w:lvnh")) != -1)
+	while ((opt = getopt(argc, argv, "U:D:W:m:lvnh")) != -1)
 	{
 		switch (opt)
 		{
@@ -786,14 +708,22 @@ main(int argc, char * argv[])
 				}
 				w_path = optarg;
 				break;
-			case 'w':
-				fork_wait_time = atoi(optarg);
+			case 'm':
+				if (!optarg)
+				{
+					usage();
+					exit(1);
+				}
+				if (!strcmp(optarg,"i"))
+				{
+					sig_opt = SIGINT;
+				}
 				break;
 			case 'l':
-				Log_Print = 1;
+				Com_Info.Log_Info.Log_Print = 1;
 				break;
 			case 'v':
-				Debug_Print = 1;
+				Com_Info.Log_Info.Debug_Print = 1;
 				break;
 			case 'n':
 				detach = false;
@@ -819,9 +749,13 @@ main(int argc, char * argv[])
 
 	if (optind == (argc-1) && !strcmp(argv[optind],"stop"))
 	{
-		stop_pgreplicate();
+		PGRstop_pgreplicate(sig_opt);
 		exit(0);
 	}
+	if (optind == (argc-1) && !strcmp(argv[optind],"restart"))
+	{
+		PGRstop_pgreplicate(SIGTERM);
+	}
 	else if (optind == argc)
 	{
 		if (is_exist_pid_file())
@@ -841,13 +775,6 @@ main(int argc, char * argv[])
 		daemonize();
 	}
 	write_pid_file();
-	pgreplicate_pid = getpid();
-
-	PGRsignal(SIGINT, set_exit_processing);
-	PGRsignal(SIGQUIT, set_exit_processing);
-	PGRsignal(SIGTERM, set_exit_processing);
-	PGRsignal(SIGCHLD, child_wait);
-	PGRsignal(SIGPIPE, SIG_IGN);
 
 	if (PGRset_Conf_Data(PGR_Data_Path) != STATUS_OK)
 	{
@@ -859,8 +786,25 @@ main(int argc, char * argv[])
 		show_error("%s:PGRinit_recovery error",func);
 		PGRreplicate_exit(0);
 	}
+	if (PGRsyn_init() != STATUS_OK)
+	{
+		show_error("%s:syn_init is failed",func);
+		PGRreplicate_exit(0);
+	}
 
-	pgid = getpgid(0);
+	/* life check process start */
+	syn_pid = PGR_SYN_Main(SYN_TO_CLUSTER_DB, (void *)Host_Tbl_Begin);
+	if (syn_pid < 0)
+	{
+		show_error("%s:PGR_SYN_Main failed",func);
+		PGRreplicate_exit(0);
+	}
+	ack_pid = PGR_ACK_Main(LifeCheck_Port_Number);
+	if (ack_pid < 0)
+	{
+		show_error("%s:PGR_ACK_Main failed",func);
+		PGRreplicate_exit(0);
+	}
 	if ( PGR_Use_Replication_Log == true )
 	{
 #ifdef PRINT_DEBUG
@@ -873,7 +817,7 @@ main(int argc, char * argv[])
 			PGRreplicate_exit(0);
 		}
 	}
-
+	pgid = getpgid(0);
 	pid = fork();
 	if (pid == (pid_t) -1)
 	{
@@ -882,45 +826,36 @@ main(int argc, char * argv[])
 	}
 	if (pid == 0)
 	{
-		PGRsignal(SIGCHLD, SIG_DFL);
-		PGRsignal(SIGHUP, quick_exit);	
-		PGRsignal(SIGINT, quick_exit);	
-		PGRsignal(SIGQUIT, quick_exit);	
-		PGRsignal(SIGTERM, quick_exit);	
-		PGRsignal(SIGPIPE, SIG_IGN);	
+		signal(SIGHUP, SIG_IGN);	
+		signal(SIGINT, quick_exit);	
+		signal(SIGQUIT, quick_exit);	
+		signal(SIGTERM, quick_exit);	
+		signal(SIGPIPE, SIG_IGN);
 		/*
 		 * in child process,
 		 * call recovery module
 		 */
 		setpgid(0,pgid);
-
-		if (fork_wait_time > 0) {
-#ifdef PRINT_DEBUG
-			show_debug("recovery process: wait fork(): pid = %d", getpid());
-#endif			
-			sleep(fork_wait_time);
-		}
-
 		PGRrecovery_main();
 	}
 	else
 	{
+		signal(SIGSEGV, PGRreplicate_exit);
+		signal(SIGINT, PGRreplicate_exit);
+		signal(SIGQUIT, PGRreplicate_exit);
+		signal(SIGTERM, PGRreplicate_exit);
+		signal(SIGCHLD, child_wait);
+		signal(SIGPIPE, SIG_IGN);
 		/*
 		 * call replicate module
 		 */
-		Replicateion_Log->r_log_sock =-1;
-
-		if (fork_wait_time > 0) {
-#ifdef PRINT_DEBUG
-			show_debug("replicate process: wait fork(): pid = %d", getpid());
-#endif			
-			sleep(fork_wait_time);
-		}
-
+		Replicateion_Log->r_log_sock = -1;
 		replicate_main();
 	}
 
 	PGRreplicate_exit(0);
 	return STATUS_OK;
 }
-
+#else
+void main() { }
+#endif /* USE_REPLICATION */
