@@ -307,10 +307,13 @@ PGRsend_load_balance_packet(RecoveryPacket * packet)
 	while (lbp->hostName[0] != 0)
 	{
 		status = send_packet(lbp,packet);
-		if (lbp->recovery_sock != -1)
+		if (status == STATUS_OK)
 		{
-			close(lbp->recovery_sock);
-			lbp->recovery_sock = -1;
+			if (lbp->sock != -1)
+			{
+				close(lbp->sock);
+				lbp->sock = -1;
+			}
 		}
 		lbp ++;
 	}
@@ -631,7 +634,7 @@ PGRsend_queue(RecoveryTbl * master, RecoveryTbl * target)
 	RecoveryQueue.queue_fp = PGRget_recovery_queue_file_for_read(0);
 	if (RecoveryQueue.queue_fp == NULL)
 	{
-		return STATUS_OK;
+		return STATUS_ERROR;
 	}
 	clearerr(RecoveryQueue.queue_fp);
 	fseek(RecoveryQueue.queue_fp,0,SEEK_SET);
@@ -688,8 +691,8 @@ PGRsend_queue(RecoveryTbl * master, RecoveryTbl * target)
 				if (fread(query,query_size,1, RecoveryQueue.queue_fp) >0)
 				{
 					read_lock = 0;
-					PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
 					status = send_recovery_queue(master_ptr,target_ptr, &header, query);
+					PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
 				}
 			}
 		}
@@ -708,11 +711,6 @@ PGRsend_queue(RecoveryTbl * master, RecoveryTbl * target)
 	{
 		free(query);
 	}
-	if (read_lock)
-	{
-		read_lock = 0;
-		PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
-	}
 	return status;
 }
 
@@ -835,6 +833,7 @@ pgrecovery_loop(int fd)
 				show_debug("%s:last target %s - %d",
 					func,Target.hostName,Target.port);
 #endif			
+				sleep(5);
 				status = PGRsend_queue(&Master,&Target);
 				if (status == STATUS_OK)
 				{
@@ -888,15 +887,12 @@ pgrecovery_loop(int fd)
 			if (Master.sock != -1)
 			{
 				close (Master.sock);
-				Master.sock = -1;
 			}
 			if (Master.recovery_sock != -1)
 			{
 				close (Master.recovery_sock);
-				Master.recovery_sock = -1;
 			}
-			close(sock);
-			sock = -1;
+			PGR_Close_Sock(&sock);
 			return;
 		}
 	}
@@ -1080,7 +1076,6 @@ clear_recovery_queue_file(void)
 	read_queue_no = Recovery_Status_Inf->read_queue_no;
 	if (write_queue_no != read_queue_no)
 	{
-		PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
 		return;
 	}
 	if (RecoveryQueue.queue_fp != NULL)
