@@ -1259,15 +1259,12 @@ PGRsend_replicate_packet_to_server( HostTbl * host_ptr, ReplicateHeader * header
 			return STATUS_OK;
 		}
 	}
-	else
-	{
-		send_sync_data(conn,header,sync_command_flg,current_cluster);
-	}
 	if ((header->cmdType == CMD_TYPE_COPY_DATA) ||
 		(header->cmdType == CMD_TYPE_COPY_DATA_END))
 	{
 		/* copy data replication */
-		rtn =PQputnbytes(conn, query,query_size);
+		/*rtn =PQputnbytes(conn, query,query_size);*/
+		rtn =PQputline(conn, query);
 		if (header->cmdType == CMD_TYPE_COPY_DATA_END)
 		{
 			rtn = PQendcopy(conn);
@@ -1288,6 +1285,7 @@ PGRsend_replicate_packet_to_server( HostTbl * host_ptr, ReplicateHeader * header
 	}
 	else
 	{
+		send_sync_data(conn,header,sync_command_flg,current_cluster);
 		status = replicate_query(res, conn, host_ptr,header, query, sync_command_flg, current_cluster);
 		if ((status == STATUS_OK) && (res == NULL))
 		{
@@ -1785,6 +1783,7 @@ PGRset_queue(ReplicateHeader * header,char * query)
 	RecoveryQueue.queue_fp = PGRget_recovery_queue_file_for_write();
 	if (RecoveryQueue.queue_fp == (FILE *)NULL)
 	{
+		PGRsem_unlock(RecoverySemID, SEM_NUM_OF_RECOVERY_QUEUE);
 		return STATUS_ERROR;
 	}
 	fseek(RecoveryQueue.queue_fp,0,SEEK_END);
