@@ -6,7 +6,7 @@
  *     This file is composed of the functions to call with the source
  *     at pgreplicate for replicate ahead log.
  *
- * Portions Copyright (c) 2003-2006, Atsushi Mitani
+ * Portions Copyright (c) 2004, Atsushi Mitani
  *--------------------------------------------------------------------
  */
 #ifdef USE_REPLICATION
@@ -53,11 +53,27 @@
 #include "replicate_com.h"
 #include "pgreplicate.h"
 
-static int RLog_Recv_Sock = -1;
+int RLog_Recv_Sock = -1;
 /*--------------------------------------
  * PROTOTYPE DECLARATION
  *--------------------------------------
  */
+int PGRwrite_rlog(ReplicateHeader * header, char * query);
+ReplicateHeader * PGRget_requested_query(ReplicateHeader * header);
+void PGRreconfirm_commit(int sock, ReplicateHeader * header);
+void PGRset_rlog(ReplicateHeader * header, char * query);
+void PGRunset_rlog(ReplicateHeader * header, char * query);
+int PGRresend_rlog_to_db(void);
+void PGRreconfirm_query(int sock, ReplicateHeader * header);
+pid_t  PGR_RLog_Main(void);
+int PGRcreate_send_rlog_socket(void);
+int PGRget_sync_data(ReplicateHeader *header);
+int PGRdelete_sync_data(ReplicateHeader *header);
+int PGRsend_rlog_packet(int sock,ReplicateHeader * header, const char * query_string);
+int PGRrecv_rlog_result(int sock,void * result, int size);
+int PGRsend_rlog_to_local(ReplicateHeader * header,char * query);
+int PGRget_rlog_header(ReplicateHeader * header);
+
 static int set_query_log(ReplicateHeader * header, char * query);
 static QueryLogType * get_query_log_by_header(ReplicateHeader * header);
 static QueryLogType * get_query_log(ReplicateHeader * header);
@@ -72,8 +88,6 @@ static int recv_message(int sock,char * buf, int len);
 static int send_message(int sock, char * msg, int len);
 static void exit_rlog(int sig);
 static int reconfirm_commit(ReplicateHeader * header);
-static int PGRget_sync_data(ReplicateHeader *header);
-static int PGRdelete_sync_data(ReplicateHeader *header);
 
 
 int
@@ -118,9 +132,6 @@ PGRwrite_rlog(ReplicateHeader * header, char * query)
 				delete_commit_log(header);
 			}
 			break;
-	default:
-	  show_error("%s:unknown status %c",func,header->cmdSts);
-	  break;
 	}
 	return STATUS_OK;
 }
@@ -191,7 +202,6 @@ set_query_log(ReplicateHeader * header, char * query)
 		return STATUS_ERROR;
 	}
 	size = ntohl(header->query_size);
-
 	query_log->query = (char *)malloc(size+4);
 	if (query_log->query == (char *)NULL)
 	{
@@ -216,20 +226,8 @@ get_query_log_by_header(ReplicateHeader * header)
 		return (QueryLogType *)NULL;
 	}
 	query_log = Query_Log_Top;
-	    show_debug("get_query_log_by_header:header is %d,%d,%d,%s",
-		       header->request_id,
-		       header->pid,
-		       header->port,
-		       header->from_host);
-
 	while(query_log != (QueryLogType *)NULL)
-	  {
-	    show_debug("get_query_log_by_header:comparing to %d,%d,%d,%s",
-		       query_log->header->request_id,
-		        query_log->header->pid,
-		        query_log->header->port,
-		        query_log->header->from_host);
-
+	{
 		if ((query_log->header->request_id == header->request_id) &&
 			(query_log->header->pid == header->pid) &&
 			(query_log->header->port == header->port) &&
@@ -254,7 +252,6 @@ get_query_log(ReplicateHeader * header)
 	query_log = Query_Log_Top;
 	while(query_log != (QueryLogType *)NULL)
 	{
-	  show_debug("get_qurey_log: comparing in log is %d,header is %d",query_log->header->replicate_id,header->replicate_id);
 		if (query_log->header->replicate_id == header->replicate_id)
 		{
 			return query_log;
@@ -322,11 +319,11 @@ set_commit_log(ReplicateHeader * header)
 		return STATUS_ERROR;
 	}
 	commit_log = Commit_Log_Tbl + 1;
-	while (	commit_log->inf.useFlag != DB_TBL_END )
+	while (	commit_log->inf.useFlag != DATA_END )
 	{
-		if (commit_log->inf.useFlag != DB_TBL_USE)
+		if (commit_log->inf.useFlag != DATA_USE)
 		{
-			commit_log->inf.useFlag = DB_TBL_USE;
+			commit_log->inf.useFlag = DATA_USE;
 			c_header = &(commit_log->header);
 			memcpy(c_header,header,sizeof(ReplicateHeader));
 			Commit_Log_Tbl->inf.commit_log_num ++;
@@ -349,9 +346,9 @@ get_commit_log(ReplicateHeader * header)
 		return (CommitLogInf *)NULL;
 	}
 	commit_log = Commit_Log_Tbl + 1;
-	while (	commit_log->inf.useFlag != DB_TBL_END )
+	while (	commit_log->inf.useFlag != DATA_END )
 	{
-		if (commit_log->inf.useFlag == DB_TBL_USE)
+		if (commit_log->inf.useFlag == DATA_USE)
 		{
 			cnt ++;
 			c_header = &(commit_log->header);
@@ -386,7 +383,7 @@ delete_commit_log(ReplicateHeader * header)
 	if (commit_log != NULL)
 	{
 		memset(&(commit_log->header),0,sizeof(commit_log->header));
-		commit_log->inf.useFlag = DB_TBL_INIT;
+		commit_log->inf.useFlag = DATA_INIT;
 		Commit_Log_Tbl->inf.commit_log_num --;
 	}
 }
@@ -413,27 +410,23 @@ PGRreconfirm_commit(int sock, ReplicateHeader * header)
 	{
 		return ;
 	}
-	
 	if (Replicateion_Log->r_log_sock > 0)
 	{
 		close(Replicateion_Log->r_log_sock );
 		Replicateion_Log->r_log_sock = -1;
 	}
 	Replicateion_Log->r_log_sock = PGRcreate_send_rlog_socket();
-	if (Replicateion_Log->r_log_sock == -1)
-		return;
-	
+
 	header->query_size = 0;
-	PGRsend_rlog_packet(Replicateion_Log->r_log_sock,header,"");
+	PGRsend_rlog_packet(Replicateion_Log->r_log_sock,header,NULL);
 	PGRrecv_rlog_result(Replicateion_Log->r_log_sock,&result, sizeof(result));
 
-	
 	close(Replicateion_Log->r_log_sock );
 	Replicateion_Log->r_log_sock = -1;
-	
+		
 	snprintf(PGR_Result,PGR_MESSAGE_BUFSIZE,"%d,%d", PGR_TRANSACTION_CONFIRM_ANSWER_FUNC_NO,result);
 
-	PGRreturn_result(sock, PGR_Result,PGR_NOWAIT_ANSWER);
+	PGRreturn_result(sock, PGR_NOWAIT_ANSWER);
 }
 
 static int 
@@ -462,61 +455,58 @@ reconfirm_commit(ReplicateHeader * header)
 void
 PGRset_rlog(ReplicateHeader * header, char * query)
 {
-		char * func = "PGRset_rlog()";
-		int status = STATUS_OK;
-		bool send_flag = false;
+	char * func = "PGRset_rlog()";
+	int status = STATUS_OK;
+	bool send_flag = false;
 
-		if (PGR_Log_Header == NULL)
+ 	if (PGR_Log_Header == NULL)
+	{
+		return;
+	}
+	switch (header->cmdSts)
+	{
+		case CMD_STS_QUERY:
+			send_flag = true;
+			break;
+		case CMD_STS_TRANSACTION:
+			if (header->cmdType == CMD_TYPE_COMMIT)
+			{
+				send_flag = true;
+				PGR_Log_Header->cmdType = header->cmdType;
+				PGR_Log_Header->query_size = htonl(strlen(query));
+			}
+			break;
+	}
+	if (send_flag != true)
+	{
+		show_error("%s:send_flag is false",func);
+		return;
+	}
+	PGR_Log_Header->cmdSys = CMD_SYS_LOG;
+	if (Cascade_Inf->useFlag == DATA_USE)
+	{
+		/* save log data in remote server */
+
+		status = PGRsend_lower_cascade(PGR_Log_Header, query);	
+		if (status != STATUS_OK)
 		{
-				return;
-		}
-		switch (header->cmdSts)
-		{
-				case CMD_STS_QUERY:
-						send_flag = true;
-						break;
-				case CMD_STS_TRANSACTION:
-						if (header->cmdType == CMD_TYPE_COMMIT)
-						{
-								send_flag = true;
-								PGR_Log_Header->cmdType = header->cmdType;
-								PGR_Log_Header->query_size = htonl(strlen(query));
-						}
-						break;
-		}
-		if (send_flag != true)
-		{
-				show_error("%s:send_flag is false",func);
-				return;
-		}
-		PGR_Log_Header->cmdSys = CMD_SYS_LOG;
-		if (Cascade_Inf->useFlag == DB_TBL_USE)
-		{
-				/* save log data in remote server */
-				show_debug("%s:set rlog %s",func,query);
-				status = PGRsend_lower_cascade(PGR_Log_Header, query);
-		        if (status == STATUS_OK) {
-						status=PGRwait_notice_rlog_done();
-				}
-				if (status != STATUS_OK)
-				{
 #ifdef PRINT_DEBUG
-						show_debug("%s:PGRsend_lower_cascade failed",func);
+			show_debug("%s:PGRsend_lower_cascade failed",func);
 #endif			
-						PGRwrite_rlog(PGR_Log_Header, query);
-				}
-		}
-		else
-		{
-				/* save log data in local server */
-				PGRwrite_rlog(PGR_Log_Header, query);
+			PGRwrite_rlog(PGR_Log_Header, query);
 		}
+	}
+	else
+	{
+		/* save log data in local server */
+		PGRwrite_rlog(PGR_Log_Header, query);
+	}
 }
 
 void
 PGRunset_rlog(ReplicateHeader * header, char * query)
 {
-        int status = STATUS_OK;
+	int status = STATUS_OK;
 	bool send_flag = false;
 
  	if (PGR_Log_Header == NULL)
@@ -531,7 +521,7 @@ PGRunset_rlog(ReplicateHeader * header, char * query)
 			break;
 		case CMD_STS_TRANSACTION:
 			if (PGR_Log_Header->cmdType == CMD_TYPE_COMMIT)
-		{
+			{
 				PGR_Log_Header->cmdSts = CMD_STS_DELETE_TRANSACTION;
 				PGR_Log_Header->query_size = htonl(strlen(query));
 				send_flag = true;
@@ -543,21 +533,13 @@ PGRunset_rlog(ReplicateHeader * header, char * query)
 		return;
 	}
 	PGR_Log_Header->cmdSys = CMD_SYS_LOG;
-	if (Cascade_Inf->useFlag == DB_TBL_USE)
+	if (Cascade_Inf->useFlag == DATA_USE)
 	{
 		/* save log data in remote server */
-	  show_debug("unset rlog %s",query);
 
 		status = PGRsend_lower_cascade(PGR_Log_Header, query);	
-		if (status == STATUS_OK)
-		{
-				status=PGRwait_notice_rlog_done();
-		}
 		if (status != STATUS_OK)
 		{
-#ifdef PRINT_DEBUG
-			show_debug("PGRsend_lower_cascade recv failed");
-#endif			
 			PGRwrite_rlog(PGR_Log_Header, query);
 		}
 	}
@@ -571,49 +553,47 @@ PGRunset_rlog(ReplicateHeader * header, char * query)
 int
 PGRresend_rlog_to_db(void)
 {
-  char *func="PGRresend_rlog_to_db";
 	QueryLogType * query_log = NULL;
 	QueryLogType * next = NULL;
 	int status = STATUS_OK;
 	int dest = 0;
 
-	  show_debug("%s:enter.",func);
-
 	query_log = Query_Log_Top;
 
 	while (query_log != NULL)
 	{
-
-
-	  show_debug("%s:processing qlog,query=%s",func,query_log->query);
 		if (query_log->header->rlog != FROM_R_LOG_TYPE )
 		{
 			query_log = (QueryLogType *)query_log->next;
 			continue;
 		}
-	        status = replicate_packet_send_internal(query_log->header,query_log->query, dest,RECOVERY_INIT,false);
-		show_debug("%s:status=%d",func,status);
-		
+		status = PGRreplicate_packet_send(query_log->header,query_log->query, dest);
 		if (status == STATUS_SKIP_REPLICATE )
 		{
 			Query_Log_Top = query_log;
 			query_log = (QueryLogType *)query_log->next;
-       		}
-		else		 
-		{
-		        if (query_log->header != NULL)
-			  {
-			    free(query_log->header );
-			  }
-			if (query_log->query != NULL)
-			  {
-			    free(query_log->query );
-			  }
-			next = (QueryLogType *)query_log->next;
-			free(query_log);
-			query_log = next;
-			Query_Log_Top = query_log;
+			if (query_log != NULL)
+			{
+				Query_Log_End = (QueryLogType *)query_log->next;
+			}
+			else
+			{
+				Query_Log_End = (QueryLogType *)NULL;
+			}
+			continue;
 		}
+		if (query_log->header != NULL)
+		{
+			free(query_log->header );
+		}
+		if (query_log->query != NULL)
+		{
+			free(query_log->query );
+		}
+		next = (QueryLogType *)query_log->next;
+		free(query_log);
+		query_log = next;
+		Query_Log_Top = query_log;
 		if (query_log != NULL)
 		{
 			Query_Log_End = (QueryLogType *)query_log->next;
@@ -623,9 +603,6 @@ PGRresend_rlog_to_db(void)
 			Query_Log_End = (QueryLogType *)NULL;
 		}
 	}
-
-	  show_debug("%s:exit.",func);
-
 	return STATUS_OK;
 }
 	
@@ -640,8 +617,6 @@ PGR_RLog_Main(void)
 	pid_t pid = 0;
 	pid_t pgid = 0;
 
-	extern int fork_wait_time;
-
 	if (Replicateion_Log == NULL)
 	{
 		show_error("%s:Replicateion_Log is NULL",func);
@@ -652,33 +627,18 @@ PGR_RLog_Main(void)
 	{
 		return pid;
 	}
-	PGRsignal(SIGTERM,exit_rlog);
-	PGRsignal(SIGINT,exit_rlog);
-	PGRsignal(SIGQUIT,exit_rlog);
-	PGRsignal(SIGPIPE,SIG_IGN);
-
-	if (PGRinit_transaction_table() != STATUS_OK)
-	{
-		show_error("RLog process transaction table memory allocate failed");
-		return -1;
-	}
+	signal(SIGTERM,exit_rlog);
+	signal(SIGINT,exit_rlog);
+	signal(SIGQUIT,exit_rlog);
+	signal(SIGPIPE,SIG_IGN);
 
 	setpgid(0,pgid);	
 	RLog_Recv_Sock = create_recv_rlog_socket();
-	if(RLog_Recv_Sock == -1) 
+
+	if (RLog_Recv_Sock  < 0)
 	{
-		show_error("rlog socket creation failure.quit all process.");
-		kill(pgreplicate_pid, SIGINT);
-		exit_rlog(0);
+		exit(0);
 	}
-
-	if (fork_wait_time > 0) {
-#ifdef PRINT_DEBUG
-		show_debug("rlog process: wait fork(): pid = %d", getpid());
-#endif			
-		sleep(fork_wait_time);
-	}
-
 	for (;;)
 	{
 		fd_set	  rmask;
@@ -698,12 +658,7 @@ PGR_RLog_Main(void)
 		FD_ZERO(&rmask);
 		FD_SET(RLog_Recv_Sock,&rmask);
 		rtn = select(RLog_Recv_Sock+1, &rmask, (fd_set *)NULL, (fd_set *)NULL, &timeout);
-		if (rtn < 0)
-		{
-			if (errno == EINTR || errno == EAGAIN)
-				continue;
-		}
-		else if (rtn && FD_ISSET(RLog_Recv_Sock, &rmask))
+		if (rtn && FD_ISSET(RLog_Recv_Sock, &rmask))
 		{
 			Idle_Flag = BUSY_MODE ;
 			addrlen = sizeof(addr);
@@ -715,7 +670,7 @@ PGR_RLog_Main(void)
 			else
 			{
 				do_rlog(afd);
-		       		close(afd);
+				close(afd);
 			}
 		}
 	}
@@ -763,7 +718,7 @@ fprintf(stderr,"Replicateion_Log->RLog_Sock_Path[%s]\n",Replicateion_Log->RLog_S
 		return -1;
 	}
 
-	status = listen(fd, 1000000);
+	status = listen(fd, PGR_MAX_SOCKET_QUEUE);
 	if (status < 0)
 	{
 		show_error("%s: listen() failed. reason: %s", func, strerror(errno));
@@ -816,7 +771,6 @@ do_rlog(int fd)
 
 	memset(&header,0,sizeof(header));
 	query = PGRread_packet(fd, &header);
-	show_debug("%s:got result:cmdSys='%c'",func,header.cmdSys);
 	if (header.cmdSys == 0)
 	{
 		return STATUS_ERROR;
@@ -857,20 +811,16 @@ do_rlog(int fd)
 			{
 #ifdef PRINT_DEBUG
 				show_debug("%s: CMD_STS_QUERY_SUSPEND",func);
-#endif			       
-				//			status = PGRresend_rlog_to_db();
-			}
-#ifdef PRINT_DEBUG
-				show_debug("%s: SYS_CALL process done",func);
 #endif
+				status = PGRresend_rlog_to_db();
+			}
 			send_message(fd,(char *)&status,sizeof(status));
 			break;
 	}
-	show_debug("%s:process result done:cmdSys='%c'",func,header.cmdSys);
 	return STATUS_OK;
 }
 
-static int
+int
 PGRget_sync_data(ReplicateHeader *header)
 {
 	ReplicateHeader rlog_header;
@@ -880,35 +830,28 @@ PGRget_sync_data(ReplicateHeader *header)
 		return STATUS_ERROR;
 	}
 
-
 	if (Replicateion_Log->r_log_sock > 0)
 	{
 		close(Replicateion_Log->r_log_sock );
 		Replicateion_Log->r_log_sock = -1;
 	}
 	Replicateion_Log->r_log_sock = PGRcreate_send_rlog_socket();
-	if (Replicateion_Log->r_log_sock == -1)
-		return STATUS_ERROR;
 
 	memset(&rlog_header,0,sizeof(ReplicateHeader));
 	send_message( Replicateion_Log->r_log_sock, (char *)header,sizeof(ReplicateHeader));
 	recv_message( Replicateion_Log->r_log_sock, (char *)&rlog_header,sizeof(ReplicateHeader));
 	if (rlog_header.cmdSts != 0)
 	{
-	  close(Replicateion_Log->r_log_sock );
-	  Replicateion_Log->r_log_sock = -1;
 		memcpy(header,&rlog_header, sizeof(ReplicateHeader));
 		return STATUS_OK;
 	}
-
 	close(Replicateion_Log->r_log_sock );
 	Replicateion_Log->r_log_sock = -1;
-
 	return STATUS_ERROR;
 
 }
 
-static int
+int
 PGRdelete_sync_data(ReplicateHeader *header)
 {
 	int status;
@@ -920,23 +863,17 @@ PGRdelete_sync_data(ReplicateHeader *header)
 	{
 		return STATUS_ERROR;
 	}
-	
 	if (Replicateion_Log->r_log_sock > 0)
 	{
 		close(Replicateion_Log->r_log_sock );
 		Replicateion_Log->r_log_sock = -1;
 	}
 	Replicateion_Log->r_log_sock = PGRcreate_send_rlog_socket();
-	if (Replicateion_Log->r_log_sock == -1)
-		return STATUS_ERROR;
-	
 	send_message( Replicateion_Log->r_log_sock, (char *)header,sizeof(ReplicateHeader));
 	recv_message( Replicateion_Log->r_log_sock, (char *)&status,sizeof(status));
 	header->cmdSts = cmdSts;
-	
 	close(Replicateion_Log->r_log_sock );
 	Replicateion_Log->r_log_sock = -1;
-	
 	return status;
 
 }
@@ -981,7 +918,6 @@ PGRsend_rlog_packet(int sock,ReplicateHeader * header, const char * query_string
 int
 PGRrecv_rlog_result(int sock,void * result, int size)
 {
-	char *func = "PGRrecv_rlog_result";
 	fd_set      rmask;
 	struct timeval timeout;
 	int rtn;
@@ -991,30 +927,29 @@ PGRrecv_rlog_result(int sock,void * result, int size)
 		return -1;
 	}
 
+	timeout.tv_sec = PGR_RECV_TIMEOUT;
+	timeout.tv_usec = 0;
+
 	/*
 	 * Wait for something to happen.
 	 */
 	rtn = 1;
-	for (;;)
+	while (rtn)
 	{
-		timeout.tv_sec = PGR_RECV_TIMEOUT;
-		timeout.tv_usec = 0;
-
 		FD_ZERO(&rmask);
 		FD_SET(sock,&rmask);
 		rtn = select(sock+1, &rmask, (fd_set *)NULL, (fd_set *)NULL, &timeout);
-		if (rtn < 0)
-		{
-			if (errno != EINTR || errno != EAGAIN)
-			{
-				show_error("%s: select() failed (%s)",func,strerror(errno));
-				return -1;
-			}
-		}
-		else if (rtn && FD_ISSET(sock, &rmask))
+		if (rtn && FD_ISSET(sock, &rmask))
 		{
 			return (recv_message(sock, (char*)result, size));
 		}
+		else
+		{
+			if (errno != EINTR)
+			{
+				return -1;
+			}
+		}
 	}
 	return -1;
 }
@@ -1023,7 +958,6 @@ PGRrecv_rlog_result(int sock,void * result, int size)
 static int
 recv_message(int sock,char * buf, int len)
 {
-	char *func = "recv_message";
 	int cnt = 0;
 	int r = 0;
 	char * read_ptr;
@@ -1036,27 +970,48 @@ recv_message(int sock,char * buf, int len)
 		r = recv(sock,read_ptr + read_size ,len - read_size, 0); 
 		if (r < 0)
 		{
-			if (errno == EINTR || errno == EAGAIN)
+			if (errno == EINTR)
+			{
+				usleep(PGR_RECV_WAIT_MSEC);
 				continue;
+			}
+			if (cnt < PGR_RECV_RETRY_CNT )
+			{
+				cnt ++;
+				usleep(PGR_RECV_WAIT_MSEC);
+				continue;
+			}
 			else
 			{
-				show_error("%s:recv failed: %d(%s)",func, errno, strerror(errno));
-				return -1;
+				if (read_size == 0)
+				{
+					return -1;
+				}
 			}
 		}
-		else if (r > 0)
+		if (r > 0)
 		{
 			read_size += r;
 			if (read_size == len)
+			{
 				return read_size;
+			}
 		}
-		else /* r == 0 */
+		if ((r == 0) && (read_size == 0))
+		{
+			return -1;
+		}
+		if (cnt < PGR_RECV_RETRY_CNT )
+		{
+			cnt ++;
+			usleep(PGR_RECV_WAIT_MSEC);
+			continue;
+		}
+		else
 		{
-			show_error("%s:unexpected EOF", func);
 			return -1;
 		}
 	}
-	return -1;
 }
 
 int
@@ -1068,29 +1023,22 @@ PGRsend_rlog_to_local(ReplicateHeader * header,char * query)
 	{
 		return STATUS_ERROR;
 	}
-	
 	if (Replicateion_Log->r_log_sock > 0)
 	{
 		close(Replicateion_Log->r_log_sock );
 		Replicateion_Log->r_log_sock = -1;
 	}
-	
 	Replicateion_Log->r_log_sock = PGRcreate_send_rlog_socket();
-	if (Replicateion_Log->r_log_sock == -1)
-		return STATUS_ERROR;
-	
-	show_debug("send_to_local %s",query);
+
 	status = PGRsend_rlog_packet(Replicateion_Log->r_log_sock,header,query);
-	show_debug("send_to_local result is %d,errno=%d(%s)",status,errno ,strerror(errno));
-        
 	if (status != STATUS_ERROR)
 	{
 		PGRrecv_rlog_result(Replicateion_Log->r_log_sock,&status, sizeof(status));
 	}
-	
+
 	close(Replicateion_Log->r_log_sock );
 	Replicateion_Log->r_log_sock = -1;
-	
+		
 	return status;
 }
 
@@ -1105,20 +1053,17 @@ PGRget_rlog_header(ReplicateHeader * header)
 	{
 		return STATUS_ERROR;
 	}
-	
 	if (Replicateion_Log->r_log_sock > 0)
 	{
 		close(Replicateion_Log->r_log_sock );
 		Replicateion_Log->r_log_sock = -1;
 	}
 	Replicateion_Log->r_log_sock = PGRcreate_send_rlog_socket();
-	if (Replicateion_Log->r_log_sock == -1)
-		return STATUS_ERROR;
-	
+
 	memcpy(&rlog_header,header,sizeof(ReplicateHeader));
 	rlog_header.cmdSys = CMD_SYS_REPLICATE;
 	rlog_header.query_size = 0;
-	status =PGRsend_rlog_packet(Replicateion_Log->r_log_sock,&rlog_header,"");
+	status =PGRsend_rlog_packet(Replicateion_Log->r_log_sock,&rlog_header,NULL);
 	if (status != STATUS_ERROR)
 	{
 		status = PGRrecv_rlog_result(Replicateion_Log->r_log_sock,&rlog_header, sizeof(ReplicateHeader));
@@ -1132,7 +1077,7 @@ PGRget_rlog_header(ReplicateHeader * header)
 			status = STATUS_ERROR;
 		}
 	}
-	
+
 	close(Replicateion_Log->r_log_sock );
 	Replicateion_Log->r_log_sock = -1;
 		
@@ -1150,6 +1095,7 @@ send_message(int sock, char * msg, int len)
 	int send_size= 0;
 	int buf_size = 0;
 	int s = 0;
+	int cnt = 0;
 	int flag = 0;
 	
 	if ((msg == NULL) || (len <= 0) || (sock <= 0))
@@ -1159,6 +1105,9 @@ send_message(int sock, char * msg, int len)
 	send_ptr = msg;
 	buf_size = len;
 
+	timeout.tv_sec = PGR_SEND_TIMEOUT;
+	timeout.tv_usec = 0;
+
 	/*
 	 * Wait for something to happen.
 	 */
@@ -1168,45 +1117,55 @@ send_message(int sock, char * msg, int len)
 #ifdef MSG_NOSIGNAL
 	flag |= MSG_NOSIGNAL;
 #endif
-
-	for (;;)
+	FD_ZERO(&wmask);
+	FD_SET(sock,&wmask);
+	rtn = select(sock+1, (fd_set *)NULL, &wmask, (fd_set *)NULL, &timeout);
+	if (rtn && FD_ISSET(sock, &wmask))
 	{
-		timeout.tv_sec = PGR_SEND_TIMEOUT;
-		timeout.tv_usec = 0;
-
-		FD_ZERO(&wmask);
-		FD_SET(sock,&wmask);
-		rtn = select(sock+1, (fd_set *)NULL, &wmask, (fd_set *)NULL, &timeout);
-	  
-		if (rtn < 0 )
-		{
-			if (errno == EAGAIN || errno == EINTR)
-				continue;
-
-			show_error("%s:send-select error: %d(%s)",func,errno,strerror(errno));
-			return STATUS_ERROR;
-		}
-		else if (rtn & FD_ISSET(sock, &wmask))
+		for (;;)
 		{
 			s = send(sock,send_ptr + send_size,buf_size - send_size ,flag); 
-			if (s < 0)
-			{
-				if (errno == EINTR || errno == EAGAIN)
+			if (s < 0){
+				show_error("%s:send error: %d(%s)",func,errno,strerror(errno));
+				if (errno == EINTR)
+				{
+					usleep(PGR_SEND_WAIT_MSEC);
 					continue;
+				}
+#ifdef EPIPE
+				if (errno == EPIPE)
+				{
+					memset(send_ptr, 0, len);
+					return STATUS_ABORTED;
+				}
+#endif /* EPIPE */
+#ifdef ENSOCK
+				if (errno == ENSOCK)
+				{
+					memset(send_ptr, 0, len);
+					return STATUS_ABORTED;
+				}
+#endif /* ENSOCK */
+#ifdef EHOSTUNREACH
+				if (errno == EHOSTUNREACH)
+				{
+					memset(send_ptr, 0, len);
+					return STATUS_ABORTED;
+				}
+#endif /* EHOSTUNREACH */
+				if (cnt < PGR_SEND_RETRY_CNT )
+				{
+					cnt ++;
+					usleep(PGR_SEND_WAIT_MSEC);
+					continue;
+				}
 				else
 				{
-					show_error("%s:send error: %d(%s)",func,errno,strerror(errno));
 					memset(send_ptr, 0, len);
 					return STATUS_ERROR;
 				}
 			}
-			else if (s == 0)
-			{
-				show_error("%s:unexpected EOF");
-				memset(send_ptr, 0, len);
-				return STATUS_ERROR;
-			}
-			else /* s > 0 */
+			if (s > 0)
 			{
 				send_size += s;
 				if (send_size == buf_size)
@@ -1214,18 +1173,29 @@ send_message(int sock, char * msg, int len)
 					return STATUS_OK;
 				}
 			}
+			usleep(PGR_SEND_WAIT_MSEC);
+			if (flag != 0)
+			{
+				if (cnt < PGR_SEND_RETRY_CNT )
+				{
+					cnt ++;
+					usleep(PGR_SEND_WAIT_MSEC);
+					continue;
+				}
+				else
+				{
+					memset(send_ptr, 0, len);
+					return STATUS_ERROR;
+				}
+			}
 		}
 	}
-	show_error("%s:send-select unknown error: %d(%s)",
-			   func,errno,strerror(errno));
 	return STATUS_ERROR;
 }
 
 static void
 exit_rlog(int sig)
 {
-	sigset_t mask;
-
 	Exit_Request = true;
 	if (sig == SIGTERM)
 	{
@@ -1234,13 +1204,6 @@ exit_rlog(int sig)
 			return;
 		}
 	}
-	
-	sigemptyset(&mask);
-	sigaddset(&mask, SIGINT);
-	sigaddset(&mask, SIGTERM);
-	sigaddset(&mask, SIGQUIT);
-	sigprocmask(SIG_BLOCK, &mask, NULL);
-
 	if (RLog_Recv_Sock >= 0)
 	{
 		close(RLog_Recv_Sock);
